@@ -10,13 +10,15 @@ salt-ssh calls and return the data from them.
 No access control is needed because calls cannot originate from the minions.
 '''
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function
 import copy
 import logging
 
 # Import salt libs
 import salt.client.ssh
 import salt.runner
+import salt.utils.args
+import salt.utils.versions
 
 log = logging.getLogger(__name__)
 
@@ -24,7 +26,7 @@ log = logging.getLogger(__name__)
 def _publish(tgt,
              fun,
              arg=None,
-             expr_form='glob',
+             tgt_type='glob',
              returner='',
              timeout=None,
              form='clean',
@@ -72,13 +74,15 @@ def _publish(tgt,
         arg = []
 
     # Set up opts for the SSH object
-    opts = copy.deepcopy(__opts__)
+    opts = copy.deepcopy(__context__['master_opts'])
+    minopts = copy.deepcopy(__opts__)
+    opts.update(minopts)
     if roster:
         opts['roster'] = roster
     if timeout:
         opts['timeout'] = timeout
     opts['argv'] = [fun] + arg
-    opts['selected_target_option'] = expr_form
+    opts['selected_target_option'] = tgt_type
     opts['tgt'] = tgt
     opts['arg'] = arg
 
@@ -105,10 +109,11 @@ def _publish(tgt,
 def publish(tgt,
             fun,
             arg=None,
-            expr_form='glob',
+            tgt_type='glob',
             returner='',
             timeout=5,
-            roster=None):
+            roster=None,
+            expr_form=None):
     '''
     Publish a command "from the minion out to other minions". In reality, the
     minion does not execute this function, it is executed by the master. Thus,
@@ -122,12 +127,16 @@ def publish(tgt,
 
     Returners are not currently supported
 
-    The expr_form argument is used to pass a target other than a glob into
+    The tgt_type argument is used to pass a target other than a glob into
     the execution, the available options are:
 
     - glob
     - pcre
 
+    .. versionchanged:: 2017.7.0
+        The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
+        releases must use ``expr_form``.
+
     The arguments sent to the minion publish function are separated with
     commas. This means that for a minion executing a command with multiple
     args it will look like this:
@@ -163,10 +172,21 @@ def publish(tgt,
 
 
     '''
+    # remember to remove the expr_form argument from this function when
+    # performing the cleanup on this deprecation.
+    if expr_form is not None:
+        salt.utils.versions.warn_until(
+            'Fluorine',
+            'the target type should be passed using the \'tgt_type\' '
+            'argument instead of \'expr_form\'. Support for using '
+            '\'expr_form\' will be removed in Salt Fluorine.'
+        )
+        tgt_type = expr_form
+
     return _publish(tgt,
                     fun,
                     arg=arg,
-                    expr_form=expr_form,
+                    tgt_type=tgt_type,
                     returner=returner,
                     timeout=timeout,
                     form='clean',
@@ -176,10 +196,11 @@ def publish(tgt,
 def full_data(tgt,
               fun,
               arg=None,
-              expr_form='glob',
+              tgt_type='glob',
               returner='',
               timeout=5,
-              roster=None):
+              roster=None,
+              expr_form=None):
     '''
     Return the full data about the publication, this is invoked in the same
     way as the publish function
@@ -201,10 +222,21 @@ def full_data(tgt,
             salt-ssh '*' publish.full_data test.kwarg arg='cheese=spam'
 
     '''
+    # remember to remove the expr_form argument from this function when
+    # performing the cleanup on this deprecation.
+    if expr_form is not None:
+        salt.utils.versions.warn_until(
+            'Fluorine',
+            'the target type should be passed using the \'tgt_type\' '
+            'argument instead of \'expr_form\'. Support for using '
+            '\'expr_form\' will be removed in Salt Fluorine.'
+        )
+        tgt_type = expr_form
+
     return _publish(tgt,
                     fun,
                     arg=arg,
-                    expr_form=expr_form,
+                    tgt_type=tgt_type,
                     returner=returner,
                     timeout=timeout,
                     form='full',
