@@ -206,8 +206,8 @@ def get_rsa_pub_key(path):
     '''
     log.debug('salt.crypt.get_rsa_pub_key: Loading public key')
     if HAS_M2:
-        with salt.utils.files.fopen(path) as f:
-            data = f.read().replace(b'RSA ', '')
+        with salt.utils.files.fopen(path, 'rb') as f:
+            data = f.read().replace(b'RSA ', b'')
         bio = BIO.MemoryBuffer(data)
         key = RSA.load_pub_key_bio(bio)
     else:
@@ -608,9 +608,17 @@ class AsyncAuth(object):
                     error = SaltClientError('Detect mode is on')
                     break
                 if self.opts.get('caller'):
-                    print('Minion failed to authenticate with the master, '
-                          'has the minion key been accepted?')
-                    sys.exit(2)
+                    # We have a list of masters, so we should break
+                    # and try the next one in the list.
+                    if self.opts.get('local_masters', None):
+                        error = SaltClientError('Minion failed to authenticate'
+                                                ' with the master, has the '
+                                                'minion key been accepted?')
+                        break
+                    else:
+                        print('Minion failed to authenticate with the master, '
+                              'has the minion key been accepted?')
+                        sys.exit(2)
                 if acceptance_wait_time:
                     log.info(
                         'Waiting %s seconds before retry.', acceptance_wait_time
