@@ -1,27 +1,86 @@
 # -*- coding: utf-8 -*-
 '''
-An engine that reads messages from Slack and can act on them.
-
-It has two major uses.
-
-1. When the ``control`` parameter is set to ``True`` and a message is prefaced
-   with the ``trigger`` (which defaults to ``!``) then the engine will
-   validate that the user has permission, and if so will run the command
-
-2. In addition, when the parameter ``fire_all`` is set (defaults to False),
-   all other messages (the messages that aren't control messages) will be
-   fired off to the salt event bus with the tag prefixed by the string
-   provided by the ``tag`` config option (defaults to ``salt/engines/slack``).
-
-This allows for configuration to be gotten from either the engine config, or from
-the saltmaster's minion pillar.
+An engine that reads messages from Slack and can act on them
 
 .. versionadded: 2016.3.0
 
-:configuration: Example configuration using only a 'default' group. The default group is not special.
-In addition, other groups are being loaded from pillars.
+:depends: `slackclient <https://pypi.org/project/slackclient/>`_ Python module
 
-.. code-block:: yaml
+.. important::
+    This engine requires a bot user. To create a bot user, first go to the
+    **Custom Integrations** page in your Slack Workspace. Copy and paste the
+    following URL, and replace ``myworkspace`` with the proper value for your
+    workspace:
+
+    ``https://myworkspace.slack.com/apps/manage/custom-integrations``
+
+    Next, click on the ``Bots`` integration and request installation. Once
+    approved by an admin, you will be able to proceed with adding the bot user.
+    Once the bot user has been added, you can configure it by adding an avatar,
+    setting the display name, etc. You will also at this time have access to
+    your API token, which will be needed to configure this engine.
+
+    Finally, add this bot user to a channel by switching to the channel and
+    using ``/invite @mybotuser``. Keep in mind that this engine will process
+    messages from each channel in which the bot is a member, so it is
+    recommended to narrowly define the commands which can be executed, and the
+    Slack users which are allowed to run commands.
+
+
+This engine has two boolean configuration parameters that toggle specific
+features (both default to ``False``):
+
+1. ``control`` - If set to ``True``, then any message which starts with the
+   trigger string (which defaults to ``!`` and can be overridden by setting the
+   ``trigger`` option in the engine configuration) will be interpreted as a
+   Salt CLI command and the engine will attempt to run it. The permissions
+   defined in the various ``groups`` will determine if the Slack user is
+   allowed to run the command. The ``targets`` and ``default_target`` options
+   can be used to set targets for a given command, but the engine can also read
+   the following two keyword arguments:
+
+   - ``target`` - The target expression to use for the command
+
+   - ``tgt_type`` - The match type, can be one of ``glob``, ``list``,
+     ``pcre``, ``grain``, ``grain_pcre``, ``pillar``, ``nodegroup``, ``range``,
+     ``ipcidr``, or ``compound``. The default value is ``glob``.
+
+   Here are a few examples:
+
+   .. code-block:: text
+
+       !test.ping target=*
+       !state.apply foo target=os:CentOS tgt_type=grain
+       !pkg.version mypkg target=role:database tgt_type=pillar
+
+2. ``fire_all`` - If set to ``True``, all messages which are not prefixed with
+   the trigger string will fired as events onto Salt's ref:`event bus
+   <event-system>`. The tag for these veents will be prefixed with the string
+   specified by the ``tag`` config option (default: ``salt/engines/slack``).
+
+
+The ``groups_pillar_name`` config option can be used to pull group
+configuration from the specified pillar key.
+
+.. note::
+    In order to use ``groups_pillar_name``, the engine must be running as a
+    minion running on the master, so that the ``Caller`` client can be used to
+    retrieve that minions pillar data, because the master process does not have
+    pillar data.
+
+
+Configuration Examples
+======================
+
+.. versionchanged:: 2017.7.0
+    Access control group support added
+
+This example uses a single group called ``default``. In addition, other groups
+are being loaded from pillar data. The group names do not have any
+significance, it is the users and commands defined within them that are used to
+determine whether the Slack user has permission to run the desired command.
+
+.. code-block:: text
 
     engines:
       - slack:
@@ -32,7 +91,7 @@ In addition, other groups are being loaded from pillars.
           groups:
             default:
               users:
-                - *
+                - '*'
               commands:
                 - test.ping
                 - cmd.run
@@ -42,7 +101,7 @@ In addition, other groups are being loaded from pillars.
                 list_jobs:
                   cmd: jobs.list_jobs
                 list_commands:
-                  cmd: pillar.get salt:engines:slack:valid_commands target=saltmaster tgt_type=list
+                  cmd: 'pillar.get salt:engines:slack:valid_commands target=saltmaster tgt_type=list'
               default_target:
                 target: saltmaster
                 tgt_type: glob
@@ -54,12 +113,11 @@ In addition, other groups are being loaded from pillars.
                   target: saltmaster
                   tgt_type: list
 
-:configuration: Example configuration using the 'default' group and a non-default group and a pillar that will be merged in
-    If the user is '*' (without the quotes) then the group's users or commands will match all users as appropriate
+This example shows multiple groups applying to different users, with all users
+having access to run test.ping. Keep in mind that when using ``*``, the value
+must be quoted, or else PyYAML will fail to load the configuration.
 
-.. versionadded: 2017.7.0
-
-.. code-block:: yaml
+.. code-block:: text
 
     engines:
       - slack:
@@ -71,29 +129,20 @@ In addition, other groups are being loaded from pillars.
           groups_pillar_name: 'slack_engine:groups_pillar'
           groups:
             default:
-              valid_users:
-                - *
-              valid_commands:
+              users:
+                - '*'
+              commands:
                 - test.ping
               aliases:
                 list_jobs:
                   cmd: jobs.list_jobs
                 list_commands:
-                  cmd: pillar.get salt:engines:slack:valid_commands target=saltmaster tgt_type=list
+                  cmd: 'pillar.get salt:engines:slack:valid_commands target=saltmaster tgt_type=list'
             gods:
               users:
                 - garethgreenaway
               commands:
-                - *
-
-:depends: slackclient
-
-
-.. note:: groups_pillar_name
-
-    In order to use this, the engine must be running as a minion running on
-    the master, so that the ``Caller`` client can be used to retrieve that
-    minions pillar data, because the master process does not have pillars.
+                - '*'
 
 '''
 
@@ -327,7 +376,6 @@ class SlackClient(object):
                 cmdlist.append(cmditem)
         return cmdlist
 
-# m_data -> m_data, _text -> test, all_slack_users -> all_slack_users,
     def control_message_target(self, slack_user_name, text, loaded_groups, trigger_string):
         '''Returns a tuple of (target, cmdline,) for the response
 
@@ -401,13 +449,16 @@ class SlackClient(object):
         input_valid_users = set
         input_valid_commands = set
 
-        When the trigger_string prefixes the message text, yields a dictionary of {
-            'message_data': m_data,
-            'cmdline': cmdline_list, # this is a list
-            'channel': channel,
-            'user': m_data['user'],
-            'slack_client': sc
-        }
+        When the trigger_string prefixes the message text, yields a dictionary
+        of::
+
+            {
+                'message_data': m_data,
+                'cmdline': cmdline_list, # this is a list
+                'channel': channel,
+                'user': m_data['user'],
+                'slack_client': sc
+            }
 
         else yields {'message_data': m_data} and the caller can handle that
 
@@ -526,17 +577,18 @@ class SlackClient(object):
         If no configured target is provided, the command line will be parsed
         for target=foo and tgt_type=bar
 
-        Test for this:
-        h = {'aliases': {}, 'commands': {'cmd.run', 'pillar.get'},
-            'default_target': {'target': '*', 'tgt_type': 'glob'},
-            'targets': {'pillar.get': {'target': 'you_momma', 'tgt_type': 'list'}},
-            'users': {'dmangot', 'jmickle', 'pcn'}}
-        f = {'aliases': {}, 'commands': {'cmd.run', 'pillar.get'},
-             'default_target': {}, 'targets': {},'users': {'dmangot', 'jmickle', 'pcn'}}
+        Test for this::
 
-        g = {'aliases': {}, 'commands': {'cmd.run', 'pillar.get'},
-             'default_target': {'target': '*', 'tgt_type': 'glob'},
-             'targets': {}, 'users': {'dmangot', 'jmickle', 'pcn'}}
+            h = {'aliases': {}, 'commands': {'cmd.run', 'pillar.get'},
+                'default_target': {'target': '*', 'tgt_type': 'glob'},
+                'targets': {'pillar.get': {'target': 'you_momma', 'tgt_type': 'list'}},
+                'users': {'dmangot', 'jmickle', 'pcn'}}
+            f = {'aliases': {}, 'commands': {'cmd.run', 'pillar.get'},
+                 'default_target': {}, 'targets': {},'users': {'dmangot', 'jmickle', 'pcn'}}
+
+            g = {'aliases': {}, 'commands': {'cmd.run', 'pillar.get'},
+                 'default_target': {'target': '*', 'tgt_type': 'glob'},
+                 'targets': {}, 'users': {'dmangot', 'jmickle', 'pcn'}}
 
         Run each of them through ``get_configured_target(('foo', f), 'pillar.get')`` and confirm a valid target
 
@@ -740,7 +792,7 @@ class SlackClient(object):
         :param interval: time to wait between ending a loop and beginning the next
 
         '''
-        log.debug('Going to run a command async')
+        log.debug('Going to run a command asynchronous')
         runner_functions = sorted(salt.runner.Runner(__opts__).functions)
         # Parse args and kwargs
         cmd = msg['cmdline'][0]
@@ -762,7 +814,7 @@ class SlackClient(object):
             log.debug('Command %s will run via runner_functions', cmd)
             # pylint is tripping
             # pylint: disable=missing-whitespace-after-comma
-            job_id_dict = runner.async(cmd, {'args': args, 'kwargs': kwargs})
+            job_id_dict = runner.asynchronous(cmd, {'args': args, 'kwargs': kwargs})
             job_id = job_id_dict['jid']
 
         # Default to trying to run as a client module.
