@@ -6,15 +6,19 @@ The :mod:`cp.push <salt.modules.cp.push>` function allows Minions to push files
 up to the Master. Using this backend, these pushed files are exposed to other
 Minions via the Salt fileserver.
 
-To enable minionfs, :conf_master:`file_recv` needs to be set to ``True`` in
-the master config file (otherwise :mod:`cp.push <salt.modules.cp.push>` will
-not be allowed to push files to the Master), and ``minion`` must be added to
-the :conf_master:`fileserver_backends` list.
+To enable minionfs, :conf_master:`file_recv` needs to be set to ``True`` in the
+master config file (otherwise :mod:`cp.push <salt.modules.cp.push>` will not be
+allowed to push files to the Master), and ``minionfs`` must be added to the
+:conf_master:`fileserver_backends` list.
 
 .. code-block:: yaml
 
     fileserver_backend:
-      - minion
+      - minionfs
+
+.. note::
+    ``minion`` also works here. Prior to the 2018.3.0 release, *only*
+    ``minion`` would work.
 
 Other minionfs settings include: :conf_master:`minionfs_whitelist`,
 :conf_master:`minionfs_blacklist`, :conf_master:`minionfs_mountpoint`, and
@@ -23,7 +27,7 @@ Other minionfs settings include: :conf_master:`minionfs_whitelist`,
 .. seealso:: :ref:`tutorial-minionfs`
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import os
@@ -31,14 +35,22 @@ import logging
 
 # Import salt libs
 import salt.fileserver
-import salt.utils
+import salt.utils.files
+import salt.utils.gzip_util
+import salt.utils.hashutils
+import salt.utils.path
+import salt.utils.stringutils
 import salt.utils.url
+import salt.utils.versions
+
+# Import third party libs
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
 
 # Define the module's virtual name
-__virtualname__ = 'minion'
+__virtualname__ = 'minionfs'
 
 
 def __virtual__():
@@ -54,7 +66,7 @@ def _is_exposed(minion):
     '''
     Check if the minion is exposed, based on the whitelist and blacklist
     '''
-    return salt.utils.check_whitelist_blacklist(
+    return salt.utils.stringutils.check_whitelist_blacklist(
         minion,
         whitelist=__opts__['minionfs_whitelist'],
         blacklist=__opts__['minionfs_blacklist']
@@ -71,8 +83,10 @@ def find_file(path, tgt_env='base', **kwargs):  # pylint: disable=W0613
     if tgt_env not in envs():
         return fnd
     if os.path.basename(path) == 'top.sls':
-        log.debug('minionfs will NOT serve top.sls '
-                  'for security reasons (path requested: {0})'.format(path))
+        log.debug(
+            'minionfs will NOT serve top.sls '
+            'for security reasons (path requested: %s)', path
+        )
         return fnd
 
     mountpoint = salt.utils.url.strip_proto(__opts__['minionfs_mountpoint'])
@@ -121,13 +135,16 @@ def serve_file(load, fnd):
         return ret
     ret['dest'] = fnd['rel']
     gzip = load.get('gzip', None)
+    fpath = os.path.normpath(fnd['path'])
 
     # AP
     # May I sleep here to slow down serving of big files?
     # How many threads are serving files?
-    with salt.utils.fopen(fnd['path'], 'rb') as fp_:
+    with salt.utils.files.fopen(fpath, 'rb') as fp_:
         fp_.seek(load['loc'])
         data = fp_.read(__opts__['file_buffer_size'])
+        if data and six.PY3 and not salt.utils.files.is_binary(fpath):
+            data = data.decode(__salt_system_encoding__)
         if gzip and data:
             data = salt.utils.gzip_util.compress(data, gzip)
             ret['gzip'] = gzip
@@ -154,13 +171,10 @@ def file_hash(load, fnd):
     '''
     path = fnd['path']
     ret = {}
+
     if 'env' in load:
-        salt.utils.warn_until(
-            'Carbon',
-            'Passing a salt environment should be done using \'saltenv\' '
-            'not \'env\'. This functionality will be removed in Salt Carbon.'
-        )
-        load['saltenv'] = load.pop('env')
+        # "env" is not supported; Use "saltenv".
+        load.pop('env')
 
     if load['saltenv'] not in envs():
         return {}
@@ -176,16 +190,17 @@ def file_hash(load, fnd):
     # cache file's contents should be "hash:mtime"
     cache_path = os.path.join(
         __opts__['cachedir'],
-        'minionfs/hash',
+        'minionfs',
+        'hash',
         load['saltenv'],
         '{0}.hash.{1}'.format(fnd['rel'], __opts__['hash_type'])
     )
     # if we have a cache, serve that if the mtime hasn't changed
     if os.path.exists(cache_path):
         try:
-            with salt.utils.fopen(cache_path, 'rb') as fp_:
+            with salt.utils.files.fopen(cache_path, 'rb') as fp_:
                 try:
-                    hsum, mtime = fp_.read().split(':')
+                    hsum, mtime = salt.utils.stringutils.to_unicode(fp_.read()).split(':')
                 except ValueError:
                     log.debug(
                         'Fileserver attempted to read incomplete cache file. '
@@ -207,14 +222,14 @@ def file_hash(load, fnd):
             return ret
 
     # if we don't have a cache entry-- lets make one
-    ret['hsum'] = salt.utils.get_hash(path, __opts__['hash_type'])
+    ret['hsum'] = salt.utils.hashutils.get_hash(path, __opts__['hash_type'])
     cache_dir = os.path.dirname(cache_path)
     # make cache directory if it doesn't exist
     if not os.path.exists(cache_dir):
         os.makedirs(cache_dir)
     # save the cache object "hash:mtime"
     cache_object = '{0}:{1}'.format(ret['hsum'], os.path.getmtime(path))
-    with salt.utils.flopen(cache_path, 'w') as fp_:
+    with salt.utils.files.flopen(cache_path, 'w') as fp_:
         fp_.write(cache_object)
     return ret
 
@@ -224,12 +239,8 @@ def file_list(load):
     Return a list of all files on the file server in a specified environment
     '''
     if 'env' in load:
-        salt.utils.warn_until(
-            'Carbon',
-            'Passing a salt environment should be done using \'saltenv\' '
-            'not \'env\'. This functionality will be removed in Salt Carbon.'
-        )
-        load['saltenv'] = load.pop('env')
+        # "env" is not supported; Use "saltenv".
+        load.pop('env')
 
     if load['saltenv'] not in envs():
         return []
@@ -253,8 +264,8 @@ def file_list(load):
         # pushed files
         if tgt_minion not in minion_dirs:
             log.warning(
-                'No files found in minionfs cache for minion ID \'{0}\''
-                .format(tgt_minion)
+                'No files found in minionfs cache for minion ID \'%s\'',
+                tgt_minion
             )
             return []
         minion_dirs = [tgt_minion]
@@ -266,13 +277,13 @@ def file_list(load):
         minion_files_dir = os.path.join(minions_cache_dir, minion, 'files')
         if not os.path.isdir(minion_files_dir):
             log.debug(
-                'minionfs: could not find files directory under {0}!'
-                .format(os.path.join(minions_cache_dir, minion))
+                'minionfs: could not find files directory under %s!',
+                os.path.join(minions_cache_dir, minion)
             )
             continue
         walk_dir = os.path.join(minion_files_dir, prefix)
         # Do not follow links for security reasons
-        for root, _, files in os.walk(walk_dir, followlinks=False):
+        for root, _, files in salt.utils.path.os_walk(walk_dir, followlinks=False):
             for fname in files:
                 # Ignore links for security reasons
                 if os.path.islink(os.path.join(root, fname)):
@@ -307,12 +318,8 @@ def dir_list(load):
             - source-minion/absolute/path
     '''
     if 'env' in load:
-        salt.utils.warn_until(
-            'Carbon',
-            'Passing a salt environment should be done using \'saltenv\' '
-            'not \'env\'. This functionality will be removed in Salt Carbon.'
-        )
-        load['saltenv'] = load.pop('env')
+        # "env" is not supported; Use "saltenv".
+        load.pop('env')
 
     if load['saltenv'] not in envs():
         return []
@@ -336,8 +343,8 @@ def dir_list(load):
         # pushed files
         if tgt_minion not in minion_dirs:
             log.warning(
-                'No files found in minionfs cache for minion ID \'{0}\''
-                .format(tgt_minion)
+                'No files found in minionfs cache for minion ID \'%s\'',
+                tgt_minion
             )
             return []
         minion_dirs = [tgt_minion]
@@ -349,13 +356,13 @@ def dir_list(load):
         minion_files_dir = os.path.join(minions_cache_dir, minion, 'files')
         if not os.path.isdir(minion_files_dir):
             log.warning(
-                'minionfs: could not find files directory under {0}!'
-                .format(os.path.join(minions_cache_dir, minion))
+                'minionfs: could not find files directory under %s!',
+                os.path.join(minions_cache_dir, minion)
             )
             continue
         walk_dir = os.path.join(minion_files_dir, prefix)
         # Do not follow links for security reasons
-        for root, _, _ in os.walk(walk_dir, followlinks=False):
+        for root, _, _ in salt.utils.path.os_walk(walk_dir, followlinks=False):
             relpath = os.path.relpath(root, minion_files_dir)
             # Ensure that the current directory and directories outside of
             # the minion dir do not end up in return list
