@@ -50,16 +50,14 @@ The dependencies listed above can be installed via package or pip.
 #pylint: disable=E0602
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
-import json
 import datetime
 
 # Import Salt libs
-import salt.utils.boto3
 import salt.utils.compat
-import salt.utils
-from salt.utils.versions import LooseVersion as _LooseVersion
+import salt.utils.json
+import salt.utils.versions
 
 log = logging.getLogger(__name__)
 
@@ -85,22 +83,13 @@ def __virtual__():
     Only load if boto libraries exist and if boto libraries are greater than
     a given version.
     '''
-    required_boto3_version = '1.2.1'
-    required_botocore_version = '1.4.41'
     # the boto_lambda execution module relies on the connect_to_region() method
     # which was added in boto 2.8.0
     # https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
-    if not HAS_BOTO:
-        return (False, 'The boto_iot module could not be loaded: '
-                'boto libraries not found')
-    elif _LooseVersion(boto3.__version__) < _LooseVersion(required_boto3_version):
-        return (False, 'The boto_iot module could not be loaded: '
-                'boto3 version {0} or later must be installed.'.format(required_boto3_version))
-    elif _LooseVersion(found_botocore_version) < _LooseVersion(required_botocore_version):
-        return (False, 'The boto_iot module could not be loaded: '
-                'botocore version {0} or later must be installed.'.format(required_botocore_version))
-    else:
-        return True
+    return salt.utils.versions.check_boto_reqs(
+        boto3_ver='1.2.1',
+        botocore_ver='1.4.41'
+    )
 
 
 def __init__(opts):
@@ -135,7 +124,7 @@ def thing_type_exists(thingTypeName,
         else:
             return {'exists': False}
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'exists': False}
         return {'error': err}
@@ -172,7 +161,7 @@ def describe_thing_type(thingTypeName,
         else:
             return {'thing_type': None}
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'thing_type': None}
         return {'error': err}
@@ -210,14 +199,14 @@ def create_thing_type(thingTypeName, thingTypeDescription,
         )
 
         if thingtype:
-            log.info('The newly created thing type ARN is {0}'.format(thingtype['thingTypeArn']))
+            log.info('The newly created thing type ARN is %s', thingtype['thingTypeArn'])
 
             return {'created': True, 'thingTypeArn': thingtype['thingTypeArn']}
         else:
             log.warning('thing type was not created')
             return {'created': False}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def deprecate_thing_type(thingTypeName, undoDeprecate=False,
@@ -248,7 +237,7 @@ def deprecate_thing_type(thingTypeName, undoDeprecate=False,
         deprecated = True if undoDeprecate is False else False
         return {'deprecated': deprecated}
     except ClientError as e:
-        return {'deprecated': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deprecated': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_thing_type(thingTypeName,
@@ -274,7 +263,7 @@ def delete_thing_type(thingTypeName,
         conn.delete_thing_type(thingTypeName=thingTypeName)
         return {'deleted': True}
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'deleted': True}
         return {'deleted': False, 'error': err}
@@ -301,7 +290,7 @@ def policy_exists(policyName,
         conn.get_policy(policyName=policyName)
         return {'exists': True}
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'exists': False}
         return {'error': err}
@@ -330,18 +319,18 @@ def create_policy(policyName, policyDocument,
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         if not isinstance(policyDocument, string_types):
-            policyDocument = json.dumps(policyDocument)
+            policyDocument = salt.utils.json.dumps(policyDocument)
         policy = conn.create_policy(policyName=policyName,
                                     policyDocument=policyDocument)
         if policy:
-            log.info('The newly created policy version is {0}'.format(policy['policyVersionId']))
+            log.info('The newly created policy version is %s', policy['policyVersionId'])
 
             return {'created': True, 'versionId': policy['policyVersionId']}
         else:
             log.warning('Policy was not created')
             return {'created': False}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_policy(policyName,
@@ -365,7 +354,7 @@ def delete_policy(policyName,
         conn.delete_policy(policyName=policyName)
         return {'deleted': True}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_policy(policyName,
@@ -393,10 +382,10 @@ def describe_policy(policyName,
         else:
             return {'policy': None}
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'policy': None}
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def policy_version_exists(policyName, policyVersionId,
@@ -421,10 +410,10 @@ def policy_version_exists(policyName, policyVersionId,
                                          policyversionId=policyVersionId)
         return {'exists': bool(policy)}
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'exists': False}
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def create_policy_version(policyName, policyDocument, setAsDefault=False,
@@ -447,19 +436,19 @@ def create_policy_version(policyName, policyDocument, setAsDefault=False,
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         if not isinstance(policyDocument, string_types):
-            policyDocument = json.dumps(policyDocument)
+            policyDocument = salt.utils.json.dumps(policyDocument)
         policy = conn.create_policy_version(policyName=policyName,
                                     policyDocument=policyDocument,
                                     setAsDefault=setAsDefault)
         if policy:
-            log.info('The newly created policy version is {0}'.format(policy['policyVersionId']))
+            log.info('The newly created policy version is %s', policy['policyVersionId'])
 
             return {'created': True, 'name': policy['policyVersionId']}
         else:
             log.warning('Policy version was not created')
             return {'created': False}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_policy_version(policyName, policyVersionId,
@@ -484,7 +473,7 @@ def delete_policy_version(policyName, policyVersionId,
                                    policyVersionId=policyVersionId)
         return {'deleted': True}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_policy_version(policyName, policyVersionId,
@@ -513,10 +502,10 @@ def describe_policy_version(policyName, policyVersionId,
         else:
             return {'policy': None}
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'policy': None}
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def list_policies(region=None, key=None, keyid=None, profile=None):
@@ -543,7 +532,7 @@ def list_policies(region=None, key=None, keyid=None, profile=None):
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         policies = []
-        for ret in salt.utils.boto3.paged_call(conn.list_policies,
+        for ret in __utils__['boto3.paged_call'](conn.list_policies,
                                  marker_flag='nextMarker',
                                  marker_arg='marker'):
             policies.extend(ret['policies'])
@@ -551,7 +540,7 @@ def list_policies(region=None, key=None, keyid=None, profile=None):
             log.warning('No policies found')
         return {'policies': policies}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def list_policy_versions(policyName,
@@ -577,7 +566,7 @@ def list_policy_versions(policyName,
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         vers = []
-        for ret in salt.utils.boto3.paged_call(conn.list_policy_versions,
+        for ret in __utils__['boto3.paged_call'](conn.list_policy_versions,
                                  marker_flag='nextMarker',
                                  marker_arg='marker',
                                  policyName=policyName):
@@ -586,7 +575,7 @@ def list_policy_versions(policyName,
             log.warning('No versions found')
         return {'policyVersions': vers}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def set_default_policy_version(policyName, policyVersionId,
@@ -610,10 +599,10 @@ def set_default_policy_version(policyName, policyVersionId,
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         conn.set_default_policy_version(policyName=policyName,
-                                 policyVersionId=str(policyVersionId))
+                                 policyVersionId=str(policyVersionId))  # future lint: disable=blacklisted-function
         return {'changed': True}
     except ClientError as e:
-        return {'changed': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'changed': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def list_principal_policies(principal,
@@ -639,7 +628,7 @@ def list_principal_policies(principal,
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         vers = []
-        for ret in salt.utils.boto3.paged_call(conn.list_principal_policies,
+        for ret in __utils__['boto3.paged_call'](conn.list_principal_policies,
                                  principal=principal,
                                  marker_flag='nextMarker',
                                  marker_arg='marker'):
@@ -648,7 +637,7 @@ def list_principal_policies(principal,
             log.warning('No policies found')
         return {'policies': vers}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def attach_principal_policy(policyName, principal,
@@ -674,7 +663,7 @@ def attach_principal_policy(policyName, principal,
                                  principal=principal)
         return {'attached': True}
     except ClientError as e:
-        return {'attached': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'attached': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def detach_principal_policy(policyName, principal,
@@ -699,7 +688,7 @@ def detach_principal_policy(policyName, principal,
                                  principal=principal)
         return {'detached': True}
     except ClientError as e:
-        return {'detached': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'detached': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def topic_rule_exists(ruleName,
@@ -728,10 +717,10 @@ def topic_rule_exists(ruleName,
         # use, it's more useful to assume lack of existence than to assume a
         # genuine authorization problem; authorization problems should not be
         # the common case.
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'UnauthorizedException':
             return {'exists': False}
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def create_topic_rule(ruleName, sql, actions, description,
@@ -764,7 +753,7 @@ def create_topic_rule(ruleName, sql, actions, description,
                                })
         return {'created': True}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def replace_topic_rule(ruleName, sql, actions, description,
@@ -797,7 +786,7 @@ def replace_topic_rule(ruleName, sql, actions, description,
                                })
         return {'replaced': True}
     except ClientError as e:
-        return {'replaced': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'replaced': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_topic_rule(ruleName,
@@ -821,7 +810,7 @@ def delete_topic_rule(ruleName,
         conn.delete_topic_rule(ruleName=ruleName)
         return {'deleted': True}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_topic_rule(ruleName,
@@ -850,7 +839,7 @@ def describe_topic_rule(ruleName,
         else:
             return {'rule': None}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def list_topic_rules(topic=None, ruleDisabled=None,
@@ -883,7 +872,7 @@ def list_topic_rules(topic=None, ruleDisabled=None,
         if ruleDisabled is not None:
             kwargs['ruleDisabled'] = ruleDisabled
         rules = []
-        for ret in salt.utils.boto3.paged_call(conn.list_topic_rules,
+        for ret in __utils__['boto3.paged_call'](conn.list_topic_rules,
                                  marker_flag='nextToken',
                                  marker_arg='nextToken',
                                  **kwargs):
@@ -892,4 +881,4 @@ def list_topic_rules(topic=None, ruleDisabled=None,
             log.warning('No rules found')
         return {'rules': rules}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
