@@ -2,7 +2,7 @@
 '''
 Module for managing disks and blockdevices
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import logging
@@ -17,9 +17,10 @@ from salt.ext import six
 from salt.ext.six.moves import zip
 
 # Import salt libs
-import salt.utils
-import salt.utils.decorators as decorators
-from salt.utils.decorators import depends
+import salt.utils.decorators
+import salt.utils.decorators.path
+import salt.utils.path
+import salt.utils.platform
 from salt.exceptions import CommandExecutionError
 
 __func_alias__ = {
@@ -28,15 +29,15 @@ __func_alias__ = {
 
 log = logging.getLogger(__name__)
 
-HAS_HDPARM = salt.utils.which('hdparm') is not None
-HAS_IOSTAT = salt.utils.which('iostat') is not None
+HAS_HDPARM = salt.utils.path.which('hdparm') is not None
+HAS_IOSTAT = salt.utils.path.which('iostat') is not None
 
 
 def __virtual__():
     '''
     Only work on POSIX-like systems
     '''
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return False, 'This module doesn\'t work on Windows.'
     return True
 
@@ -249,7 +250,10 @@ def percent(args=None):
             log.error('Problem parsing disk usage information')
             ret = {}
     if args and args not in ret:
-        log.error('Problem parsing disk usage information: Partition \'{0}\' does not exist!'.format(args))
+        log.error(
+            'Problem parsing disk usage information: Partition \'%s\' '
+            'does not exist!', args
+        )
         ret = {}
     elif args:
         return ret[args]
@@ -257,7 +261,7 @@ def percent(args=None):
     return ret
 
 
-@decorators.which('blkid')
+@salt.utils.decorators.path.which('blkid')
 def blkid(device=None):
     '''
     Return block device attributes: UUID, LABEL, etc. This function only works
@@ -353,7 +357,7 @@ def wipe(device):
     if out['retcode'] == 0:
         return True
     else:
-        log.error('Error wiping device {0}: {1}'.format(device, out['stderr']))
+        log.error('Error wiping device %s: %s', device, out['stderr'])
         return False
 
 
@@ -406,8 +410,8 @@ def resize2fs(device):
         return True
 
 
-@decorators.which('sync')
-@decorators.which('mkfs')
+@salt.utils.decorators.path.which('sync')
+@salt.utils.decorators.path.which('mkfs')
 def format_(device,
             fs_type='ext4',
             inode_size=None,
@@ -452,10 +456,10 @@ def format_(device,
 
         salt '*' disk.format /dev/sdX1
     '''
-    cmd = ['mkfs', '-t', str(fs_type)]
+    cmd = ['mkfs', '-t', six.text_type(fs_type)]
     if inode_size is not None:
         if fs_type[:3] == 'ext':
-            cmd.extend(['-i', str(inode_size)])
+            cmd.extend(['-i', six.text_type(inode_size)])
         elif fs_type == 'xfs':
             cmd.extend(['-i', 'size={0}'.format(inode_size)])
     if lazy_itable_init is not None:
@@ -466,7 +470,7 @@ def format_(device,
             cmd.append('-F')
         elif fs_type == 'xfs':
             cmd.append('-f')
-    cmd.append(str(device))
+    cmd.append(six.text_type(device))
 
     mkfs_success = __salt__['cmd.retcode'](cmd, ignore_retcode=True) == 0
     sync_success = __salt__['cmd.retcode']('sync', ignore_retcode=True) == 0
@@ -474,7 +478,7 @@ def format_(device,
     return all([mkfs_success, sync_success])
 
 
-@decorators.which_bin(['lsblk', 'df'])
+@salt.utils.decorators.path.which_bin(['lsblk', 'df'])
 def fstype(device):
     '''
     Return the filesystem name of the specified device
@@ -490,14 +494,14 @@ def fstype(device):
 
         salt '*' disk.fstype /dev/sdX1
     '''
-    if salt.utils.which('lsblk'):
+    if salt.utils.path.which('lsblk'):
         lsblk_out = __salt__['cmd.run']('lsblk -o fstype {0}'.format(device)).splitlines()
         if len(lsblk_out) > 1:
             fs_type = lsblk_out[1].strip()
             if fs_type:
                 return fs_type
 
-    if salt.utils.which('df'):
+    if salt.utils.path.which('df'):
         # the fstype was not set on the block device, so inspect the filesystem
         # itself for its type
         if __grains__['kernel'] == 'AIX' and os.path.isfile('/usr/sysv/bin/df'):
@@ -516,7 +520,7 @@ def fstype(device):
     return ''
 
 
-@depends(HAS_HDPARM)
+@salt.utils.decorators.depends(HAS_HDPARM)
 def _hdparm(args, failhard=True):
     '''
     Execute hdparm
@@ -535,7 +539,7 @@ def _hdparm(args, failhard=True):
     return result['stdout']
 
 
-@depends(HAS_HDPARM)
+@salt.utils.decorators.depends(HAS_HDPARM)
 def hdparms(disks, args=None):
     '''
     Retrieve all info's for all disks
@@ -612,7 +616,7 @@ def hdparms(disks, args=None):
     return out
 
 
-@depends(HAS_HDPARM)
+@salt.utils.decorators.depends(HAS_HDPARM)
 def hpa(disks, size=None):
     '''
     Get/set Host Protected Area settings
@@ -741,7 +745,7 @@ def smart_attributes(dev, attributes=None, values=None):
     return smart_attr
 
 
-@depends(HAS_IOSTAT)
+@salt.utils.decorators.depends(HAS_IOSTAT)
 def iostat(interval=1, count=5, disks=None):
     '''
     Gather and return (averaged) IO stats.
@@ -757,11 +761,11 @@ def iostat(interval=1, count=5, disks=None):
 
         salt '*' disk.iostat 1 5 disks=sda
     '''
-    if salt.utils.is_linux():
+    if salt.utils.platform.is_linux():
         return _iostat_linux(interval, count, disks)
-    elif salt.utils.is_freebsd():
+    elif salt.utils.platform.is_freebsd():
         return _iostat_fbsd(interval, count, disks)
-    elif salt.utils.is_aix():
+    elif salt.utils.platform.is_aix():
         return _iostat_aix(interval, count, disks)
 
 
