@@ -2,17 +2,19 @@
 '''
 A collection of hashing and encoding functions
 '''
-from __future__ import absolute_import
 
-# Import python libs
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import base64
 import hashlib
 import hmac
 
 # Import Salt libs
 import salt.exceptions
-import salt.ext.six as six
-import salt.utils
+from salt.ext import six
+import salt.utils.files
+import salt.utils.hashutils
+import salt.utils.stringutils
 
 if six.PY2:
     import StringIO
@@ -71,7 +73,7 @@ def digest_file(infile, checksum='md5'):
         raise salt.exceptions.CommandExecutionError(
                 "File path '{0}' not found.".format(infile))
 
-    with open(infile, 'rb') as f:
+    with salt.utils.files.fopen(infile, 'rb') as f:
         file_hash = __salt__['hashutil.digest'](f.read(), checksum)
 
     return file_hash
@@ -92,11 +94,7 @@ def base64_b64encode(instr):
 
         salt '*' hashutil.base64_b64encode 'get salted'
     '''
-    if six.PY3:
-        b = salt.utils.to_bytes(instr)
-        b64 = base64.b64encode(b)
-        return salt.utils.to_str(b64)
-    return base64.b64encode(instr)
+    return salt.utils.hashutils.base64_b64encode(instr)
 
 
 def base64_b64decode(instr):
@@ -111,14 +109,7 @@ def base64_b64decode(instr):
 
         salt '*' hashutil.base64_b64decode 'Z2V0IHNhbHRlZA=='
     '''
-    if six.PY3:
-        b = salt.utils.to_bytes(instr)
-        data = base64.b64decode(b)
-        try:
-            return salt.utils.to_str(data)
-        except UnicodeDecodeError:
-            return data
-    return base64.b64decode(instr)
+    return salt.utils.hashutils.base64_b64decode(instr)
 
 
 def base64_encodestring(instr):
@@ -137,11 +128,7 @@ def base64_encodestring(instr):
 
         salt '*' hashutil.base64_encodestring 'get salted'
     '''
-    if six.PY3:
-        b = salt.utils.to_bytes(instr)
-        b64 = base64.encodebytes(b)
-        return salt.utils.to_str(b64)
-    return base64.encodestring(instr)
+    return salt.utils.hashutils.base64_encodestring(instr)
 
 
 def base64_encodefile(fname):
@@ -170,7 +157,7 @@ def base64_encodefile(fname):
     '''
     encoded_f = StringIO.StringIO()
 
-    with open(fname, 'rb') as f:
+    with salt.utils.files.fopen(fname, 'rb') as f:
         base64.encode(f, encoded_f)
 
     encoded_f.seek(0)
@@ -190,21 +177,14 @@ def base64_decodestring(instr):
         salt '*' hashutil.base64_decodestring instr='Z2V0IHNhbHRlZAo='
 
     '''
-    if six.PY3:
-        b = salt.utils.to_bytes(instr)
-        data = base64.decodebytes(b)
-        try:
-            return salt.utils.to_str(data)
-        except UnicodeDecodeError:
-            return data
-    return base64.decodestring(instr)
+    return salt.utils.hashutils.base64_decodestring(instr)
 
 
 def base64_decodefile(instr, outfile):
     r'''
     Decode a base64-encoded string and write the result to a file
 
-    .. versionadded:: 2015.2.0
+    .. versionadded:: 2016.3.0
 
     CLI Example:
 
@@ -214,7 +194,7 @@ def base64_decodefile(instr, outfile):
     '''
     encoded_f = StringIO.StringIO(instr)
 
-    with open(outfile, 'wb') as f:
+    with salt.utils.files.fopen(outfile, 'wb') as f:
         base64.decode(encoded_f, f)
 
     return True
@@ -232,10 +212,7 @@ def md5_digest(instr):
 
         salt '*' hashutil.md5_digest 'get salted'
     '''
-    if six.PY3:
-        b = salt.utils.to_bytes(instr)
-        return hashlib.md5(b).hexdigest()
-    return hashlib.md5(instr).hexdigest()
+    return salt.utils.hashutils.md5_digest(instr)
 
 
 def sha256_digest(instr):
@@ -250,10 +227,7 @@ def sha256_digest(instr):
 
         salt '*' hashutil.sha256_digest 'get salted'
     '''
-    if six.PY3:
-        b = salt.utils.to_bytes(instr)
-        return hashlib.sha256(b).hexdigest()
-    return hashlib.sha256(instr).hexdigest()
+    return salt.utils.hashutils.sha256_digest(instr)
 
 
 def sha512_digest(instr):
@@ -268,10 +242,7 @@ def sha512_digest(instr):
 
         salt '*' hashutil.sha512_digest 'get salted'
     '''
-    if six.PY3:
-        b = salt.utils.to_bytes(instr)
-        return hashlib.sha512(b).hexdigest()
-    return hashlib.sha512(instr).hexdigest()
+    return salt.utils.hashutils.sha512_digest(instr)
 
 
 def hmac_signature(string, shared_secret, challenge_hmac):
@@ -288,14 +259,29 @@ def hmac_signature(string, shared_secret, challenge_hmac):
 
         salt '*' hashutil.hmac_signature 'get salted' 'shared secret' 'eBWf9bstXg+NiP5AOwppB5HMvZiYMPzEM9W5YMm/AmQ='
     '''
-    if six.PY3:
-        msg = salt.utils.to_bytes(string)
-        key = salt.utils.to_bytes(shared_secret)
-        challenge = salt.utils.to_bytes(challenge_hmac)
-    else:
-        msg = string
-        key = shared_secret
-        challenge = challenge_hmac
-    hmac_hash = hmac.new(key, msg, hashlib.sha256)
-    valid_hmac = base64.b64encode(hmac_hash.digest())
-    return valid_hmac == challenge
+    return salt.utils.hashutils.hmac_signature(string, shared_secret, challenge_hmac)
+
+
+def github_signature(string, shared_secret, challenge_hmac):
+    '''
+    Verify a challenging hmac signature against a string / shared-secret for
+    github webhooks.
+
+    .. versionadded:: 2017.7.0
+
+    Returns a boolean if the verification succeeded or failed.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' hashutil.github_signature '{"ref":....} ' 'shared secret' 'sha1=bc6550fc290acf5b42283fa8deaf55cea0f8c206'
+    '''
+    msg = string
+    key = shared_secret
+    hashtype, challenge = challenge_hmac.split('=')
+    if six.text_type:
+        msg = salt.utils.stringutils.to_bytes(msg)
+        key = salt.utils.stringutils.to_bytes(key)
+    hmac_hash = hmac.new(key, msg, getattr(hashlib, hashtype))
+    return hmac_hash.hexdigest() == challenge
