@@ -9,37 +9,75 @@ Module for handling kubernetes calls.
         kubernetes.user: admin
         kubernetes.password: verybadpass
         kubernetes.api_url: 'http://127.0.0.1:8080'
+        kubernetes.certificate-authority-data: '...'
+        kubernetes.client-certificate-data: '....n
+        kubernetes.client-key-data: '...'
+        kubernetes.certificate-authority-file: '/path/to/ca.crt'
+        kubernetes.client-certificate-file: '/path/to/client.crt'
+        kubernetes.client-key-file: '/path/to/client.key'
 
-These settings can be also overrided by adding `api_url`, `api_user`,
-or `api_password` parameters when calling a function:
+
+These settings can be also overridden by adding `api_url`, `api_user`,
+`api_password`, `api_certificate_authority_file`, `api_client_certificate_file`
+or `api_client_key_file` parameters when calling a function:
+
+The data format for `kubernetes.*-data` values is the same as provided in `kubeconfig`.
+It's base64 encoded certificates/keys in one line.
+
+For an item only one field should be provided. Either a `data` or a `file` entry.
+In case both are provided the `file` entry is preferred.
 
 .. code-block:: bash
+
     salt '*' kubernetes.nodes api_url=http://k8s-api-server:port api_user=myuser api_password=pass
 
+.. versionadded: 2017.7.0
+
+.. warning::
+
+    Configuration options will change in Fluorine. All options above will be replaced by:
+
+    - kubernetes.kubeconfig or kubernetes.kubeconfig-data
+    - kubernetes.context
 '''
 
 # Import Python Futures
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
+import sys
+import os.path
 import base64
 import logging
-import yaml
+import tempfile
+import signal
+from time import sleep
+from contextlib import contextmanager
 
 from salt.exceptions import CommandExecutionError
 from salt.ext.six import iteritems
-import salt.utils
+from salt.ext import six
+import salt.utils.files
 import salt.utils.templates
+import salt.utils.yaml
+from salt.exceptions import TimeoutError
+from salt.ext.six.moves import range  # pylint: disable=import-error
 
 try:
     import kubernetes  # pylint: disable=import-self
     import kubernetes.client
     from kubernetes.client.rest import ApiException
     from urllib3.exceptions import HTTPError
+    try:
+        # There is an API change in Kubernetes >= 2.0.0.
+        from kubernetes.client import V1beta1Deployment as AppsV1beta1Deployment
+        from kubernetes.client import V1beta1DeploymentSpec as AppsV1beta1DeploymentSpec
+    except ImportError:
+        from kubernetes.client import AppsV1beta1Deployment
+        from kubernetes.client import AppsV1beta1DeploymentSpec
 
     HAS_LIBS = True
 except ImportError:
     HAS_LIBS = False
 
-
 log = logging.getLogger(__name__)
 
 __virtualname__ = 'kubernetes'
@@ -55,6 +93,21 @@ def __virtual__():
     return False, 'python kubernetes library not found'
 
 
+if not salt.utils.platform.is_windows():
+    @contextmanager
+    def _time_limit(seconds):
+        def signal_handler(signum, frame):
+            raise TimeoutError
+        signal.signal(signal.SIGALRM, signal_handler)
+        signal.alarm(seconds)
+        try:
+            yield
+        finally:
+            signal.alarm(0)
+
+    POLLING_TIME_LIMIT = 30
+
+
 # pylint: disable=no-member
 def _setup_conn(**kwargs):
     '''
@@ -64,16 +117,31 @@ def _setup_conn(**kwargs):
                                      'http://localhost:8080')
     username = __salt__['config.option']('kubernetes.user')
     password = __salt__['config.option']('kubernetes.password')
+    ca_cert = __salt__['config.option']('kubernetes.certificate-authority-data')
+    client_cert = __salt__['config.option']('kubernetes.client-certificate-data')
+    client_key = __salt__['config.option']('kubernetes.client-key-data')
+    ca_cert_file = __salt__['config.option']('kubernetes.certificate-authority-file')
+    client_cert_file = __salt__['config.option']('kubernetes.client-certificate-file')
+    client_key_file = __salt__['config.option']('kubernetes.client-key-file')
 
     # Override default API settings when settings are provided
-    if kwargs.get('api_url'):
-        host = kwargs['api_url']
+    if 'api_url' in kwargs:
+        host = kwargs.get('api_url')
 
-    if kwargs.get('api_user'):
-        username = kwargs['api_user']
+    if 'api_user' in kwargs:
+        username = kwargs.get('api_user')
 
-    if kwargs.get('api_password'):
-        password = kwargs['api_password']
+    if 'api_password' in kwargs:
+        password = kwargs.get('api_password')
+
+    if 'api_certificate_authority_file' in kwargs:
+        ca_cert_file = kwargs.get('api_certificate_authority_file')
+
+    if 'api_client_certificate_file' in kwargs:
+        client_cert_file = kwargs.get('api_client_certificate_file')
+
+    if 'api_client_key_file' in kwargs:
+        client_key_file = kwargs.get('api_client_key_file')
 
     if (
             kubernetes.client.configuration.host != host or
@@ -85,6 +153,51 @@ def _setup_conn(**kwargs):
     kubernetes.client.configuration.host = host
     kubernetes.client.configuration.user = username
     kubernetes.client.configuration.passwd = password
+    if __salt__['config.option']('kubernetes.api_key'):
+        kubernetes.client.configuration.api_key = {'authorization': __salt__['config.option']('kubernetes.api_key')}
+        kubernetes.client.configuration.api_key_prefix = {'authorization': __salt__['config.option']('kubernetes.api_key_prefix')}
+
+    if ca_cert_file:
+        kubernetes.client.configuration.ssl_ca_cert = ca_cert_file
+    elif ca_cert:
+        with tempfile.NamedTemporaryFile(prefix='salt-kube-', delete=False) as ca:
+            ca.write(base64.b64decode(ca_cert))
+            kubernetes.client.configuration.ssl_ca_cert = ca.name
+    else:
+        kubernetes.client.configuration.ssl_ca_cert = None
+
+    if client_cert_file:
+        kubernetes.client.configuration.cert_file = client_cert_file
+    elif client_cert:
+        with tempfile.NamedTemporaryFile(prefix='salt-kube-', delete=False) as c:
+            c.write(base64.b64decode(client_cert))
+            kubernetes.client.configuration.cert_file = c.name
+    else:
+        kubernetes.client.configuration.cert_file = None
+
+    if client_key_file:
+        kubernetes.client.configuration.key_file = client_key_file
+    elif client_key:
+        with tempfile.NamedTemporaryFile(prefix='salt-kube-', delete=False) as k:
+            k.write(base64.b64decode(client_key))
+            kubernetes.client.configuration.key_file = k.name
+    else:
+        kubernetes.client.configuration.key_file = None
+
+    # The return makes unit testing easier
+    return vars(kubernetes.client.configuration)
+
+
+def _cleanup(**kwargs):
+    ca = kubernetes.client.configuration.ssl_ca_cert
+    cert = kubernetes.client.configuration.cert_file
+    key = kubernetes.client.configuration.key_file
+    if cert and os.path.exists(cert) and os.path.basename(cert).startswith('salt-kube-'):
+        salt.utils.files.safe_rm(cert)
+    if key and os.path.exists(key) and os.path.basename(key).startswith('salt-kube-'):
+        salt.utils.files.safe_rm(key)
+    if ca and os.path.exists(ca) and os.path.basename(ca).startswith('salt-kube-'):
+        salt.utils.files.safe_rm(ca)
 
 
 def ping(**kwargs):
@@ -123,10 +236,10 @@ def nodes(**kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->list_node: {0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->list_node')
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def node(name, **kwargs):
@@ -145,10 +258,10 @@ def node(name, **kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->list_node: {0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->list_node')
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
     for k8s_node in api_response.items:
         if k8s_node.metadata.name == name:
@@ -168,7 +281,7 @@ def node_labels(name, **kwargs):
     match = node(name, **kwargs)
 
     if match is not None:
-        return match.metadata.labels
+        return match['metadata']['labels']
 
     return {}
 
@@ -199,10 +312,10 @@ def node_add_label(node_name, label_name, label_value, **kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->patch_node: {0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->patch_node')
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
     return None
 
@@ -232,10 +345,10 @@ def node_remove_label(node_name, label_name, **kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->patch_node: {0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->patch_node')
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
     return None
 
@@ -259,11 +372,10 @@ def namespaces(**kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->list_namespace: '
-                '{0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->list_namespace')
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def deployments(namespace='default', **kwargs):
@@ -287,10 +399,11 @@ def deployments(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->list_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->list_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def services(namespace='default', **kwargs):
@@ -314,9 +427,11 @@ def services(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->list_namespaced_service: {0}'.format(exc)
+                'CoreV1Api->list_namespaced_service'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def pods(namespace='default', **kwargs):
@@ -340,9 +455,11 @@ def pods(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->list_namespaced_pod: {0}'.format(exc)
+                'CoreV1Api->list_namespaced_pod'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def secrets(namespace='default', **kwargs):
@@ -366,9 +483,11 @@ def secrets(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->list_namespaced_secret: {0}'.format(exc)
+                'CoreV1Api->list_namespaced_secret'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def configmaps(namespace='default', **kwargs):
@@ -392,9 +511,11 @@ def configmaps(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->list_namespaced_config_map: {0}'.format(exc)
+                'CoreV1Api->list_namespaced_config_map'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def show_deployment(name, namespace='default', **kwargs):
@@ -418,10 +539,11 @@ def show_deployment(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->read_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->read_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def show_service(name, namespace='default', **kwargs):
@@ -445,9 +567,11 @@ def show_service(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespaced_service: {0}'.format(exc)
+                'CoreV1Api->read_namespaced_service'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def show_pod(name, namespace='default', **kwargs):
@@ -471,9 +595,11 @@ def show_pod(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespaced_pod: {0}'.format(exc)
+                'CoreV1Api->read_namespaced_pod'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def show_namespace(name, **kwargs):
@@ -496,9 +622,11 @@ def show_namespace(name, **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespace: {0}'.format(exc)
+                'CoreV1Api->read_namespace'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def show_secret(name, namespace='default', decode=False, **kwargs):
@@ -530,10 +658,11 @@ def show_secret(name, namespace='default', decode=False, **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespaced_secret: '
-                '{0}'.format(exc)
+                'CoreV1Api->read_namespaced_secret'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def show_configmap(name, namespace='default', **kwargs):
@@ -559,10 +688,11 @@ def show_configmap(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespaced_config_map: '
-                '{0}'.format(exc)
+                'CoreV1Api->read_namespaced_config_map'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def delete_deployment(name, namespace='default', **kwargs):
@@ -583,17 +713,41 @@ def delete_deployment(name, namespace='default', **kwargs):
             name=name,
             namespace=namespace,
             body=body)
-        return api_response.to_dict()
+        mutable_api_response = api_response.to_dict()
+        if not salt.utils.platform.is_windows():
+            try:
+                with _time_limit(POLLING_TIME_LIMIT):
+                    while show_deployment(name, namespace) is not None:
+                        sleep(1)
+                    else:  # pylint: disable=useless-else-on-loop
+                        mutable_api_response['code'] = 200
+            except TimeoutError:
+                pass
+        else:
+            # Windows has not signal.alarm implementation, so we are just falling
+            # back to loop-counting.
+            for i in range(60):
+                if show_deployment(name, namespace) is None:
+                    mutable_api_response['code'] = 200
+                    break
+                else:
+                    sleep(1)
+        if mutable_api_response['code'] != 200:
+            log.warning('Reached polling time limit. Deployment is not yet '
+                        'deleted, but we are backing off. Sorry, but you\'ll '
+                        'have to check manually.')
+        return mutable_api_response
     except (ApiException, HTTPError) as exc:
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->delete_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->delete_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def delete_service(name, namespace='default', **kwargs):
@@ -619,10 +773,11 @@ def delete_service(name, namespace='default', **kwargs):
             return None
         else:
             log.exception(
-                'Exception when calling CoreV1Api->delete_namespaced_service: '
-                '{0}'.format(exc)
+                'Exception when calling CoreV1Api->delete_namespaced_service'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def delete_pod(name, namespace='default', **kwargs):
@@ -651,9 +806,11 @@ def delete_pod(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->delete_namespaced_pod: {0}'.format(exc)
+                'CoreV1Api->delete_namespaced_pod'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def delete_namespace(name, **kwargs):
@@ -678,10 +835,11 @@ def delete_namespace(name, **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->delete_namespace: '
-                '{0}'.format(exc)
+                'CoreV1Api->delete_namespace'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def delete_secret(name, namespace='default', **kwargs):
@@ -709,10 +867,11 @@ def delete_secret(name, namespace='default', **kwargs):
             return None
         else:
             log.exception(
-                'Exception when calling CoreV1Api->delete_namespaced_secret: '
-                '{0}'.format(exc)
+                'Exception when calling CoreV1Api->delete_namespaced_secret'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def delete_configmap(name, namespace='default', **kwargs):
@@ -741,10 +900,11 @@ def delete_configmap(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->delete_namespaced_config_map: '
-                '{0}'.format(exc)
+                'CoreV1Api->delete_namespaced_config_map'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def create_deployment(
@@ -761,7 +921,7 @@ def create_deployment(
     '''
     body = __create_object_body(
         kind='Deployment',
-        obj_class=kubernetes.client.V1beta1Deployment,
+        obj_class=AppsV1beta1Deployment,
         spec_creator=__dict_to_deployment_spec,
         name=name,
         namespace=namespace,
@@ -785,10 +945,11 @@ def create_deployment(
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->create_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->create_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def create_pod(
@@ -829,10 +990,11 @@ def create_pod(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespaced_pod: '
-                '{0}'.format(exc)
+                'CoreV1Api->create_namespaced_pod'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def create_service(
@@ -873,21 +1035,31 @@ def create_service(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespaced_service: {0}'.format(exc)
+                'CoreV1Api->create_namespaced_service'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def create_secret(
         name,
-        namespace,
-        data,
-        source,
-        template,
-        saltenv,
+        namespace='default',
+        data=None,
+        source=None,
+        template=None,
+        saltenv='base',
         **kwargs):
     '''
     Creates the kubernetes secret as defined by the user.
+
+    CLI Examples::
+
+        salt 'minion1' kubernetes.create_secret \
+            passwords default '{"db": "letmein"}'
+
+        salt 'minion2' kubernetes.create_secret \
+            name=passwords namespace=default data='{"db": "letmein"}'
     '''
     if source:
         data = __read_and_render_yaml_file(source, template, saltenv)
@@ -918,21 +1090,31 @@ def create_secret(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespaced_secret: {0}'.format(exc)
+                'CoreV1Api->create_namespaced_secret'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def create_configmap(
         name,
         namespace,
         data,
-        source,
-        template,
-        saltenv,
+        source=None,
+        template=None,
+        saltenv='base',
         **kwargs):
     '''
     Creates the kubernetes configmap as defined by the user.
+
+    CLI Examples::
+
+        salt 'minion1' kubernetes.create_configmap \
+            settings default '{"example.conf": "# example file"}'
+
+        salt 'minion2' kubernetes.create_configmap \
+            name=settings namespace=default data='{"example.conf": "# example file"}'
     '''
     if source:
         data = __read_and_render_yaml_file(source, template, saltenv)
@@ -959,9 +1141,11 @@ def create_configmap(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespaced_config_map: {0}'.format(exc)
+                'CoreV1Api->create_namespaced_config_map'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def create_namespace(
@@ -992,10 +1176,11 @@ def create_namespace(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespace: '
-                '{0}'.format(exc)
+                'CoreV1Api->create_namespace'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def replace_deployment(name,
@@ -1012,7 +1197,7 @@ def replace_deployment(name,
     '''
     body = __create_object_body(
         kind='Deployment',
-        obj_class=kubernetes.client.V1beta1Deployment,
+        obj_class=AppsV1beta1Deployment,
         spec_creator=__dict_to_deployment_spec,
         name=name,
         namespace=namespace,
@@ -1036,10 +1221,11 @@ def replace_deployment(name,
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->replace_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->replace_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def replace_service(name,
@@ -1086,21 +1272,31 @@ def replace_service(name,
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->replace_namespaced_service: {0}'.format(exc)
+                'CoreV1Api->replace_namespaced_service'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def replace_secret(name,
                    data,
-                   source,
-                   template,
-                   saltenv,
+                   source=None,
+                   template=None,
+                   saltenv='base',
                    namespace='default',
                    **kwargs):
     '''
     Replaces an existing secret with a new one defined by name and namespace,
     having the specificed data.
+
+    CLI Examples::
+
+        salt 'minion1' kubernetes.replace_secret \
+            name=passwords data='{"db": "letmein"}'
+
+        salt 'minion2' kubernetes.replace_secret \
+            name=passwords namespace=saltstack data='{"db": "passw0rd"}'
     '''
     if source:
         data = __read_and_render_yaml_file(source, template, saltenv)
@@ -1131,21 +1327,31 @@ def replace_secret(name,
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->replace_namespaced_secret: {0}'.format(exc)
+                'CoreV1Api->replace_namespaced_secret'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def replace_configmap(name,
                       data,
-                      source,
-                      template,
-                      saltenv,
+                      source=None,
+                      template=None,
+                      saltenv='base',
                       namespace='default',
                       **kwargs):
     '''
     Replaces an existing configmap with a new one defined by name and
-    namespace, having the specificed data.
+    namespace with the specified data.
+
+    CLI Examples::
+
+        salt 'minion1' kubernetes.replace_configmap \
+            settings default '{"example.conf": "# example file"}'
+
+        salt 'minion2' kubernetes.replace_configmap \
+            name=settings namespace=default data='{"example.conf": "# example file"}'
     '''
     if source:
         data = __read_and_render_yaml_file(source, template, saltenv)
@@ -1170,9 +1376,11 @@ def replace_configmap(name,
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->replace_namespaced_configmap: {0}'.format(exc)
+                'CoreV1Api->replace_namespaced_configmap'
             )
             raise CommandExecutionError(exc)
+    finally:
+        _cleanup()
 
 
 def __create_object_body(kind,
@@ -1220,7 +1428,7 @@ def __read_and_render_yaml_file(source,
         raise CommandExecutionError(
             'Source file \'{0}\' not found'.format(source))
 
-    with salt.utils.fopen(sfn, 'r') as src:
+    with salt.utils.files.fopen(sfn, 'r') as src:
         contents = src.read()
 
         if template:
@@ -1251,7 +1459,7 @@ def __read_and_render_yaml_file(source,
                     'Unknown template specified: {0}'.format(
                         template))
 
-        return yaml.load(contents)
+        return salt.utils.yaml.safe_load(contents)
 
 
 def __dict_to_object_meta(name, namespace, metadata):
@@ -1260,6 +1468,13 @@ def __dict_to_object_meta(name, namespace, metadata):
     '''
     meta_obj = kubernetes.client.V1ObjectMeta()
     meta_obj.namespace = namespace
+
+    # Replicate `kubectl [create|replace|apply] --record`
+    if 'annotations' not in metadata:
+        metadata['annotations'] = {}
+    if 'kubernetes.io/change-cause' not in metadata['annotations']:
+        metadata['annotations']['kubernetes.io/change-cause'] = ' '.join(sys.argv)
+
     for key, value in iteritems(metadata):
         if hasattr(meta_obj, key):
             setattr(meta_obj, key, value)
@@ -1275,9 +1490,9 @@ def __dict_to_object_meta(name, namespace, metadata):
 
 def __dict_to_deployment_spec(spec):
     '''
-    Converts a dictionary into kubernetes V1beta1DeploymentSpec instance.
+    Converts a dictionary into kubernetes AppsV1beta1DeploymentSpec instance.
     '''
-    spec_obj = kubernetes.client.V1beta1DeploymentSpec()
+    spec_obj = AppsV1beta1DeploymentSpec(template=spec.get('template', ''))
     for key, value in iteritems(spec):
         if hasattr(spec_obj, key):
             setattr(spec_obj, key, value)
@@ -1327,6 +1542,6 @@ def __enforce_only_strings_dict(dictionary):
     ret = {}
 
     for key, value in iteritems(dictionary):
-        ret[str(key)] = str(value)
+        ret[six.text_type(key)] = six.text_type(value)
 
     return ret
