@@ -2,15 +2,20 @@
 '''
 Salt module to manage RAID arrays with mdadm
 '''
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import os
 import logging
+import re
 
 # Import salt libs
-import salt.utils
+import salt.utils.path
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 
+# Import 3rd-party libs
+from salt.ext import six
+
 # Set up logger
 log = logging.getLogger(__name__)
 
@@ -29,9 +34,9 @@ def __virtual__():
     mdadm provides raid functions for Linux
     '''
     if __grains__['kernel'] != 'Linux':
-        return False
-    if not salt.utils.which('mdadm'):
-        return False
+        return (False, 'The mdadm execution module cannot be loaded: only available on Linux.')
+    if not salt.utils.path.which('mdadm'):
+        return (False, 'The mdadm execution module cannot be loaded: the mdadm binary is not in the path.')
     return __virtualname__
 
 
@@ -148,6 +153,24 @@ def destroy(device):
         return False
 
 
+def stop():
+    '''
+    Shut down all arrays that can be shut down (i.e. are not currently in use).
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' raid.stop
+    '''
+    cmd = 'mdadm --stop --scan'
+
+    if __salt__['cmd.retcode'](cmd):
+        return True
+
+    return False
+
+
 def create(name,
            level,
            devices,
@@ -183,6 +206,9 @@ def create(name,
     devices
         A list of devices used to build the array.
 
+    metadata
+        Version of metadata to use when creating the array.
+
     kwargs
         Optional arguments to be passed to mdadm.
 
@@ -205,19 +231,23 @@ def create(name,
     For more info, read the ``mdadm(8)`` manpage
     '''
     opts = []
+    raid_devices = len(devices)
+
     for key in kwargs:
         if not key.startswith('__'):
             opts.append('--{0}'.format(key))
             if kwargs[key] is not True:
-                opts.append(str(kwargs[key]))
+                opts.append(six.text_type(kwargs[key]))
+        if key == 'spare-devices':
+            raid_devices -= int(kwargs[key])
 
     cmd = ['mdadm',
            '-C', name,
            '-R',
            '-v'] + opts + [
-           '-l', str(level),
+           '-l', six.text_type(level),
            '-e', metadata,
-           '-n', str(len(devices))] + devices
+           '-n', six.text_type(raid_devices)] + devices
 
     cmd_str = ' '.join(cmd)
 
@@ -244,26 +274,34 @@ def save_config():
         salt '*' raid.save_config
 
     '''
-    scan = __salt__['cmd.run']('mdadm --detail --scan', python_shell=False).split()
+    scan = __salt__['cmd.run']('mdadm --detail --scan', python_shell=False).splitlines()
     # Issue with mdadm and ubuntu
     # REF: http://askubuntu.com/questions/209702/why-is-my-raid-dev-md1-showing-up-as-dev-md126-is-mdadm-conf-being-ignored
     if __grains__['os'] == 'Ubuntu':
         buggy_ubuntu_tags = ['name', 'metadata']
-        for bad_tag in buggy_ubuntu_tags:
-            for i, elem in enumerate(scan):
-                if not elem.find(bad_tag):
-                    del scan[i]
+        for i, elem in enumerate(scan):
+            for bad_tag in buggy_ubuntu_tags:
+                pattern = r'\s{0}=\S+'.format(re.escape(bad_tag))
+                pattern = re.compile(pattern, flags=re.I)
+                scan[i] = re.sub(pattern, '', scan[i])
 
-    scan = ' '.join(scan)
     if __grains__.get('os_family') == 'Debian':
         cfg_file = '/etc/mdadm/mdadm.conf'
     else:
         cfg_file = '/etc/mdadm.conf'
 
-    if not __salt__['file.search'](cfg_file, scan):
-        __salt__['file.append'](cfg_file, scan)
+    try:
+        vol_d = dict([(line.split()[1], line) for line in scan])
+        for vol in vol_d:
+            pattern = r'^ARRAY\s+{0}.*$'.format(re.escape(vol))
+            __salt__['file.replace'](cfg_file, pattern, vol_d[vol], append_if_not_found=True)
+    except SaltInvocationError:  # File is missing
+        __salt__['file.write'](cfg_file, args=scan)
 
-    return __salt__['cmd.run']('update-initramfs -u')
+    if __grains__.get('os_family') == 'Debian':
+        return __salt__['cmd.run']('update-initramfs -u')
+    elif __grains__.get('os_family') == 'RedHat':
+        return __salt__['cmd.run']('dracut --force')
 
 
 def assemble(name,
@@ -309,12 +347,49 @@ def assemble(name,
                 opts.append(kwargs[key])
 
     # Devices may have been written with a blob:
-    if type(devices) is str:
+    if isinstance(devices, six.string_types):
         devices = devices.split(',')
 
-    cmd = ['mdadm', '-A', name, '-v', opts] + devices
+    cmd = ['mdadm', '-A', name, '-v'] + opts + devices
 
     if test_mode is True:
         return cmd
     elif test_mode is False:
         return __salt__['cmd.run'](cmd, python_shell=False)
+
+
+def examine(device):
+    '''
+    Show detail for a specified RAID component device
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' raid.examine '/dev/sda1'
+    '''
+    res = __salt__['cmd.run_stdout']('mdadm -Y -E {0}'.format(device), output_loglevel='trace', python_shell=False)
+    ret = {}
+
+    for line in res.splitlines():
+        name, var = line.partition("=")[::2]
+        ret[name] = var
+    return ret
+
+
+def add(name, device):
+    '''
+    Add new device to RAID array.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' raid.add /dev/md0 /dev/sda1
+
+    '''
+
+    cmd = 'mdadm --manage {0} --add {1}'.format(name, device)
+    if __salt__['cmd.retcode'](cmd) == 0:
+        return True
+    return False
