@@ -2,6 +2,12 @@
 '''
 Support for ``pkgng``, the new package manager for FreeBSD
 
+.. important::
+    If you feel that Salt should be using this module to manage packages on a
+    minion, and it is using a different module (or gives an error similar to
+    *'pkg.install' is not available*), see :ref:`here
+    <module-provider-override>`.
+
 .. warning::
 
     This module has been completely rewritten. Up to and including version
@@ -30,7 +36,7 @@ file, in order to use this module to manage packages, like so:
       pkg: pkgng
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import copy
@@ -38,9 +44,15 @@ import logging
 import os
 
 # Import salt libs
-import salt.utils
+import salt.utils.data
+import salt.utils.files
+import salt.utils.functools
+import salt.utils.itertools
+import salt.utils.pkg
+import salt.utils.stringutils
+import salt.utils.versions
 from salt.exceptions import CommandExecutionError, MinionError
-import salt.ext.six as six
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -50,32 +62,58 @@ __virtualname__ = 'pkg'
 
 def __virtual__():
     '''
-    Load as 'pkg' on FreeBSD 10 and greater
+    Load as 'pkg' on FreeBSD 10 and greater.
+    Load as 'pkg' on DragonFly BSD.
+    Load as 'pkg' on FreeBSD 9 when config option
+    ``providers:pkg`` is set to 'pkgng'.
     '''
+    if __grains__['kernel'] == 'DragonFly':
+        return __virtualname__
     if __grains__['os'] == 'FreeBSD' and float(__grains__['osrelease']) >= 10:
         return __virtualname__
-    return False
+    if __grains__['os'] == 'FreeBSD' and int(__grains__['osmajorrelease']) == 9:
+        providers = {}
+        if 'providers' in __opts__:
+            providers = __opts__['providers']
+        log.debug('__opts__.providers: %s', providers)
+        if providers and 'pkg' in providers and providers['pkg'] == 'pkgng':
+            log.debug('Configuration option \'providers:pkg\' is set to '
+                '\'pkgng\', using \'pkgng\' in favor of \'freebsdpkg\'.')
+            return __virtualname__
+    return (False,
+            'The pkgng execution module cannot be loaded: only available '
+            'on FreeBSD 10 or FreeBSD 9 with providers.pkg set to pkgng.')
 
 
-def _pkg(jail=None, chroot=None):
+def _pkg(jail=None, chroot=None, root=None):
     '''
     Returns the prefix for a pkg command, using -j if a jail is specified, or
     -c if chroot is specified.
     '''
-    ret = 'pkg'
+    ret = ['pkg']
     if jail:
-        ret += ' -j {0!r}'.format(jail)
+        ret.extend(['-j', jail])
     elif chroot:
-        ret += ' -c {0!r}'.format(chroot)
+        ret.extend(['-c', chroot])
+    elif root:
+        ret.extend(['-r', root])
     return ret
 
 
+def _get_pkgng_version(jail=None, chroot=None, root=None):
+    '''
+    return the version of 'pkg'
+    '''
+    cmd = _pkg(jail, chroot, root) + ['--version']
+    return __salt__['cmd.run'](cmd).strip()
+
+
 def _get_version(name, results):
     '''
     ``pkg search`` will return all packages for which the pattern is a match.
     Narrow this down and return the package version, or None if no exact match.
     '''
-    for line in results.splitlines():
+    for line in salt.utils.itertools.split(results, '\n'):
         if not line:
             continue
         try:
@@ -87,16 +125,18 @@ def _get_version(name, results):
     return None
 
 
-def _contextkey(jail=None, chroot=None, prefix='pkg.list_pkgs'):
+def _contextkey(jail=None, chroot=None, root=None, prefix='pkg.list_pkgs'):
     '''
     As this module is designed to manipulate packages in jails and chroots, use
     the passed jail/chroot to ensure that a key in the __context__ dict that is
     unique to that jail/chroot is used.
     '''
     if jail:
-        return str(prefix) + '.jail_{0}'.format(jail)
+        return six.text_type(prefix) + '.jail_{0}'.format(jail)
     elif chroot:
-        return str(prefix) + '.chroot_{0}'.format(chroot)
+        return six.text_type(prefix) + '.chroot_{0}'.format(chroot)
+    elif root:
+        return six.text_type(prefix) + '.root_{0}'.format(root)
     return prefix
 
 
@@ -116,8 +156,9 @@ def parse_config(file_name='/usr/local/etc/pkg.conf'):
     if not os.path.isfile(file_name):
         return 'Unable to find {0} on file system'.format(file_name)
 
-    with salt.utils.fopen(file_name) as ifile:
+    with salt.utils.files.fopen(file_name) as ifile:
         for line in ifile:
+            line = salt.utils.stringutils.to_unicode(line)
             if line.startswith('#') or line.startswith('\n'):
                 pass
             else:
@@ -145,6 +186,10 @@ def version(*names, **kwargs):
         Get package version information for the specified chroot (ignored if
         ``jail`` is specified)
 
+    root
+        Get package version information for the specified root (ignored if
+        ``jail`` is specified)
+
     with_origin : False
         Return a nested dictionary containing both the origin name and version
         for each specified package.
@@ -162,7 +207,7 @@ def version(*names, **kwargs):
     '''
     with_origin = kwargs.pop('with_origin', False)
     ret = __salt__['pkg_resource.version'](*names, **kwargs)
-    if not salt.utils.is_true(with_origin):
+    if not salt.utils.data.is_true(with_origin):
         return ret
     # Put the return value back into a dict since we're adding a subdict
     if len(names) == 1:
@@ -174,10 +219,10 @@ def version(*names, **kwargs):
     ])
 
 # Support pkg.info get version info, since this is the CLI usage
-info = version
+info = salt.utils.functools.alias_function(version, 'info')
 
 
-def refresh_db(jail=None, chroot=None, force=False):
+def refresh_db(jail=None, chroot=None, root=None, force=False):
     '''
     Refresh PACKAGESITE contents
 
@@ -199,6 +244,10 @@ def refresh_db(jail=None, chroot=None, force=False):
         Refresh the pkg database within the specified chroot (ignored if
         ``jail`` is specified)
 
+    root
+        Refresh the pkg database within the specified root (ignored if
+        ``jail`` is specified)
+
     force
         Force a full download of the repository catalog without regard to the
         respective ages of the local and remote copies of the catalog.
@@ -209,16 +258,17 @@ def refresh_db(jail=None, chroot=None, force=False):
 
             salt '*' pkg.refresh_db force=True
     '''
-    opts = ''
+    # Remove rtag file to keep multiple refreshes from happening in pkg states
+    salt.utils.pkg.clear_rtag(__opts__)
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('update')
     if force:
-        opts += ' -f'
-    return __salt__['cmd.retcode'](
-        '{0} update{1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False) == 0
+        cmd.append('-f')
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 # Support pkg.update to refresh the db, since this is the CLI usage
-update = refresh_db
+update = salt.utils.functools.alias_function(refresh_db, 'update')
 
 
 def latest_version(*names, **kwargs):
@@ -241,26 +291,49 @@ def latest_version(*names, **kwargs):
     if len(names) == 0:
         return ''
     ret = {}
+
     # Initialize the dict with empty strings
     for name in names:
         ret[name] = ''
     jail = kwargs.get('jail')
     chroot = kwargs.get('chroot')
-    pkgs = list_pkgs(versions_as_list=True, jail=jail, chroot=chroot)
+    refresh = kwargs.get('refresh')
+    root = kwargs.get('root')
+    pkgs = list_pkgs(versions_as_list=True, jail=jail, chroot=chroot, root=root)
 
+    if salt.utils.versions.compare(_get_pkgng_version(jail, chroot, root), '>=', '1.6.0'):
+        quiet = True
+    else:
+        quiet = False
+
+    cmd_prefix = _pkg(jail, chroot, root) + ['search']
     for name in names:
-        cmd = '{0} search {1}'.format(_pkg(jail, chroot), name)
+        # FreeBSD supports packages in format java/openjdk7
+        if '/' in name:
+            cmd = _pkg(jail, chroot, root) + ['search']
+        else:
+            cmd = _pkg(jail, chroot, root) + ['search', '-S', 'name', '-Q', 'version', '-e']
+        if quiet:
+            cmd.append('-q')
+        if not salt.utils.data.is_true(refresh):
+            cmd.append('-U')
+        cmd.append(name)
+
         pkgver = _get_version(
             name,
-            __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
+            sorted(
+                __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace').splitlines(),
+                reverse=True
+            ).pop(0)
         )
+
         if pkgver is not None:
             installed = pkgs.get(name, [])
             if not installed:
                 ret[name] = pkgver
             else:
                 if not any(
-                    (salt.utils.compare_versions(ver1=x,
+                    (salt.utils.versions.compare(ver1=x,
                                                  oper='>=',
                                                  ver2=pkgver)
                      for x in installed)
@@ -274,12 +347,13 @@ def latest_version(*names, **kwargs):
 
 
 # available_version is being deprecated
-available_version = latest_version
+available_version = salt.utils.functools.alias_function(latest_version, 'available_version')
 
 
 def list_pkgs(versions_as_list=False,
               jail=None,
               chroot=None,
+              root=None,
               with_origin=False,
               **kwargs):
     '''
@@ -294,6 +368,10 @@ def list_pkgs(versions_as_list=False,
         List the packages in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        List the packages in the specified root (ignored if ``jail`` is
+        specified)
+
     with_origin : False
         Return a nested dictionary containing both the origin name and version
         for each installed package.
@@ -309,19 +387,19 @@ def list_pkgs(versions_as_list=False,
         salt '*' pkg.list_pkgs chroot=/path/to/chroot
     '''
     # not yet implemented or not applicable
-    if any([salt.utils.is_true(kwargs.get(x))
+    if any([salt.utils.data.is_true(kwargs.get(x))
             for x in ('removed', 'purge_desired')]):
         return {}
 
-    versions_as_list = salt.utils.is_true(versions_as_list)
-    contextkey_pkg = _contextkey(jail, chroot)
-    contextkey_origins = _contextkey(jail, chroot, prefix='pkg.origin')
+    versions_as_list = salt.utils.data.is_true(versions_as_list)
+    contextkey_pkg = _contextkey(jail, chroot, root)
+    contextkey_origins = _contextkey(jail, chroot, root, prefix='pkg.origin')
 
     if contextkey_pkg in __context__:
         ret = copy.deepcopy(__context__[contextkey_pkg])
         if not versions_as_list:
             __salt__['pkg_resource.stringify'](ret)
-        if salt.utils.is_true(with_origin):
+        if salt.utils.data.is_true(with_origin):
             origins = __context__.get(contextkey_origins, {})
             return dict([
                 (x, {'origin': origins.get(x, ''), 'version': y})
@@ -331,12 +409,11 @@ def list_pkgs(versions_as_list=False,
 
     ret = {}
     origins = {}
-    cmd = '{0} info -ao'.format(_pkg(jail, chroot))
     out = __salt__['cmd.run_stdout'](
-            cmd,
-            python_shell=False,
-            output_loglevel='trace')
-    for line in out.splitlines():
+        _pkg(jail, chroot, root) + ['info', '-ao'],
+        output_loglevel='trace',
+        python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if not line:
             continue
         try:
@@ -352,7 +429,7 @@ def list_pkgs(versions_as_list=False,
     __context__[contextkey_origins] = origins
     if not versions_as_list:
         __salt__['pkg_resource.stringify'](ret)
-    if salt.utils.is_true(with_origin):
+    if salt.utils.data.is_true(with_origin):
         return dict([
             (x, {'origin': origins.get(x, ''), 'version': y})
             for x, y in six.iteritems(ret)
@@ -381,7 +458,7 @@ def update_package_site(new_url):
     return True
 
 
-def stats(local=False, remote=False, jail=None, chroot=None):
+def stats(local=False, remote=False, jail=None, chroot=None, root=None):
     '''
     Return pkgng stats.
 
@@ -424,6 +501,10 @@ def stats(local=False, remote=False, jail=None, chroot=None):
         Retrieve stats from the specified chroot (ignored if ``jail`` is
         specified).
 
+    root
+        Retrieve stats from the specified root (ignored if ``jail`` is
+        specified).
+
         CLI Example:
 
         .. code-block:: bash
@@ -438,19 +519,16 @@ def stats(local=False, remote=False, jail=None, chroot=None):
         opts += 'l'
     if remote:
         opts += 'r'
+
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('stats')
     if opts:
-        opts = '-' + opts
-
-    res = __salt__['cmd.run'](
-        '{0} stats {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
-    )
-    res = [x.strip("\t") for x in res.split("\n")]
-    return res
+        cmd.append('-' + opts)
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    return [x.strip('\t') for x in salt.utils.itertools.split(out, '\n')]
 
 
-def backup(file_name, jail=None, chroot=None):
+def backup(file_name, jail=None, chroot=None, root=None):
     '''
     Export installed packages into yaml+mtree file
 
@@ -477,21 +555,27 @@ def backup(file_name, jail=None, chroot=None):
         so the path to the backup file will be relative to the root of the
         chroot.
 
+    root
+        Backup packages from the specified root (ignored if ``jail`` is
+        specified). Note that this will run the command within the root, and
+        so the path to the backup file will be relative to the root of the
+        root.
+
         CLI Example:
 
         .. code-block:: bash
 
             salt '*' pkg.backup /tmp/pkg chroot=/path/to/chroot
     '''
-    res = __salt__['cmd.run'](
-        '{0} backup -d {1!r}'.format(_pkg(jail, chroot), file_name),
-        python_shell=False,
-        output_loglevel='trace'
+    ret = __salt__['cmd.run'](
+        _pkg(jail, chroot, root) + ['backup', '-d', file_name],
+        output_loglevel='trace',
+        python_shell=False
     )
-    return res.split('...')[1]
+    return ret.split('...')[1]
 
 
-def restore(file_name, jail=None, chroot=None):
+def restore(file_name, jail=None, chroot=None, root=None):
     '''
     Reads archive created by pkg backup -d and recreates the database.
 
@@ -518,6 +602,12 @@ def restore(file_name, jail=None, chroot=None):
         so the path to the file from which the pkg database will be restored is
         relative to the root of the chroot.
 
+    root
+        Restore database to the specified root (ignored if ``jail`` is
+        specified). Note that this will run the command within the root, and
+        so the path to the file from which the pkg database will be restored is
+        relative to the root of the root.
+
         CLI Example:
 
         .. code-block:: bash
@@ -525,13 +615,13 @@ def restore(file_name, jail=None, chroot=None):
             salt '*' pkg.restore /tmp/pkg chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} backup -r {1!r}'.format(_pkg(jail, chroot), file_name),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot, root) + ['backup', '-r', file_name],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
-def audit(jail=None, chroot=None):
+def audit(jail=None, chroot=None, root=None):
     '''
     Audits installed packages against known vulnerabilities
 
@@ -554,6 +644,10 @@ def audit(jail=None, chroot=None):
         Audit packages within the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Audit packages within the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -561,9 +655,9 @@ def audit(jail=None, chroot=None):
             salt '*' pkg.audit chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} audit -F'.format(_pkg(jail, chroot)),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot, root) + ['audit', '-F'],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -573,6 +667,7 @@ def install(name=None,
             sources=None,
             jail=None,
             chroot=None,
+            root=None,
             orphan=False,
             force=False,
             glob=False,
@@ -582,6 +677,7 @@ def install(name=None,
             reinstall_requires=False,
             regex=False,
             pcre=False,
+            batch=False,
             **kwargs):
     '''
     Install package(s) from a repository
@@ -602,6 +698,10 @@ def install(name=None,
         Install the package into the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Install the package into the specified root (ignored if ``jail`` is
+        specified)
+
     orphan
         Mark the installed package as orphan. Will be automatically removed
         if no other packages depend on them. For more information please
@@ -701,7 +801,16 @@ def install(name=None,
 
         .. code-block:: bash
 
-            salt '*' pkg.install <extended regular expression> pcre=True
+
+    batch
+        Use BATCH=true for pkg install, skipping all questions.
+        Be careful when using in production.
+
+        CLI Example:
+
+        .. code-block:: bash
+
+            salt '*' pkg.install <package name> batch=True
     '''
     try:
         pkg_params, pkg_type = __salt__['pkg_resource.parse_targets'](
@@ -713,40 +822,39 @@ def install(name=None,
     if pkg_params is None or len(pkg_params) == 0:
         return {}
 
-    opts = ''
-    repo_opts = ''
-    if salt.utils.is_true(orphan):
+    env = {}
+    opts = 'y'
+    if salt.utils.data.is_true(orphan):
         opts += 'A'
-    if salt.utils.is_true(force):
+    if salt.utils.data.is_true(force):
         opts += 'f'
-    if salt.utils.is_true(glob):
+    if salt.utils.data.is_true(glob):
         opts += 'g'
-    if salt.utils.is_true(local):
+    if salt.utils.data.is_true(local):
         opts += 'U'
-    if salt.utils.is_true(dryrun):
+    if salt.utils.data.is_true(dryrun):
         opts += 'n'
-    if not salt.utils.is_true(dryrun):
-        opts += 'y'
-    if salt.utils.is_true(quiet):
+    if salt.utils.data.is_true(quiet):
         opts += 'q'
-    if salt.utils.is_true(reinstall_requires):
+    if salt.utils.data.is_true(reinstall_requires):
         opts += 'R'
-    if fromrepo:
-        repo_opts += 'r {0}'.format(fromrepo)
-    if salt.utils.is_true(regex):
+    if salt.utils.data.is_true(regex):
         opts += 'x'
-    if salt.utils.is_true(pcre):
+    if salt.utils.data.is_true(pcre):
         opts += 'X'
-    if opts:
-        opts = '-' + opts
-    if repo_opts:
-        repo_opts = '-' + repo_opts
+    if salt.utils.data.is_true(batch):
+        env = {
+                "BATCH": "true",
+                "ASSUME_ALWAYS_YES": "YES"
+              }
 
-    old = list_pkgs(jail=jail, chroot=chroot)
+    old = list_pkgs(jail=jail, chroot=chroot, root=root)
 
     if pkg_type == 'file':
         pkg_cmd = 'add'
-        targets = list(pkg_params.keys())
+        # pkg add has smaller set of options (i.e. no -y or -n), filter below
+        opts = ''.join([opt for opt in opts if opt in 'AfIMq'])
+        targets = pkg_params
     elif pkg_type == 'repository':
         pkg_cmd = 'install'
         if pkgs is None and kwargs.get('version') and len(pkg_params) == 1:
@@ -760,20 +868,49 @@ def install(name=None,
             else:
                 targets.append('{0}-{1}'.format(param, version_num))
 
-    cmd = '{0} {1} {2} {3} {4}'.format(
-        _pkg(jail, chroot), pkg_cmd, repo_opts, opts, ' '.join(targets)
+    cmd = _pkg(jail, chroot, root)
+    cmd.append(pkg_cmd)
+    if fromrepo:
+        cmd.extend(['-r', fromrepo])
+    if opts:
+        cmd.append('-' + opts)
+    cmd.extend(targets)
+
+    if pkg_cmd == 'add' and salt.utils.data.is_true(dryrun):
+        # pkg add doesn't have a dryrun mode, so echo out what will be run
+        return ' '.join(cmd)
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False,
+        env=env
     )
-    __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
-    __context__.pop(_contextkey(jail, chroot), None)
-    __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
-    new = list_pkgs(jail=jail, chroot=chroot)
-    return salt.utils.compare_dicts(old, new)
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
+    new = list_pkgs(jail=jail, chroot=chroot, root=root)
+    ret = salt.utils.data.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def remove(name=None,
            pkgs=None,
            jail=None,
            chroot=None,
+           root=None,
            all_installed=False,
            force=False,
            glob=False,
@@ -806,6 +943,10 @@ def remove(name=None,
         Delete the package from the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Delete the package from the specified root (ignored if ``jail`` is
+        specified)
+
     all_installed
         Deletes all installed packages from the system and empties the
         database. USE WITH CAUTION!
@@ -877,50 +1018,86 @@ def remove(name=None,
     except MinionError as exc:
         raise CommandExecutionError(exc)
 
-    old = list_pkgs(jail=jail, chroot=chroot)
-    targets = [x for x in pkg_params if x in old]
+    targets = []
+    old = list_pkgs(jail=jail, chroot=chroot, root=root, with_origin=True)
+    for pkg in pkg_params.items():
+        # FreeBSD pkg supports `openjdk` and `java/openjdk7` package names
+        if pkg[0].find("/") > 0:
+            origin = pkg[0]
+            pkg = [k for k, v in six.iteritems(old) if v['origin'] == origin][0]
+
+        if pkg[0] in old:
+            targets.append(pkg[0])
+
     if not targets:
         return {}
 
     opts = ''
-    if salt.utils.is_true(all_installed):
+    if salt.utils.data.is_true(all_installed):
         opts += 'a'
-    if salt.utils.is_true(force):
+    if salt.utils.data.is_true(force):
         opts += 'f'
-    if salt.utils.is_true(glob):
+    if salt.utils.data.is_true(glob):
         opts += 'g'
-    if salt.utils.is_true(dryrun):
+    if salt.utils.data.is_true(dryrun):
         opts += 'n'
-    if not salt.utils.is_true(dryrun):
+    if not salt.utils.data.is_true(dryrun):
         opts += 'y'
-    if salt.utils.is_true(recurse):
+    if salt.utils.data.is_true(recurse):
         opts += 'R'
-    if salt.utils.is_true(regex):
+    if salt.utils.data.is_true(regex):
         opts += 'x'
-    if salt.utils.is_true(pcre):
+    if salt.utils.data.is_true(pcre):
         opts += 'X'
-    if opts:
-        opts = '-' + opts
 
-    cmd = '{0} delete {1} {2}'.format(
-        _pkg(jail, chroot), opts, ' '.join(targets)
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('delete')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.extend(targets)
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
-    __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
-    __context__.pop(_contextkey(jail, chroot), None)
-    __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
-    new = list_pkgs(jail=jail, chroot=chroot)
-    return salt.utils.compare_dicts(old, new)
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
+    new = list_pkgs(jail=jail, chroot=chroot, root=root, with_origin=True)
+    ret = salt.utils.data.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 # Support pkg.delete to remove packages, since this is the CLI usage
-delete = remove
+delete = salt.utils.functools.alias_function(remove, 'delete')
 # No equivalent to purge packages, use remove instead
-purge = remove
+purge = salt.utils.functools.alias_function(remove, 'purge')
 
 
 def upgrade(*names, **kwargs):
     '''
     Upgrade named or all packages (run a ``pkg upgrade``). If <package name> is
-    ommitted, the operation is executed on all packages.
+    omitted, the operation is executed on all packages.
+
+    Returns a dictionary containing the changes:
+
+    .. code-block:: python
+
+        {'<package>':  {'old': '<old-version>',
+                        'new': '<new-version>'}}
+
 
     CLI Example:
 
@@ -941,6 +1118,10 @@ def upgrade(*names, **kwargs):
         Audit packages within the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Audit packages within the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -981,13 +1162,9 @@ def upgrade(*names, **kwargs):
 
             salt '*' pkg.upgrade <package name> dryrun=True
     '''
-    ret = {'changes': {},
-           'result': True,
-           'comment': '',
-           }
-
     jail = kwargs.pop('jail', None)
     chroot = kwargs.pop('chroot', None)
+    root = kwargs.pop('root', None)
     force = kwargs.pop('force', False)
     local = kwargs.pop('local', False)
     dryrun = kwargs.pop('dryrun', False)
@@ -1000,29 +1177,32 @@ def upgrade(*names, **kwargs):
         opts += 'n'
     if not dryrun:
         opts += 'y'
+
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('upgrade')
     if opts:
-        opts = '-' + opts
+        cmd.append('-' + opts)
+    cmd.extend(names)
 
     old = list_pkgs()
-    call = __salt__['cmd.run_all'](
-        '{0} upgrade {1} {2}'.format(_pkg(jail, chroot), opts, ' '.join(names)),
-        python_shell=False,
-        output_loglevel='trace'
-    )
-    if call['retcode'] != 0:
-        ret['result'] = False
-        if 'stderr' in call:
-            ret['comment'] += call['stderr']
-        if 'stdout' in call:
-            ret['comment'] += call['stdout']
-    else:
-        __context__.pop('pkg.list_pkgs', None)
-        new = list_pkgs()
-        ret['changes'] = salt.utils.compare_dicts(old, new)
+    result = __salt__['cmd.run_all'](cmd,
+                                     output_loglevel='trace',
+                                     python_shell=False)
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
+    new = list_pkgs()
+    ret = salt.utils.data.compare_dicts(old, new)
+
+    if result['retcode'] != 0:
+        raise CommandExecutionError(
+            'Problem encountered upgrading packages',
+            info={'changes': ret, 'result': result}
+        )
+
     return ret
 
 
-def clean(jail=None, chroot=None):
+def clean(jail=None, chroot=None, root=None):
     '''
     Cleans the local cache of fetched remote packages
 
@@ -1035,13 +1215,13 @@ def clean(jail=None, chroot=None):
         salt '*' pkg.clean chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} clean'.format(_pkg(jail, chroot)),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot, root) + ['clean'],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
-def autoremove(jail=None, chroot=None, dryrun=False):
+def autoremove(jail=None, chroot=None, root=None, dryrun=False):
     '''
     Delete packages which were automatically installed as dependencies and are
     not required anymore.
@@ -1064,17 +1244,21 @@ def autoremove(jail=None, chroot=None, dryrun=False):
         opts += 'n'
     else:
         opts += 'y'
+
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('autoremove')
     if opts:
-        opts = '-' + opts
+        cmd.append('-' + opts)
     return __salt__['cmd.run'](
-        '{0} autoremove {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
 def check(jail=None,
           chroot=None,
+          root=None,
           depends=False,
           recompute=False,
           checksum=False):
@@ -1094,6 +1278,10 @@ def check(jail=None,
         Perform the sanity check in the specified chroot (ignored if ``jail``
         is specified)
 
+    root
+        Perform the sanity check in the specified root (ignored if ``jail``
+        is specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1140,17 +1328,19 @@ def check(jail=None,
         opts += 'r'
     if checksum:
         opts += 's'
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('check')
+    if opts:
+        cmd.append('-' + opts)
     return __salt__['cmd.run'](
-        '{0} check {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
-def which(path, jail=None, chroot=None, origin=False, quiet=False):
+def which(path, jail=None, chroot=None, root=None, origin=False, quiet=False):
     '''
     Displays which package installed a specific file
 
@@ -1173,6 +1363,10 @@ def which(path, jail=None, chroot=None, origin=False, quiet=False):
         Perform the check in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Perform the check in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1203,18 +1397,23 @@ def which(path, jail=None, chroot=None, origin=False, quiet=False):
         opts += 'q'
     if origin:
         opts += 'o'
+
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('which')
     if opts:
-        opts = '-' + opts
+        cmd.append('-' + opts)
+    cmd.append(path)
     return __salt__['cmd.run'](
-        '{0} which {1} {2}'.format(_pkg(jail, chroot), opts, path),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
 def search(name,
            jail=None,
            chroot=None,
+           root=None,
            exact=False,
            glob=False,
            regex=False,
@@ -1249,6 +1448,10 @@ def search(name,
         Perform the search using the ``pkg.conf(5)`` from the specified chroot
         (ignored if ``jail`` is specified)
 
+    root
+        Perform the search using the ``pkg.conf(5)`` from the specified root
+        (ignored if ``jail`` is specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1390,19 +1593,23 @@ def search(name,
         opts += 'o'
     if prefix:
         opts += 'p'
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('search')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} search {1} {2}'.format(_pkg(jail, chroot), opts, name),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
 def fetch(name,
           jail=None,
           chroot=None,
+          root=None,
           fetch_all=False,
           quiet=False,
           fromrepo=None,
@@ -1433,6 +1640,10 @@ def fetch(name,
         Fetch package in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Fetch package in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1514,13 +1725,10 @@ def fetch(name,
             salt '*' pkg.fetch <package name> depends=True
     '''
     opts = ''
-    repo_opts = ''
     if fetch_all:
         opts += 'a'
     if quiet:
         opts += 'q'
-    if fromrepo:
-        repo_opts += 'r {0}'.format(fromrepo)
     if glob:
         opts += 'g'
     if regex:
@@ -1531,23 +1739,25 @@ def fetch(name,
         opts += 'L'
     if depends:
         opts += 'd'
-    if opts:
-        opts = '-' + opts
-    if repo_opts:
-        repo_opts = '-' + repo_opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.extend(['fetch', '-y'])
+    if fromrepo:
+        cmd.extend(['-r', fromrepo])
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} fetch -y {1} {2} {3}'.format(
-            _pkg(jail, chroot), opts, repo_opts, name
-        ),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
 def updating(name,
              jail=None,
              chroot=None,
+             root=None,
              filedate=None,
              filename=None):
     ''''
@@ -1572,6 +1782,10 @@ def updating(name,
         Perform the action in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Perform the action in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1602,11 +1816,46 @@ def updating(name,
         opts += 'd {0}'.format(filedate)
     if filename:
         opts += 'f {0}'.format(filename)
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('updating')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} updating {1} {2}'.format(_pkg(jail, chroot), opts, name),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
+
+
+def version_cmp(pkg1, pkg2, ignore_epoch=False):
+    '''
+    Do a cmp-style comparison on two packages. Return -1 if pkg1 < pkg2, 0 if
+    pkg1 == pkg2, and 1 if pkg1 > pkg2. Return None if there was a problem
+    making the comparison.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.version_cmp '2.1.11' '2.1.12'
+    '''
+    # Don't worry about ignore_epoch since we're shelling out to pkg.
+    sym = {
+        '<': -1,
+        '>': 1,
+        '=': 0,
+    }
+    try:
+        cmd = ['pkg', 'version', '--test-version', pkg1, pkg2]
+        ret = __salt__['cmd.run_all'](cmd,
+                                            output_loglevel='trace',
+                                            python_shell=False,
+                                            ignore_retcode=True)
+    except Exception as exc:
+        log.error(exc)
+
+    if ret['stdout'] in sym:
+        return sym[ret['stdout']]
+    return None
