@@ -2,13 +2,14 @@
 '''
 Module for managing quotas on POSIX-like systems.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import logging
 
 # Import salt libs
-import salt.utils
+import salt.utils.path
+import salt.utils.platform
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 
 log = logging.getLogger(__name__)
@@ -24,9 +25,14 @@ def __virtual__():
     '''
     Only work on POSIX-like systems with setquota binary available
     '''
-    if not salt.utils.is_windows() and salt.utils.which('setquota'):
+    if not salt.utils.platform.is_windows() \
+            and salt.utils.path.which('setquota'):
         return 'quota'
-    return False
+    return (
+        False,
+        'The quota execution module cannot be loaded: the module is only '
+        'available on POSIX-like systems with the setquota binary available.'
+    )
 
 
 def report(mount):
@@ -64,9 +70,7 @@ def _parse_quota(mount, opts):
             continue
         comps = line.split()
         if mode == 'header':
-            if 'Report for' in line:
-                pass
-            elif 'Block grace time' in line:
+            if 'Block grace time' in line:
                 blockg, inodeg = line.split(';')
                 blockgc = blockg.split(': ')
                 inodegc = inodeg.split(': ')
