@@ -11,7 +11,7 @@ This module implements the pkgbuild interface
 '''
 
 # Import python libs
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, print_function, unicode_literals
 import errno
 import logging
 import os
@@ -23,9 +23,15 @@ import traceback
 import functools
 
 # Import salt libs
-from salt.ext.six.moves.urllib.parse import urlparse as _urlparse  # pylint: disable=no-name-in-module,import-error
 from salt.exceptions import SaltInvocationError
-import salt.utils
+import salt.utils.files
+import salt.utils.path
+import salt.utils.user
+import salt.utils.vt
+
+# Import 3rd-party libs
+from salt.ext import six
+from salt.ext.six.moves.urllib.parse import urlparse as _urlparse  # pylint: disable=no-name-in-module,import-error
 
 HAS_LIBS = False
 
@@ -48,7 +54,7 @@ def __virtual__():
     missing_util = False
     utils_reqd = ['gpg', 'rpm', 'rpmbuild', 'mock', 'createrepo']
     for named_util in utils_reqd:
-        if not salt.utils.which(named_util):
+        if not salt.utils.path.which(named_util):
             missing_util = True
             break
 
@@ -59,32 +65,35 @@ def __virtual__():
             # The module will be exposed as `rpmbuild` on non-RPM based systems
             return 'rpmbuild'
     else:
-        return False, 'The rpmbuild module could not be loaded: requires python-gnupg, gpg, rpm, rpmbuild, mock and createrepo utilities to be installed'
+        return False, 'The rpmbuild module could not be loaded: requires python-gnupg, ' \
+                      'gpg, rpm, rpmbuild, mock and createrepo utilities to be installed'
 
 
-def _create_rpmmacros():
+def _create_rpmmacros(runas='root'):
     '''
     Create the .rpmmacros file in user's home directory
     '''
     home = os.path.expanduser('~')
     rpmbuilddir = os.path.join(home, 'rpmbuild')
     if not os.path.isdir(rpmbuilddir):
-        os.makedirs(rpmbuilddir)
+        __salt__['file.makedirs_perms'](name=rpmbuilddir, user=runas, group='mock')
 
     mockdir = os.path.join(home, 'mock')
     if not os.path.isdir(mockdir):
-        os.makedirs(mockdir)
+        __salt__['file.makedirs_perms'](name=mockdir, user=runas, group='mock')
 
     rpmmacros = os.path.join(home, '.rpmmacros')
-    with salt.utils.fopen(rpmmacros, 'w') as afile:
-        afile.write('%_topdir {0}\n'.format(rpmbuilddir))
+    with salt.utils.files.fopen(rpmmacros, 'w') as afile:
+        afile.write(
+            salt.utils.stringutils.to_str('%_topdir {0}\n'.format(rpmbuilddir))
+        )
         afile.write('%signature gpg\n')
         afile.write('%_source_filedigest_algorithm 8\n')
         afile.write('%_binary_filedigest_algorithm 8\n')
         afile.write('%_gpg_name packaging@saltstack.com\n')
 
 
-def _mk_tree():
+def _mk_tree(runas='root'):
     '''
     Create the rpm build tree
     '''
@@ -92,7 +101,7 @@ def _mk_tree():
     paths = ['BUILD', 'RPMS', 'SOURCES', 'SPECS', 'SRPMS']
     for path in paths:
         full = os.path.join(basedir, path)
-        os.makedirs(full)
+        __salt__['file.makedirs_perms'](name=full, user=runas, group='mock')
     return basedir
 
 
@@ -108,7 +117,7 @@ def _get_spec(tree_base, spec, template, saltenv='base'):
         saltenv=saltenv)
 
 
-def _get_src(tree_base, source, saltenv='base'):
+def _get_src(tree_base, source, saltenv='base', runas='root'):
     '''
     Get the named sources and place them into the tree_base
     '''
@@ -119,6 +128,7 @@ def _get_src(tree_base, source, saltenv='base'):
         lsrc = __salt__['cp.get_url'](source, dest, saltenv=saltenv)
     else:
         shutil.copy(source, dest)
+    __salt__['file.chown'](path=dest, user=runas, group='mock')
 
 
 def _get_distset(tgt):
@@ -163,7 +173,7 @@ def _get_deps(deps, tree_base, saltenv='base'):
     return deps_list
 
 
-def make_src_pkg(dest_dir, spec, sources, env=None, template=None, saltenv='base'):
+def make_src_pkg(dest_dir, spec, sources, env=None, template=None, saltenv='base', runas='root'):
     '''
     Create a source rpm from the given spec file and sources
 
@@ -171,33 +181,74 @@ def make_src_pkg(dest_dir, spec, sources, env=None, template=None, saltenv='base
 
     .. code-block:: bash
 
-        salt '*' pkgbuild.make_src_pkg /var/www/html/ https://raw.githubusercontent.com/saltstack/libnacl/master/pkg/rpm/python-libnacl.spec https://pypi.python.org/packages/source/l/libnacl/libnacl-1.3.5.tar.gz
+        salt '*' pkgbuild.make_src_pkg /var/www/html/
+                https://raw.githubusercontent.com/saltstack/libnacl/master/pkg/rpm/python-libnacl.spec
+                https://pypi.python.org/packages/source/l/libnacl/libnacl-1.3.5.tar.gz
 
     This example command should build the libnacl SOURCE package and place it in
     /var/www/html/ on the minion
 
     .. versionchanged:: 2017.7.0
 
+    dest_dir
+        The directory on the minion to place the built package(s)
+
+    spec
+        The location of the spec file (used for rpms)
+
+    sources
+        The list of package sources
+
+    env
+        A dictionary of environment variables to be set prior to execution.
+
+    template
+        Run the spec file through a templating engine
+        Optional arguement, allows for no templating engine used to be
+        if none is desired.
+
+    saltenv
+        The saltenv to use for files downloaded from the salt filesever
+
+    runas
+        The user to run the build process as
+
+        .. versionadded:: 2018.3.3
+
+
     .. note::
 
         using SHA256 as digest and minimum level dist el6
 
     '''
-    _create_rpmmacros()
-    tree_base = _mk_tree()
+    _create_rpmmacros(runas)
+    tree_base = _mk_tree(runas)
     spec_path = _get_spec(tree_base, spec, template, saltenv)
-    if isinstance(sources, str):
+    __salt__['file.chown'](path=spec_path, user=runas, group='mock')
+    __salt__['file.chown'](path=tree_base, user=runas, group='mock')
+
+    if isinstance(sources, six.string_types):
         sources = sources.split(',')
     for src in sources:
-        _get_src(tree_base, src, saltenv)
+        _get_src(tree_base, src, saltenv, runas)
 
     # make source rpms for dist el6 with SHA256, usable with mock on other dists
     cmd = 'rpmbuild --verbose --define "_topdir {0}" -bs --define "dist .el6" {1}'.format(tree_base, spec_path)
-    __salt__['cmd.run'](cmd)
+    retrc = __salt__['cmd.retcode'](cmd, runas=runas)
+    if retrc != 0:
+        raise SaltInvocationError(
+             'Make source package for destination directory {0}, spec {1}, sources {2}, failed '
+             'with return error {3}, check logs for further details'.format(
+                dest_dir,
+                spec,
+                sources,
+                retrc)
+        )
+
     srpms = os.path.join(tree_base, 'SRPMS')
     ret = []
     if not os.path.isdir(dest_dir):
-        os.makedirs(dest_dir)
+        __salt__['file.makedirs_perms'](name=dest_dir, user=runas, group='mock')
     for fn_ in os.listdir(srpms):
         full = os.path.join(srpms, fn_)
         tgt = os.path.join(dest_dir, fn_)
@@ -224,14 +275,16 @@ def build(runas,
 
     .. code-block:: bash
 
-        salt '*' pkgbuild.build mock epel-7-x86_64 /var/www/html https://raw.githubusercontent.com/saltstack/libnacl/master/pkg/rpm/python-libnacl.spec https://pypi.python.org/packages/source/l/libnacl/libnacl-1.3.5.tar.gz
+        salt '*' pkgbuild.build mock epel-7-x86_64 /var/www/html
+                    https://raw.githubusercontent.com/saltstack/libnacl/master/pkg/rpm/python-libnacl.spec
+                    https://pypi.python.org/packages/source/l/libnacl/libnacl-1.3.5.tar.gz
 
     This example command should build the libnacl package for rhel 7 using user
     mock and place it in /var/www/html/ on the minion
     '''
     ret = {}
     try:
-        os.makedirs(dest_dir)
+        __salt__['file.chown'](path=dest_dir, user=runas, group='mock')
     except OSError as exc:
         if exc.errno != errno.EEXIST:
             raise
@@ -239,7 +292,7 @@ def build(runas,
     srpm_build_dir = tempfile.mkdtemp()
     try:
         srpms = make_src_pkg(srpm_build_dir, spec, sources,
-                             env, template, saltenv)
+                             env, template, saltenv, runas)
     except Exception as exc:
         shutil.rmtree(srpm_build_dir)
         log.error('Failed to make src package')
@@ -251,17 +304,18 @@ def build(runas,
     deps_dir = tempfile.mkdtemp()
     deps_list = _get_deps(deps, deps_dir, saltenv)
 
+    retrc = 0
     for srpm in srpms:
         dbase = os.path.dirname(srpm)
         results_dir = tempfile.mkdtemp()
         try:
-            __salt__['cmd.run']('chown {0} -R {1}'.format(runas, dbase))
-            __salt__['cmd.run']('chown {0} -R {1}'.format(runas, results_dir))
+            __salt__['file.chown'](path=dbase, user=runas, group='mock')
+            __salt__['file.chown'](path=results_dir, user=runas, group='mock')
             cmd = 'mock --root={0} --resultdir={1} --init'.format(tgt, results_dir)
-            __salt__['cmd.run'](cmd, runas=runas)
+            retrc |= __salt__['cmd.retcode'](cmd, runas=runas)
             if deps_list and not deps_list.isspace():
                 cmd = 'mock --root={0} --resultdir={1} --install {2} {3}'.format(tgt, results_dir, deps_list, noclean)
-                __salt__['cmd.run'](cmd, runas=runas)
+                retrc |= __salt__['cmd.retcode'](cmd, runas=runas)
                 noclean += ' --no-clean'
 
             cmd = 'mock --root={0} --resultdir={1} {2} {3} {4}'.format(
@@ -270,17 +324,20 @@ def build(runas,
                 distset,
                 noclean,
                 srpm)
-            __salt__['cmd.run'](cmd, runas=runas)
-            cmd = ['rpm', '-qp', '--queryformat',
-                   '{0}/%{{name}}/%{{version}}-%{{release}}'.format(log_dir),
-                   srpm]
-            log_dest = __salt__['cmd.run_stdout'](cmd, python_shell=False)
+            retrc |= __salt__['cmd.retcode'](cmd, runas=runas)
+            cmdlist = [
+                        'rpm',
+                        '-qp',
+                        '--queryformat',
+                        '{0}/%{{name}}/%{{version}}-%{{release}}'.format(log_dir),
+                        srpm]
+            log_dest = __salt__['cmd.run_stdout'](cmdlist, python_shell=False)
             for filename in os.listdir(results_dir):
                 full = os.path.join(results_dir, filename)
                 if filename.endswith('src.rpm'):
                     sdest = os.path.join(srpm_dir, filename)
                     try:
-                        os.makedirs(srpm_dir)
+                        __salt__['file.makedirs_perms'](name=srpm_dir, user=runas, group='mock')
                     except OSError as exc:
                         if exc.errno != errno.EEXIST:
                             raise
@@ -293,16 +350,25 @@ def build(runas,
                 else:
                     log_file = os.path.join(log_dest, filename)
                     try:
-                        os.makedirs(log_dest)
+                        __salt__['file.makedirs_perms'](name=log_dest, user=runas, group='mock')
                     except OSError as exc:
                         if exc.errno != errno.EEXIST:
                             raise
                     shutil.copy(full, log_file)
                     ret.setdefault('Log Files', []).append(log_file)
         except Exception as exc:
-            log.error('Error building from {0}: {1}'.format(srpm, exc))
+            log.error('Error building from %s: %s', srpm, exc)
         finally:
             shutil.rmtree(results_dir)
+    if retrc != 0:
+        raise SaltInvocationError(
+             'Building packages for destination directory {0}, spec {1}, sources {2}, failed '
+             'with return error {3}, check logs for further details'.format(
+                dest_dir,
+                spec,
+                sources,
+                retrc)
+        )
     shutil.rmtree(deps_dir)
     shutil.rmtree(srpm_build_dir)
     return ret
@@ -425,7 +491,7 @@ def make_repo(repodir,
     phrase = ''
 
     if keyid is not None:
-        ## import_keys
+        # import_keys
         pkg_pub_key_file = '{0}/{1}'.format(gnupghome, __salt__['pillar.get']('gpg_pkg_pub_keyname', None))
         pkg_priv_key_file = '{0}/{1}'.format(gnupghome, __salt__['pillar.get']('gpg_pkg_priv_keyname', None))
 
@@ -469,19 +535,26 @@ def make_repo(repodir,
 
         # need to update rpm with public key
         cmd = 'rpm --import {0}'.format(pkg_pub_key_file)
-        __salt__['cmd.run'](cmd, runas=runas, use_vt=True)
+        retrc = __salt__['cmd.retcode'](cmd, runas=runas, use_vt=True)
+        if retrc != 0:
+            raise SaltInvocationError(
+                 'Failed to import public key from file {0} with return '
+                 'error {1}, check logs for further details'.format(
+                    pkg_pub_key_file,
+                    retrc)
+            )
 
-        ## sign_it_here
+        # sign_it_here
         # interval of 0.125 is really too fast on some systems
         interval = 0.5
-        for file in os.listdir(repodir):
-            if file.endswith('.rpm'):
-                abs_file = os.path.join(repodir, file)
+        for fileused in os.listdir(repodir):
+            if fileused.endswith('.rpm'):
+                abs_file = os.path.join(repodir, fileused)
                 number_retries = timeout / interval
                 times_looped = 0
                 error_msg = 'Failed to sign file {0}'.format(abs_file)
                 cmd = 'rpm {0} --addsign {1}'.format(define_gpg_name, abs_file)
-                preexec_fn = functools.partial(salt.utils.chugid_and_umask, runas, None)
+                preexec_fn = functools.partial(salt.utils.user.chugid_and_umask, runas, None)
                 try:
                     stdout, stderr = None, None
                     proc = salt.utils.vt.Terminal(
