@@ -36,14 +36,16 @@ Or you can override it globally by setting the :conf_minion:`providers` paramete
 
 '''
 # Import python libs
-from __future__ import print_function
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import copy
 import logging
 
 
 # Import salt libs
-import salt.utils
+import salt.utils.data
+import salt.utils.functools
+import salt.utils.path
+import salt.utils.pkg
 from salt.exceptions import CommandExecutionError
 
 # Define the module's virtual name
@@ -55,9 +57,9 @@ def __virtual__():
     '''
     Set the virtual pkg module if the os is Solaris 11
     '''
-    if __grains__['os'] == 'Solaris' \
+    if __grains__['os_family'] == 'Solaris' \
             and float(__grains__['kernelrelease']) > 5.10 \
-            and salt.utils.which('pkg'):
+            and salt.utils.path.which('pkg'):
         return __virtualname__
     return (False,
             'The solarisips execution module failed to load: only available '
@@ -118,6 +120,8 @@ def refresh_db(full=False):
         salt '*' pkg.refresh_db
         salt '*' pkg.refresh_db full=True
     '''
+    # Remove rtag file to keep multiple refreshes from happening in pkg states
+    salt.utils.pkg.clear_rtag(__opts__)
     if full:
         return __salt__['cmd.retcode']('/bin/pkg refresh --full') == 0
     else:
@@ -136,7 +140,7 @@ def upgrade_available(name):
         salt '*' pkg.upgrade_available apache-22
     '''
     version = None
-    cmd = 'pkg list -Huv {0}'.format(name)
+    cmd = ['pkg', 'list', '-Huv', name]
     lines = __salt__['cmd.run_stdout'](cmd).splitlines()
     if not lines:
         return {}
@@ -146,28 +150,38 @@ def upgrade_available(name):
     return ret
 
 
-def list_upgrades(refresh=False):
+def list_upgrades(refresh=True, **kwargs):  # pylint: disable=W0613
     '''
     Lists all packages available for update.
-    When run in global zone, it reports only upgradable packages for the global zone.
-    When run in non-global zone, it can report more upgradable packages than
-    "pkg update -vn" because "pkg update" hides packages that require newer
-    version of pkg://solaris/entire (which means that they can be upgraded only
-    from global zone). Simply said: if you see pkg://solaris/entire in the list
-    of upgrades, you should upgrade the global zone to get all possible
-    updates.
 
-    refresh : False
-        Set to ``True`` to force a full pkg DB refresh before listing
+    When run in global zone, it reports only upgradable packages for the global
+    zone.
+
+    When run in non-global zone, it can report more upgradable packages than
+    ``pkg update -vn``, because ``pkg update`` hides packages that require
+    newer version of ``pkg://solaris/entire`` (which means that they can be
+    upgraded only from the global zone). If ``pkg://solaris/entire`` is found
+    in the list of upgrades, then the global zone should be updated to get all
+    possible updates. Use ``refresh=True`` to refresh the package database.
+
+    refresh : True
+        Runs a full package database refresh before listing. Set to ``False`` to
+        disable running the refresh.
+
+        .. versionchanged:: 2017.7.0
+
+        In previous versions of Salt, ``refresh`` defaulted to ``False``. This was
+        changed to default to ``True`` in the 2017.7.0 release to make the behavior
+        more consistent with the other package modules, which all default to ``True``.
 
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' pkg.list_upgrades
-        salt '*' pkg.list_upgrades refresh=True
+        salt '*' pkg.list_upgrades refresh=False
     '''
-    if salt.utils.is_true(refresh):
+    if salt.utils.data.is_true(refresh):
         refresh_db(full=True)
     upgrades = {}
     # awk is in core-os package so we can use it without checking
@@ -184,9 +198,16 @@ def upgrade(refresh=False, **kwargs):
     In non-global zones upgrade is limited by dependency constrains linked to
     the version of pkg://solaris/entire.
 
-    Returns also the raw output of the ``pkg update`` command (because if
-    update creates a new boot environment, no immediate changes are visible in
-    ``pkg list``).
+    Returns a dictionary containing the changes:
+
+    .. code-block:: python
+
+        {'<package>':  {'old': '<old-version>',
+                        'new': '<new-version>'}}
+
+    When there is a failure, an explanation is also included in the error
+    message, based on the return code of the ``pkg update`` command.
+
 
     CLI Example:
 
@@ -194,12 +215,7 @@ def upgrade(refresh=False, **kwargs):
 
         salt '*' pkg.upgrade
     '''
-    ret = {'changes': {},
-           'result': True,
-           'comment': '',
-           }
-
-    if salt.utils.is_true(refresh):
+    if salt.utils.data.is_true(refresh):
         refresh_db()
 
     # Get a list of the packages before install so we can diff after to see
@@ -209,22 +225,19 @@ def upgrade(refresh=False, **kwargs):
     # Install or upgrade the package
     # If package is already installed
     cmd = ['pkg', 'update', '-v', '--accept']
-    out = __salt__['cmd.run_all'](cmd,
-                                  output_loglevel='trace',
-                                  python_shell=False)
-
+    result = __salt__['cmd.run_all'](cmd,
+                                     output_loglevel='trace',
+                                     python_shell=False)
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
 
-    if out['retcode'] != 0:
+    if result['retcode'] != 0:
         raise CommandExecutionError(
-            'Error occurred updating package(s)',
-            info={
-                'changes': ret,
-                'retcode': ips_pkg_return_values[out['retcode']],
-                'errors': [out['stderr']]
-            }
+            'Problem encountered upgrading packages',
+            info={'changes': ret,
+                  'retcode': ips_pkg_return_values[result['retcode']],
+                  'result': result}
         )
 
     return ret
@@ -243,7 +256,7 @@ def list_pkgs(versions_as_list=False, **kwargs):
         salt '*' pkg.list_pkgs
     '''
     # not yet implemented or not applicable
-    if any([salt.utils.is_true(kwargs.get(x))
+    if any([salt.utils.data.is_true(kwargs.get(x))
         for x in ('removed', 'purge_desired')]):
         return {}
 
@@ -285,16 +298,15 @@ def version(*names, **kwargs):
         salt '*' pkg_resource.version pkg://solaris/entire
 
     '''
-    namelist = ''
-    for pkgname in names:
-        namelist += '{0} '.format(pkgname)
-    cmd = '/bin/pkg list -Hv {0}'.format(namelist)
-    lines = __salt__['cmd.run_stdout'](cmd).splitlines()
-    ret = {}
-    for line in lines:
-        ret[_ips_get_pkgname(line)] = _ips_get_pkgversion(line)
-    if ret:
-        return ret
+    if names:
+        cmd = ['/bin/pkg', 'list', '-Hv']
+        cmd.extend(names)
+        lines = __salt__['cmd.run_stdout'](cmd).splitlines()
+        ret = {}
+        for line in lines:
+            ret[_ips_get_pkgname(line)] = _ips_get_pkgversion(line)
+        if ret:
+            return ret
     return ''
 
 
@@ -311,7 +323,7 @@ def latest_version(name, **kwargs):
 
         salt '*' pkg.latest_version pkg://solaris/entire
     '''
-    cmd = '/bin/pkg list -Hnv {0}'.format(name)
+    cmd = ['/bin/pkg', 'list', '-Hnv', name]
     lines = __salt__['cmd.run_stdout'](cmd).splitlines()
     ret = {}
     for line in lines:
@@ -321,7 +333,7 @@ def latest_version(name, **kwargs):
     return ''
 
 # available_version is being deprecated
-available_version = salt.utils.alias_function(latest_version, 'available_version')
+available_version = salt.utils.functools.alias_function(latest_version, 'available_version')
 
 
 def get_fmri(name, **kwargs):
@@ -338,7 +350,7 @@ def get_fmri(name, **kwargs):
     if name.startswith('pkg://'):
         # already full fmri
         return name
-    cmd = '/bin/pkg list -aHv {0}'.format(name)
+    cmd = ['/bin/pkg', 'list', '-aHv', name]
     # there can be more packages matching the name
     lines = __salt__['cmd.run_stdout'](cmd).splitlines()
     if not lines:
@@ -366,7 +378,7 @@ def normalize_name(name, **kwargs):
     if name.startswith('pkg://'):
         # already full fmri
         return name
-    cmd = '/bin/pkg list -aHv {0}'.format(name)
+    cmd = ['/bin/pkg', 'list', '-aHv', name]
     # there can be more packages matching the name
     lines = __salt__['cmd.run_stdout'](cmd).splitlines()
     # if we get more lines, it's multiple match (name not unique)
@@ -391,7 +403,7 @@ def is_installed(name, **kwargs):
         salt '*' pkg.is_installed bash
     '''
 
-    cmd = '/bin/pkg list -Hv {0}'.format(name)
+    cmd = ['/bin/pkg', 'list', '-Hv', name]
     return __salt__['cmd.retcode'](cmd) == 0
 
 
@@ -409,8 +421,8 @@ def search(name, versions_as_list=False, **kwargs):
     '''
 
     ret = {}
-    cmd = '/bin/pkg list -aHv {0}'.format(name)
-    out = __salt__['cmd.run_all'](cmd)
+    cmd = ['/bin/pkg', 'list', '-aHv', name]
+    out = __salt__['cmd.run_all'](cmd, ignore_retcode=True)
     if out['retcode'] != 0:
         # error = nothing found
         return {}
@@ -467,11 +479,8 @@ def install(name=None, refresh=False, pkgs=None, version=None, test=False, **kwa
                                               list(pkg.items())[0][1])
             else:
                 pkg2inst += '{0} '.format(list(pkg.items())[0][0])
-        log.debug(
-            'Installing these packages instead of {0}: {1}'.format(
-                name, pkg2inst
-            )
-        )
+        log.debug('Installing these packages instead of %s: %s',
+                  name, pkg2inst)
 
     else:   # install single package
         if version:
@@ -479,9 +488,9 @@ def install(name=None, refresh=False, pkgs=None, version=None, test=False, **kwa
         else:
             pkg2inst = "{0}".format(name)
 
-    cmd = 'pkg install -v --accept '
+    cmd = ['pkg', 'install', '-v', '--accept']
     if test:
-        cmd += '-n '
+        cmd.append('-n')
 
     # Get a list of the packages before install so we can diff after to see
     # what got installed.
@@ -489,14 +498,14 @@ def install(name=None, refresh=False, pkgs=None, version=None, test=False, **kwa
 
     # Install or upgrade the package
     # If package is already installed
-    cmd += '{0}'.format(pkg2inst)
+    cmd.append(pkg2inst)
 
     out = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
 
     # Get a list of the packages again, including newly installed ones.
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
 
     if out['retcode'] != 0:
         raise CommandExecutionError(
@@ -542,29 +551,24 @@ def remove(name=None, pkgs=None, **kwargs):
         salt '*' pkg.remove pkg://solaris/shell/tcsh
         salt '*' pkg.remove pkgs='["foo", "bar"]'
     '''
-    pkg2rm = ''
-    if pkgs:    # multiple packages specified
-        for pkg in pkgs:
-            pkg2rm += '{0} '.format(pkg)
-        log.debug(
-            'Installing these packages instead of {0}: {1}'.format(
-                name, pkg2rm
-            )
-        )
-    else:   # remove single package
-        pkg2rm = '{0}'.format(name)
+    targets = salt.utils.args.split_input(pkgs) if pkgs else [name]
+    if not targets:
+        return {}
+
+    if pkgs:
+        log.debug('Removing these packages instead of %s: %s', name, targets)
 
     # Get a list of the currently installed pkgs.
     old = list_pkgs()
 
     # Remove the package(s)
-    cmd = '/bin/pkg uninstall -v {0}'.format(pkg2rm)
+    cmd = ['/bin/pkg', 'uninstall', '-v'] + targets
     out = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
 
     # Get a list of the packages after the uninstall
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
 
     if out['retcode'] != 0:
         raise CommandExecutionError(
