@@ -1,20 +1,30 @@
 # -*- coding: utf-8 -*-
 '''
-Support for reboot, shutdown, etc
-'''
-from __future__ import absolute_import
+Support for reboot, shutdown, etc on POSIX-like systems.
 
-# Import python libs
+.. note::
+
+    If you have configured a wrapper such as ``molly-guard`` to
+    intercept *interactive* shutdown commands, be aware that calling
+    ``system.halt``, ``system.poweroff``, ``system.reboot``, and
+    ``system.shutdown`` with ``salt-call`` will hang indefinitely
+    while the wrapper script waits for user input. Calling them with
+    ``salt`` will work as expected.
+
+'''
+from __future__ import absolute_import, unicode_literals, print_function
+
+# Import Python libs
 from datetime import datetime, timedelta, tzinfo
 import re
 import os.path
 
-# Import salt libs
-import salt.utils
-import salt.ext.six as six
+# Import Salt libs
+import salt.utils.files
+import salt.utils.path
+import salt.utils.platform
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 
-
 __virtualname__ = 'system'
 
 
@@ -23,13 +33,13 @@ def __virtual__():
     Only supported on POSIX-like systems
     Windows, Solaris, and Mac have their own modules
     '''
-    if salt.utils.is_windows():
-        return (False, 'This module is not available on windows')
+    if salt.utils.platform.is_windows():
+        return (False, 'This module is not available on Windows')
 
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return (False, 'This module is not available on Mac OS')
 
-    if salt.utils.is_sunos():
+    if salt.utils.platform.is_sunos():
         return (False, 'This module is not available on SunOS')
 
     return __virtualname__
@@ -167,8 +177,11 @@ def has_settable_hwclock():
 
     salt '*' system.has_settable_hwclock
     '''
-    if salt.utils.which_bin(['hwclock']) is not None:
-        res = __salt__['cmd.run_all'](['hwclock', '--test', '--systohc'], python_shell=False)
+    if salt.utils.path.which_bin(['hwclock']) is not None:
+        res = __salt__['cmd.run_all'](
+            ['hwclock', '--test', '--systohc'], python_shell=False,
+            output_loglevel='quiet', ignore_retcode=True
+        )
         return res['retcode'] == 0
     return False
 
@@ -242,10 +255,10 @@ def get_system_time(utc_offset=None):
     Get the system time.
 
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: Returns the system time in HH:MM:SS AM/PM format.
     :rtype: str
 
@@ -276,10 +289,10 @@ def set_system_time(newtime, utc_offset=None):
         Meaning you may have to quote the text twice from the command line.
 
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: Returns True if successful. Otherwise False.
     :rtype: bool
 
@@ -303,10 +316,10 @@ def get_system_date_time(utc_offset=None):
     Get the system date/time.
 
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: Returns the system time in YYYY-MM-DD hh:mm:ss format.
     :rtype: str
 
@@ -344,10 +357,10 @@ def set_system_date_time(years=None,
     :param int minutes: Minutes digit: 0 - 59
     :param int seconds: Seconds digit: 0 - 59
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: True if successful. Otherwise False.
     :rtype: bool
 
@@ -396,10 +409,10 @@ def get_system_date(utc_offset=None):
     Get the system date
 
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: Returns the system date.
     :rtype: str
 
@@ -415,10 +428,11 @@ def get_system_date(utc_offset=None):
 
 def set_system_date(newdate, utc_offset=None):
     '''
-    Set the Windows system date. Use <mm-dd-yy> format for the date.
+    Set the system date. Use <mm-dd-yy> format for the date.
 
     :param str newdate:
-        The date to set. Can be any of the following formats
+        The date to set. Can be any of the following formats:
+
         - YYYY-MM-DD
         - MM-DD-YYYY
         - MM-DD-YY
@@ -451,9 +465,9 @@ def set_system_date(newdate, utc_offset=None):
 # Note that _FixedOffset(0) is a way to build a UTC tzinfo object.
 
 class _FixedOffset(tzinfo):
-    """
+    '''
     Fixed offset in minutes east from UTC.
-    """
+    '''
 
     def __init__(self, offset):
         super(self.__class__, self).__init__()
@@ -493,30 +507,31 @@ def get_computer_desc():
 
         salt '*' system.get_computer_desc
     '''
-    desc = None
-    hostname_cmd = salt.utils.which('hostnamectl')
+    hostname_cmd = salt.utils.path.which('hostnamectl')
     if hostname_cmd:
         desc = __salt__['cmd.run'](
             [hostname_cmd, 'status', '--pretty'],
             python_shell=False
         )
     else:
+        desc = None
         pattern = re.compile(r'^\s*PRETTY_HOSTNAME=(.*)$')
         try:
-            with salt.utils.fopen('/etc/machine-info', 'r') as mach_info:
+            with salt.utils.files.fopen('/etc/machine-info', 'r') as mach_info:
                 for line in mach_info.readlines():
+                    line = salt.utils.stringutils.to_unicode(line)
                     match = pattern.match(line)
                     if match:
                         # get rid of whitespace then strip off quotes
                         desc = _strip_quotes(match.group(1).strip())
                         # no break so we get the last occurance
         except IOError:
+            pass
+
+        if desc is None:
             return False
-    if six.PY3:
-        desc = desc.replace('\\"', '"')
-    else:
-        desc = desc.replace('\\"', '"').decode('string_escape')
-    return desc
+
+    return desc.replace(r'\"', r'"').replace(r'\n', '\n').replace(r'\t', '\t')
 
 
 def set_computer_desc(desc):
@@ -534,11 +549,10 @@ def set_computer_desc(desc):
 
         salt '*' system.set_computer_desc "Michael's laptop"
     '''
-    if six.PY3:
-        desc = desc.replace('"', '\\"')
-    else:
-        desc = desc.encode('string_escape').replace('"', '\\"')
-    hostname_cmd = salt.utils.which('hostnamectl')
+    desc = salt.utils.stringutils.to_unicode(
+        desc).replace('"', r'\"').replace('\n', r'\n').replace('\t', r'\t')
+
+    hostname_cmd = salt.utils.path.which('hostnamectl')
     if hostname_cmd:
         result = __salt__['cmd.retcode'](
             [hostname_cmd, 'set-hostname', '--pretty', desc],
@@ -547,26 +561,25 @@ def set_computer_desc(desc):
         return True if result == 0 else False
 
     if not os.path.isfile('/etc/machine-info'):
-        with salt.utils.fopen('/etc/machine-info', 'w'):
+        with salt.utils.files.fopen('/etc/machine-info', 'w'):
             pass
 
-    is_pretty_hostname_found = False
     pattern = re.compile(r'^\s*PRETTY_HOSTNAME=(.*)$')
-    new_line = 'PRETTY_HOSTNAME="{0}"'.format(desc)
+    new_line = salt.utils.stringutils.to_str('PRETTY_HOSTNAME="{0}"'.format(desc))
     try:
-        with salt.utils.fopen('/etc/machine-info', 'r+') as mach_info:
+        with salt.utils.files.fopen('/etc/machine-info', 'r+') as mach_info:
             lines = mach_info.readlines()
             for i, line in enumerate(lines):
-                if pattern.match(line):
-                    is_pretty_hostname_found = True
+                if pattern.match(salt.utils.stringutils.to_unicode(line)):
                     lines[i] = new_line
-            if not is_pretty_hostname_found:
+                    break
+            else:
+                # PRETTY_HOSTNAME line was not found, add it to the end
                 lines.append(new_line)
             # time to write our changes to the file
             mach_info.seek(0, 0)
             mach_info.truncate()
-            mach_info.write(''.join(lines))
-            mach_info.write('\n')
+            mach_info.writelines(lines)
             return True
     except IOError:
         return False
@@ -580,7 +593,7 @@ def set_computer_name(hostname):
 
     .. code-block:: bash
 
-        salt '*' system.set_conputer_name master.saltstack.com
+        salt '*' system.set_computer_name master.saltstack.com
     '''
     return __salt__['network.mod_hostname'](hostname)
 
