@@ -2,13 +2,16 @@
 '''
 Execute a command and read the output as JSON. The JSON data is then directly overlaid onto the minion's Pillar data.
 '''
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Don't "fix" the above docstring to put it on two lines, as the sphinx
 # autosummary pulls only the first line for its description.
 
-# Import python libs
+# Import Python libs
 import logging
-import json
+
+# Import Salt libs
+import salt.utils.json
 
 # Set up logging
 log = logging.getLogger(__name__)
@@ -21,9 +24,8 @@ def ext_pillar(minion_id,  # pylint: disable=W0613
     Execute a command and read the output as JSON
     '''
     try:
-        return json.loads(__salt__['cmd.run'](command))
+        command = command.replace('%s', minion_id)
+        return salt.utils.json.loads(__salt__['cmd.run'](command))
     except Exception:
-        log.critical(
-                'JSON data from {0} failed to parse'.format(command)
-                )
+        log.critical('JSON data from %s failed to parse', command)
         return {}
