@@ -4,7 +4,7 @@ Configuration of the alternatives system
 
 Control the alternatives system
 
-.. code-block:: yaml
+.. code-block:: jinja
 
   {% set my_hadoop_conf = '/opt/hadoop/conf' %}
 
@@ -26,6 +26,7 @@ Control the alternatives system
       - path: {{ my_hadoop_conf }}
 
 '''
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Define a function alias in order not to shadow built-in's
 __func_alias__ = {
@@ -33,6 +34,13 @@ __func_alias__ = {
 }
 
 
+def __virtual__():
+    '''
+    Only load if alternatives execution module is available.
+    '''
+    return True if 'alternatives.auto' in __salt__ else False
+
+
 def install(name, link, path, priority):
     '''
     Install new alternative for defined <name>
@@ -63,24 +71,33 @@ def install(name, link, path, priority):
            'comment': ''}
 
     isinstalled = __salt__['alternatives.check_installed'](name, path)
-    if not isinstalled:
+    if isinstalled:
+        ret['comment'] = 'Alternatives for {0} is already set to {1}'.format(name, path)
+    else:
         if __opts__['test']:
             ret['comment'] = (
                 'Alternative will be set for {0} to {1} with priority {2}'
             ).format(name, path, priority)
             ret['result'] = None
             return ret
-        __salt__['alternatives.install'](name, link, path, priority)
-        ret['comment'] = (
-            'Setting alternative for {0} to {1} with priority {2}'
-        ).format(name, path, priority)
-        ret['changes'] = {'name': name,
-                          'link': link,
-                          'path': path,
-                          'priority': priority}
-        return ret
 
-    ret['comment'] = 'Alternatives for {0} is already set to {1}'.format(name, path)
+        out = __salt__['alternatives.install'](name, link, path, priority)
+        current = __salt__['alternatives.show_current'](name)
+        master_link = __salt__['alternatives.show_link'](name)
+        if current == path and master_link == link:
+            ret['comment'] = (
+                'Alternative for {0} set to path {1} with priority {2}'
+            ).format(name, current, priority)
+            ret['changes'] = {'name': name,
+                              'link': link,
+                              'path': path,
+                              'priority': priority}
+        else:
+            ret['result'] = False
+            ret['comment'] = (
+                'Alternative for {0} not installed: {1}'
+            ).format(name, out)
+
     return ret
 
 
@@ -208,13 +225,12 @@ def set_(name, path):
         if __opts__['test']:
             ret['comment'] = (
                 'Alternative for {0} will be set to path {1}'
-            ).format(name, current)
+            ).format(name, path)
             ret['result'] = None
             return ret
         __salt__['alternatives.set'](name, path)
         current = __salt__['alternatives.show_current'](name)
         if current == path:
-            ret['result'] = True
             ret['comment'] = (
                 'Alternative for {0} set to path {1}'
             ).format(name, current)
