@@ -132,16 +132,18 @@ passed in as a dict, or as a string to pull from pillars or minion config:
 '''
 
 # Import Python Libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
-import json
 import os
 
 # Import Salt Libs
-import salt.utils
+import salt.utils.data
+import salt.utils.files
+import salt.utils.json
+import salt.utils.stringutils
 import salt.utils.odict as odict
 import salt.utils.dictupdate as dictupdate
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 
 # Import 3rd party libs
@@ -158,8 +160,8 @@ __virtualname__ = 'boto_iam'
 
 if six.PY2:
     def _byteify(thing):
-        # Note that we intentionally don't treat odicts here - they won't compare equal
-        # in many circumstances where AWS treats them the same...
+        # Note that we intentionally don't treat odicts here - they won't
+        # compare equal in many circumstances where AWS treats them the same...
         if isinstance(thing, dict):
             return dict([(_byteify(k), _byteify(v)) for k, v in six.iteritems(thing)])
         elif isinstance(thing, list):
@@ -232,7 +234,7 @@ def user_absent(name, delete_keys=True, delete_mfa_devices=True, delete_profile=
     if delete_keys:
         keys = __salt__['boto_iam.get_all_access_keys'](user_name=name, region=region, key=key,
                                                         keyid=keyid, profile=profile)
-        log.debug('Keys for user {0} are {1}.'.format(name, keys))
+        log.debug('Keys for user %s are %s.', name, keys)
         if isinstance(keys, dict):
             keys = keys['list_access_keys_response']['list_access_keys_result']['access_key_metadata']
             for k in keys:
@@ -250,12 +252,19 @@ def user_absent(name, delete_keys=True, delete_mfa_devices=True, delete_profile=
             for d in devices:
                 serial = d['serial_number']
                 if __opts__['test']:
-                    ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} MFA device {1} is set to be deleted.'.format(name, serial)])
+                    ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} MFA device {1} is set to be deactivated.'.format(name, serial)])
                     ret['result'] = None
                 else:
-                    mfa_deleted = __salt__['boto_iam.deactivate_mfa_device'](user_name=name, serial=serial, region=region, key=key, keyid=keyid, profile=profile)
+                    mfa_deactivated = __salt__['boto_iam.deactivate_mfa_device'](user_name=name, serial=serial, region=region, key=key, keyid=keyid, profile=profile)
+                    if mfa_deactivated:
+                        ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} MFA device {1} is deactivated.'.format(name, serial)])
+                if __opts__['test']:
+                    ret['comment'] = ' '.join([ret['comment'], 'Virtual MFA device {0} is set to be deleted.'.format(serial)])
+                    ret['result'] = None
+                else:
+                    mfa_deleted = __salt__['boto_iam.delete_virtual_mfa_device'](serial=serial, region=region, key=key, keyid=keyid, profile=profile)
                     if mfa_deleted:
-                        ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} MFA device {1} are deleted.'.format(name, serial)])
+                        ret['comment'] = ' '.join([ret['comment'], 'Virtual MFA device {0} is deleted.'.format(serial)])
     # delete the user's login profile
     if delete_profile:
         if __opts__['test']:
@@ -266,7 +275,7 @@ def user_absent(name, delete_keys=True, delete_mfa_devices=True, delete_profile=
             if profile_deleted:
                 ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} login profile is deleted.'.format(name)])
     if __opts__['test']:
-        ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} policies are set to be deleted.'.format(name)])
+        ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} managed policies are set to be detached.'.format(name)])
         ret['result'] = None
     else:
         _ret = _user_policies_detached(name, region, key, keyid, profile)
@@ -275,6 +284,16 @@ def user_absent(name, delete_keys=True, delete_mfa_devices=True, delete_profile=
             ret['result'] = _ret['result']
             if ret['result'] is False:
                 return ret
+    if __opts__['test']:
+        ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} inline policies are set to be deleted.'.format(name)])
+        ret['result'] = None
+    else:
+        _ret = _user_policies_deleted(name, region, key, keyid, profile)
+        ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
+        if not _ret['result']:
+            ret['result'] = _ret['result']
+            if ret['result'] is False:
+                return ret
     # finally, actually delete the user
     if __opts__['test']:
         ret['comment'] = ' '.join([ret['comment'], 'IAM user {0} is set to be deleted.'.format(name)])
@@ -294,7 +313,6 @@ def user_absent(name, delete_keys=True, delete_mfa_devices=True, delete_profile=
 def keys_present(name, number, save_dir, region=None, key=None, keyid=None, profile=None,
                  save_format="{2}\n{0}\n{3}\n{1}\n"):
     '''
-
     .. versionadded:: 2015.8.0
 
     Ensure the IAM access keys are present.
@@ -323,9 +341,10 @@ def keys_present(name, number, save_dir, region=None, key=None, keyid=None, prof
         that contains a dict with region, key and keyid.
 
     save_format (dict)
-        Save format is repeated for each key. Default format is "{2}\n{0}\n{3}\n{1}\n",
-        where {0} and {1} are placeholders for new key_id and key respectively,
-        whereas {2} and {3} are "key_id-{number}" and 'key-{number}' strings kept for compatibility.
+        Save format is repeated for each key. Default format is
+        "{2}\\n{0}\\n{3}\\n{1}\\n", where {0} and {1} are placeholders for new
+        key_id and key respectively, whereas {2} and {3} are "key_id-{number}"
+        and 'key-{number}' strings kept for compatibility.
     '''
     ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
     if not __salt__['boto_iam.get_user'](name, region, key, keyid, profile):
@@ -342,14 +361,14 @@ def keys_present(name, number, save_dir, region=None, key=None, keyid=None, prof
         return ret
     keys = __salt__['boto_iam.get_all_access_keys'](user_name=name, region=region, key=key,
                                                     keyid=keyid, profile=profile)
-    if isinstance(keys, str):
-        log.debug('keys are : false {0}'.format(keys))
+    if isinstance(keys, six.string_types):
+        log.debug('keys are : false %s', keys)
         error, message = _get_error(keys)
         ret['comment'] = 'Could not get keys.\n{0}\n{1}'.format(error, message)
         ret['result'] = False
         return ret
     keys = keys['list_access_keys_response']['list_access_keys_result']['access_key_metadata']
-    log.debug('Keys are : {0}.'.format(keys))
+    log.debug('Keys are : %s.', keys)
     if len(keys) >= number:
         ret['comment'] = 'The number of keys exist for user {0}'.format(name)
         ret['result'] = True
@@ -361,23 +380,30 @@ def keys_present(name, number, save_dir, region=None, key=None, keyid=None, prof
     new_keys = {}
     for i in range(number-len(keys)):
         created = __salt__['boto_iam.create_access_key'](name, region, key, keyid, profile)
-        if isinstance(created, str):
+        if isinstance(created, six.string_types):
             error, message = _get_error(created)
             ret['comment'] = 'Could not create keys.\n{0}\n{1}'.format(error, message)
             ret['result'] = False
             return ret
-        log.debug('Created is : {0}'.format(created))
+        log.debug('Created is : %s', created)
         response = 'create_access_key_response'
         result = 'create_access_key_result'
-        new_keys[str(i)] = {}
-        new_keys[str(i)]['key_id'] = created[response][result]['access_key']['access_key_id']
-        new_keys[str(i)]['secret_key'] = created[response][result]['access_key']['secret_access_key']
+        new_keys[six.text_type(i)] = {}
+        new_keys[six.text_type(i)]['key_id'] = created[response][result]['access_key']['access_key_id']
+        new_keys[six.text_type(i)]['secret_key'] = created[response][result]['access_key']['secret_access_key']
     try:
-        with salt.utils.fopen('{0}/{1}'.format(save_dir, name), 'a') as _wrf:
+        with salt.utils.files.fopen('{0}/{1}'.format(save_dir, name), 'a') as _wrf:
             for key_num, key in new_keys.items():
                 key_id = key['key_id']
                 secret_key = key['secret_key']
-                _wrf.write(save_format.format(key_id, secret_key, 'key_id-{0}'.format(key_num), 'key-{0}'.format(key_num)))
+                _wrf.write(salt.utils.stringutils.to_str(
+                    save_format.format(
+                        key_id,
+                        secret_key,
+                        'key_id-{0}'.format(key_num),
+                        'key-{0}'.format(key_num)
+                    )
+                ))
         ret['comment'] = 'Keys have been written to file {0}/{1}.'.format(save_dir, name)
         ret['result'] = True
         ret['changes'] = new_keys
@@ -427,15 +453,15 @@ def keys_absent(access_keys, user_name, region=None, key=None, keyid=None, profi
 def _delete_key(ret, access_key_id, user_name, region=None, key=None, keyid=None, profile=None):
     keys = __salt__['boto_iam.get_all_access_keys'](user_name=user_name, region=region, key=key,
                                                     keyid=keyid, profile=profile)
-    log.debug('Keys for user {1} are : {0}.'.format(keys, user_name))
-    if isinstance(keys, str):
-        log.debug('Keys {0} are a string. Something went wrong.'.format(keys))
+    log.debug('Keys for user %s are : %s.', keys, user_name)
+    if isinstance(keys, six.string_types):
+        log.debug('Keys %s are a string. Something went wrong.', keys)
         ret['comment'] = ' '.join([ret['comment'], 'Key {0} could not be deleted.'.format(access_key_id)])
         return ret
     keys = keys['list_access_keys_response']['list_access_keys_result']['access_key_metadata']
     for k in keys:
-        log.debug('Key is: {0} and is compared with: {1}'.format(k['access_key_id'], access_key_id))
-        if str(k['access_key_id']) == str(access_key_id):
+        log.debug('Key is: %s and is compared with: %s', k['access_key_id'], access_key_id)
+        if six.text_type(k['access_key_id']) == six.text_type(access_key_id):
             if __opts__['test']:
                 ret['comment'] = 'Access key {0} is set to be deleted.'.format(access_key_id)
                 ret['result'] = None
@@ -549,12 +575,12 @@ def _user_policies_present(name, policies=None, region=None, key=None, keyid=Non
     policies_to_delete = []
     for policy_name, policy in six.iteritems(policies):
         if isinstance(policy, six.string_types):
-            dict_policy = _byteify(json.loads(policy, object_pairs_hook=odict.OrderedDict))
+            dict_policy = _byteify(salt.utils.json.loads(policy, object_pairs_hook=odict.OrderedDict))
         else:
             dict_policy = _byteify(policy)
         _policy = _byteify(__salt__['boto_iam.get_user_policy'](name, policy_name, region, key, keyid, profile))
         if _policy != dict_policy:
-            log.debug("Policy mismatch:\n{0}\n{1}".format(_policy, dict_policy))
+            log.debug("Policy mismatch:\n%s\n%s", _policy, dict_policy)
             policies_to_create[policy_name] = policy
     _list = __salt__['boto_iam.get_all_user_policies'](
         user_name=name, region=region, key=key, keyid=keyid, profile=profile
@@ -566,8 +592,7 @@ def _user_policies_present(name, policies=None, region=None, key=None, keyid=Non
         _to_modify = list(policies_to_delete)
         _to_modify.extend(policies_to_create)
         if __opts__['test']:
-            msg = '{0} policies to be modified on user {1}.'
-            ret['comment'] = msg.format(', '.join(_to_modify), name)
+            ret['comment'] = '{0} policies to be modified on user {1}.'.format(', '.join(_to_modify), name)
             ret['result'] = None
             return ret
         ret['changes']['old'] = {'policies': _list}
@@ -581,8 +606,7 @@ def _user_policies_present(name, policies=None, region=None, key=None, keyid=Non
                 )
                 ret['changes']['new'] = {'policies': _list}
                 ret['result'] = False
-                msg = 'Failed to add policy {0} for user {1}'
-                ret['comment'] = msg.format(policy_name, name)
+                ret['comment'] = 'Failed to add policy {0} for user {1}'.format(policy_name, name)
                 return ret
         for policy_name in policies_to_delete:
             policy_unset = __salt__['boto_iam.delete_user_policy'](
@@ -594,15 +618,13 @@ def _user_policies_present(name, policies=None, region=None, key=None, keyid=Non
                 )
                 ret['changes']['new'] = {'policies': _list}
                 ret['result'] = False
-                msg = 'Failed to add policy {0} to user {1}'
-                ret['comment'] = msg.format(policy_name, name)
+                ret['comment'] = 'Failed to add policy {0} to user {1}'.format(policy_name, name)
                 return ret
         _list = __salt__['boto_iam.get_all_user_policies'](
             user_name=name, region=region, key=key, keyid=keyid, profile=profile
         )
         ret['changes']['new'] = {'policies': _list}
-        msg = '{0} policies modified on user {1}.'
-        ret['comment'] = msg.format(', '.join(_list), name)
+        ret['comment'] = '{0} policies modified on user {1}.'.format(', '.join(_list), name)
     return ret
 
 
@@ -639,8 +661,7 @@ def _user_policies_attached(
         _to_modify = list(policies_to_detach)
         _to_modify.extend(policies_to_attach)
         if __opts__['test']:
-            msg = '{0} policies to be modified on user {1}.'
-            ret['comment'] = msg.format(', '.join(_to_modify), name)
+            ret['comment'] = '{0} policies to be modified on user {1}.'.format(', '.join(_to_modify), name)
             ret['result'] = None
             return ret
         ret['changes']['old'] = {'managed_policies': oldpolicies}
@@ -658,8 +679,7 @@ def _user_policies_attached(
                 newpolicies = [x.get('policy_arn') for x in _list]
                 ret['changes']['new'] = {'managed_policies': newpolicies}
                 ret['result'] = False
-                msg = 'Failed to add policy {0} to user {1}'
-                ret['comment'] = msg.format(policy_name, name)
+                ret['comment'] = 'Failed to add policy {0} to user {1}'.format(policy_name, name)
                 return ret
         for policy_name in policies_to_detach:
             policy_unset = __salt__['boto_iam.detach_user_policy'](policy_name,
@@ -675,8 +695,7 @@ def _user_policies_attached(
                 newpolicies = [x.get('policy_arn') for x in _list]
                 ret['changes']['new'] = {'managed_policies': newpolicies}
                 ret['result'] = False
-                msg = 'Failed to remove policy {0} from user {1}'
-                ret['comment'] = msg.format(policy_name, name)
+                ret['comment'] = 'Failed to remove policy {0} from user {1}'.format(policy_name, name)
                 return ret
         _list = __salt__['boto_iam.list_attached_user_policies'](name, region=region, key=key,
                                                         keyid=keyid,
@@ -684,8 +703,7 @@ def _user_policies_attached(
         newpolicies = [x.get('policy_arn') for x in _list]
         log.debug(newpolicies)
         ret['changes']['new'] = {'managed_policies': newpolicies}
-        msg = '{0} policies modified on user {1}.'
-        ret['comment'] = msg.format(', '.join(newpolicies), name)
+        ret['comment'] = '{0} policies modified on user {1}.'.format(', '.join(newpolicies), name)
     return ret
 
 
@@ -700,12 +718,10 @@ def _user_policies_detached(
                         region=region, key=key, keyid=keyid, profile=profile)
     oldpolicies = [x.get('policy_arn') for x in _list]
     if not _list:
-        msg = 'No attached policies in user {0}.'.format(name)
-        ret['comment'] = msg
+        ret['comment'] = 'No attached policies in user {0}.'.format(name)
         return ret
     if __opts__['test']:
-        msg = '{0} policies to be detached from user {1}.'
-        ret['comment'] = msg.format(', '.join(oldpolicies), name)
+        ret['comment'] = '{0} policies to be detached from user {1}.'.format(', '.join(oldpolicies), name)
         ret['result'] = None
         return ret
     ret['changes']['old'] = {'managed_policies': oldpolicies}
@@ -722,15 +738,51 @@ def _user_policies_detached(
             newpolicies = [x.get('policy_arn') for x in _list]
             ret['changes']['new'] = {'managed_policies': newpolicies}
             ret['result'] = False
-            msg = 'Failed to detach {0} from user {1}'
-            ret['comment'] = msg.format(policy_arn, name)
+            ret['comment'] = 'Failed to detach {0} from user {1}'.format(policy_arn, name)
             return ret
     _list = __salt__['boto_iam.list_attached_user_policies'](name, region=region, key=key,
                                                     keyid=keyid, profile=profile)
     newpolicies = [x.get('policy_arn') for x in _list]
     ret['changes']['new'] = {'managed_policies': newpolicies}
-    msg = '{0} policies detached from user {1}.'
-    ret['comment'] = msg.format(', '.join(newpolicies), name)
+    ret['comment'] = '{0} policies detached from user {1}.'.format(', '.join(oldpolicies), name)
+    return ret
+
+
+def _user_policies_deleted(
+        name,
+        region=None,
+        key=None,
+        keyid=None,
+        profile=None):
+    ret = {'result': True, 'comment': '', 'changes': {}}
+    oldpolicies = __salt__['boto_iam.get_all_user_policies'](user_name=name,
+                        region=region, key=key, keyid=keyid, profile=profile)
+    if not oldpolicies:
+        ret['comment'] = 'No inline policies in user {0}.'.format(name)
+        return ret
+    if __opts__['test']:
+        ret['comment'] = '{0} policies to be deleted from user {1}.'.format(', '.join(oldpolicies), name)
+        ret['result'] = None
+        return ret
+    ret['changes']['old'] = {'inline_policies': oldpolicies}
+    for policy_name in oldpolicies:
+        policy_deleted = __salt__['boto_iam.delete_user_policy'](name,
+                                                                 policy_name,
+                                                                 region=region, key=key,
+                                                                 keyid=keyid,
+                                                                 profile=profile)
+        if not policy_deleted:
+            newpolicies = __salt__['boto_iam.get_all_user_policies'](name, region=region,
+                                                                     key=key, keyid=keyid,
+                                                                     profile=profile)
+            ret['changes']['new'] = {'inline_policies': newpolicies}
+            ret['result'] = False
+            ret['comment'] = 'Failed to detach {0} from user {1}'.format(policy_name, name)
+            return ret
+    newpolicies = __salt__['boto_iam.get_all_user_policies'](name, region=region, key=key,
+                                                             keyid=keyid, profile=profile)
+    ret['changes']['new'] = {'inline_policies': newpolicies}
+    ret['comment'] = '{0} policies deleted from user {1}.'.format(', '.join(oldpolicies), name)
     return ret
 
 
@@ -740,7 +792,7 @@ def _case_password(ret, name, password, region=None, key=None, keyid=None, profi
         ret['result'] = None
         return ret
     login = __salt__['boto_iam.create_login_profile'](name, password, region, key, keyid, profile)
-    log.debug('Login is : {0}.'.format(login))
+    log.debug('Login is : %s.', login)
     if login:
         if 'Conflict' in login:
             ret['comment'] = ' '.join([ret['comment'], 'Login profile for user {0} exists.'.format(name)])
@@ -782,7 +834,7 @@ def group_absent(name, region=None, key=None, keyid=None, profile=None):
         ret['comment'] = 'IAM Group {0} does not exist.'.format(name)
         return ret
     if __opts__['test']:
-        ret['comment'] = ' '.join([ret['comment'], 'IAM group {0} policies are set to be deleted.'.format(name)])
+        ret['comment'] = ' '.join([ret['comment'], 'IAM group {0} managed policies are set to be detached.'.format(name)])
         ret['result'] = None
     else:
         _ret = _group_policies_detached(name, region, key, keyid, profile)
@@ -791,6 +843,16 @@ def group_absent(name, region=None, key=None, keyid=None, profile=None):
             ret['result'] = _ret['result']
             if ret['result'] is False:
                 return ret
+    if __opts__['test']:
+        ret['comment'] = ' '.join([ret['comment'], 'IAM group {0} inline policies are set to be deleted.'.format(name)])
+        ret['result'] = None
+    else:
+        _ret = _group_policies_deleted(name, region, key, keyid, profile)
+        ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
+        if not _ret['result']:
+            ret['result'] = _ret['result']
+            if ret['result'] is False:
+                return ret
     ret['comment'] = ' '.join([ret['comment'], 'IAM group {0} users are set to be removed.'.format(name)])
     existing_users = __salt__['boto_iam.get_group_members'](group_name=name, region=region, key=key, keyid=keyid, profile=profile)
     _ret = _case_group(ret, [], name, existing_users, region, key, keyid, profile)
@@ -840,7 +902,7 @@ def group_present(name, policies=None, policies_from_pillars=None, managed_polic
         in the policies argument will override the keys defined in
         policies_from_pillars.
 
-    manaaged_policies (list)
+    managed_policies (list)
         A list of policy names or ARNs that should be attached to this group.
 
     users (list)
@@ -908,7 +970,7 @@ def group_present(name, policies=None, policies_from_pillars=None, managed_polic
         ret['result'] = _ret['result']
         return ret
     if users is not None:
-        log.debug('Users are : {0}.'.format(users))
+        log.debug('Users are : %s.', users)
         existing_users = __salt__['boto_iam.get_group_members'](group_name=name, region=region, key=key, keyid=keyid, profile=profile)
         ret = _case_group(ret, users, name, existing_users, region, key, keyid, profile)
     return ret
@@ -918,15 +980,15 @@ def _case_group(ret, users, group_name, existing_users, region, key, keyid, prof
     _users = []
     for user in existing_users:
         _users.append(user['user_name'])
-    log.debug('upstream users are {0}'.format(_users))
+    log.debug('upstream users are %s', _users)
     for user in users:
-        log.debug('users are {0}'.format(user))
+        log.debug('users are %s', user)
         if user in _users:
             log.debug('user exists')
             ret['comment'] = ' '.join([ret['comment'], 'User {0} is already a member of group {1}.'.format(user, group_name)])
             continue
         else:
-            log.debug('user is set to be added {0}'.format(user))
+            log.debug('user is set to be added %s', user)
             if __opts__['test']:
                 ret['comment'] = 'User {0} is set to be added to group {1}.'.format(user, group_name)
                 ret['result'] = None
@@ -960,12 +1022,12 @@ def _group_policies_present(
     policies_to_delete = []
     for policy_name, policy in six.iteritems(policies):
         if isinstance(policy, six.string_types):
-            dict_policy = _byteify(json.loads(policy, object_pairs_hook=odict.OrderedDict))
+            dict_policy = _byteify(salt.utils.json.loads(policy, object_pairs_hook=odict.OrderedDict))
         else:
             dict_policy = _byteify(policy)
         _policy = _byteify(__salt__['boto_iam.get_group_policy'](name, policy_name, region, key, keyid, profile))
         if _policy != dict_policy:
-            log.debug("Policy mismatch:\n{0}\n{1}".format(_policy, dict_policy))
+            log.debug("Policy mismatch:\n%s\n%s", _policy, dict_policy)
             policies_to_create[policy_name] = policy
     _list = __salt__['boto_iam.get_all_group_policies'](
         name, region, key, keyid, profile
@@ -977,8 +1039,7 @@ def _group_policies_present(
         _to_modify = list(policies_to_delete)
         _to_modify.extend(policies_to_create)
         if __opts__['test']:
-            msg = '{0} policies to be modified on group {1}.'
-            ret['comment'] = msg.format(', '.join(_to_modify), name)
+            ret['comment'] = '{0} policies to be modified on group {1}.'.format(', '.join(_to_modify), name)
             ret['result'] = None
             return ret
         ret['changes']['old'] = {'policies': _list}
@@ -992,8 +1053,7 @@ def _group_policies_present(
                 )
                 ret['changes']['new'] = {'policies': _list}
                 ret['result'] = False
-                msg = 'Failed to add policy {0} to group {1}'
-                ret['comment'] = msg.format(policy_name, name)
+                ret['comment'] = 'Failed to add policy {0} to group {1}'.format(policy_name, name)
                 return ret
         for policy_name in policies_to_delete:
             policy_unset = __salt__['boto_iam.delete_group_policy'](
@@ -1005,15 +1065,13 @@ def _group_policies_present(
                 )
                 ret['changes']['new'] = {'policies': _list}
                 ret['result'] = False
-                msg = 'Failed to add policy {0} to group {1}'
-                ret['comment'] = msg.format(policy_name, name)
+                ret['comment'] = 'Failed to add policy {0} to group {1}'.format(policy_name, name)
                 return ret
         _list = __salt__['boto_iam.get_all_group_policies'](
             name, region, key, keyid, profile
         )
         ret['changes']['new'] = {'policies': _list}
-        msg = '{0} policies modified on group {1}.'
-        ret['comment'] = msg.format(', '.join(_list), name)
+        ret['comment'] = '{0} policies modified on group {1}.'.format(', '.join(_list), name)
     return ret
 
 
@@ -1052,8 +1110,7 @@ def _group_policies_attached(
         _to_modify = list(policies_to_detach)
         _to_modify.extend(policies_to_attach)
         if __opts__['test']:
-            msg = '{0} policies to be modified on group {1}.'
-            ret['comment'] = msg.format(', '.join(_to_modify), name)
+            ret['comment'] = '{0} policies to be modified on group {1}.'.format(', '.join(_to_modify), name)
             ret['result'] = None
             return ret
         ret['changes']['old'] = {'managed_policies': oldpolicies}
@@ -1070,8 +1127,7 @@ def _group_policies_attached(
                 newpolicies = [x.get('policy_arn') for x in _list]
                 ret['changes']['new'] = {'managed_policies': newpolicies}
                 ret['result'] = False
-                msg = 'Failed to add policy {0} to group {1}'
-                ret['comment'] = msg.format(policy_name, name)
+                ret['comment'] = 'Failed to add policy {0} to group {1}'.format(policy_name, name)
                 return ret
         for policy_name in policies_to_detach:
             policy_unset = __salt__['boto_iam.detach_group_policy'](policy_name,
@@ -1086,16 +1142,14 @@ def _group_policies_attached(
                 newpolicies = [x.get('policy_arn') for x in _list]
                 ret['changes']['new'] = {'managed_policies': newpolicies}
                 ret['result'] = False
-                msg = 'Failed to remove policy {0} from group {1}'
-                ret['comment'] = msg.format(policy_name, name)
+                ret['comment'] = 'Failed to remove policy {0} from group {1}'.format(policy_name, name)
                 return ret
         _list = __salt__['boto_iam.list_attached_group_policies'](name, region=region, key=key,
                                                         keyid=keyid, profile=profile)
         newpolicies = [x.get('policy_arn') for x in _list]
         log.debug(newpolicies)
         ret['changes']['new'] = {'managed_policies': newpolicies}
-        msg = '{0} policies modified on group {1}.'
-        ret['comment'] = msg.format(', '.join(newpolicies), name)
+        ret['comment'] = '{0} policies modified on group {1}.'.format(', '.join(newpolicies), name)
     return ret
 
 
@@ -1110,12 +1164,10 @@ def _group_policies_detached(
                         region=region, key=key, keyid=keyid, profile=profile)
     oldpolicies = [x.get('policy_arn') for x in _list]
     if not _list:
-        msg = 'No attached policies in group {0}.'.format(name)
-        ret['comment'] = msg
+        ret['comment'] = 'No attached policies in group {0}.'.format(name)
         return ret
     if __opts__['test']:
-        msg = '{0} policies to be detached from group {1}.'
-        ret['comment'] = msg.format(', '.join(oldpolicies), name)
+        ret['comment'] = '{0} policies to be detached from group {1}.'.format(', '.join(oldpolicies), name)
         ret['result'] = None
         return ret
     ret['changes']['old'] = {'managed_policies': oldpolicies}
@@ -1132,15 +1184,51 @@ def _group_policies_detached(
             newpolicies = [x.get('policy_arn') for x in _list]
             ret['changes']['new'] = {'managed_policies': newpolicies}
             ret['result'] = False
-            msg = 'Failed to detach {0} from group {1}'
-            ret['comment'] = msg.format(policy_arn, name)
+            ret['comment'] = 'Failed to detach {0} from group {1}'.format(policy_arn, name)
             return ret
     _list = __salt__['boto_iam.list_attached_group_policies'](name, region=region, key=key,
                                                     keyid=keyid, profile=profile)
     newpolicies = [x.get('policy_arn') for x in _list]
     ret['changes']['new'] = {'managed_policies': newpolicies}
-    msg = '{0} policies detached from group {1}.'
-    ret['comment'] = msg.format(', '.join(newpolicies), name)
+    ret['comment'] = '{0} policies detached from group {1}.'.format(', '.join(newpolicies), name)
+    return ret
+
+
+def _group_policies_deleted(
+        name,
+        region=None,
+        key=None,
+        keyid=None,
+        profile=None):
+    ret = {'result': True, 'comment': '', 'changes': {}}
+    oldpolicies = __salt__['boto_iam.get_all_group_policies'](group_name=name,
+                        region=region, key=key, keyid=keyid, profile=profile)
+    if not oldpolicies:
+        ret['comment'] = 'No inline policies in group {0}.'.format(name)
+        return ret
+    if __opts__['test']:
+        ret['comment'] = '{0} policies to be deleted from group {1}.'.format(', '.join(oldpolicies), name)
+        ret['result'] = None
+        return ret
+    ret['changes']['old'] = {'inline_policies': oldpolicies}
+    for policy_name in oldpolicies:
+        policy_deleted = __salt__['boto_iam.delete_group_policy'](name,
+                                                                 policy_name,
+                                                                 region=region, key=key,
+                                                                 keyid=keyid,
+                                                                 profile=profile)
+        if not policy_deleted:
+            newpolicies = __salt__['boto_iam.get_all_group_policies'](name, region=region,
+                                                                     key=key, keyid=keyid,
+                                                                     profile=profile)
+            ret['changes']['new'] = {'inline_policies': newpolicies}
+            ret['result'] = False
+            ret['comment'] = 'Failed to detach {0} from group {1}'.format(policy_name, name)
+            return ret
+    newpolicies = __salt__['boto_iam.get_all_group_policies'](name, region=region, key=key,
+                                                             keyid=keyid, profile=profile)
+    ret['changes']['new'] = {'inline_policies': newpolicies}
+    ret['comment'] = '{0} policies deleted from group {1}.'.format(', '.join(oldpolicies), name)
     return ret
 
 
@@ -1214,9 +1302,9 @@ def account_policy(name=None, allow_users_to_change_password=None,
     for key, value in config.items():
         if key in ('region', 'key', 'keyid', 'profile', 'name'):
             continue
-        if value is not None and str(info[key]) != str(value).lower():
+        if value is not None and six.text_type(info[key]) != six.text_type(value).lower():
             ret['comment'] = ' '.join([ret['comment'], 'Policy value {0} has been set to {1}.'.format(value, info[key])])
-            ret['changes'][key] = str(value).lower()
+            ret['changes'][key] = six.text_type(value).lower()
     if not ret['changes']:
         ret['comment'] = 'Account policy is not changed.'
         return ret
@@ -1236,7 +1324,7 @@ def account_policy(name=None, allow_users_to_change_password=None,
                                                            region, key, keyid, profile):
         return ret
     ret['comment'] = 'Account policy is not changed.'
-    ret['changes'] = None
+    ret['changes'] = {}
     ret['result'] = False
     return ret
 
@@ -1318,7 +1406,7 @@ def server_cert_present(name, public_key, private_key, cert_chain=None, path=Non
     '''
     ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
     exists = __salt__['boto_iam.get_server_certificate'](name, region, key, keyid, profile)
-    log.debug('Variables are : {0}.'.format(locals()))
+    log.debug('Variables are : %s.', locals())
     if exists:
         ret['comment'] = 'Certificate {0} exists.'.format(name)
         return ret
@@ -1416,24 +1504,23 @@ def policy_present(name, policy_document, path=None, description=None,
         _describe = __salt__['boto_iam.get_policy_version'](name, policy.get('default_version_id'),
                                                        region, key, keyid, profile).get('policy_version', {})
         if isinstance(_describe['document'], six.string_types):
-            describeDict = json.loads(_describe['document'])
+            describeDict = salt.utils.json.loads(_describe['document'])
         else:
             describeDict = _describe['document']
 
         if isinstance(policy_document, six.string_types):
-            policy_document = json.loads(policy_document)
+            policy_document = salt.utils.json.loads(policy_document)
 
-        r = salt.utils.compare_dicts(describeDict, policy_document)
+        r = salt.utils.data.compare_dicts(describeDict, policy_document)
 
         if bool(r):
             if __opts__['test']:
-                msg = 'Policy {0} set to be modified.'.format(name)
-                ret['comment'] = msg
+                ret['comment'] = 'Policy {0} set to be modified.'.format(name)
                 ret['result'] = None
                 return ret
 
             ret['comment'] = ' '.join([ret['comment'], 'Policy to be modified'])
-            policy_document = json.dumps(policy_document)
+            policy_document = salt.utils.json.dumps(policy_document)
 
             r = __salt__['boto_iam.create_policy_version'](policy_name=name,
                                                policy_document=policy_document,
@@ -1498,7 +1585,7 @@ def policy_absent(name,
                                     keyid=keyid, profile=profile)
     if versions:
         for version in versions:
-            if version.get('is_default_version', False):
+            if version.get('is_default_version', False) in ('true', True):
                 continue
             r = __salt__['boto_iam.delete_policy_version'](name,
                                     version_id=version.get('version_id'),
@@ -1523,8 +1610,7 @@ def policy_absent(name,
 
 def saml_provider_present(name, saml_metadata_document, region=None, key=None, keyid=None, profile=None):
     '''
-
-    .. versionadded::
+    .. versionadded:: 2016.11.0
 
     Ensure the SAML provider with the specified name is present.
 
@@ -1581,8 +1667,7 @@ def saml_provider_present(name, saml_metadata_document, region=None, key=None, k
 
 def saml_provider_absent(name, region=None, key=None, keyid=None, profile=None):
     '''
-
-    .. versionadded::
+    .. versionadded:: 2016.11.0
 
     Ensure the SAML provider with the specified name is absent.
 
