@@ -335,7 +335,7 @@ def state(name,
         if top:
             cmd_kw['topfn'] = ''.join(cmd_kw.pop('arg'))
         elif sls:
-            cmd_kw['mods'] = cmd_kw.pop('arg')
+            cmd_kw['mods'] = ''.join(cmd_kw.pop('arg'))
         cmd_kw.update(cmd_kw.pop('kwarg'))
         tmp_ret = __salt__[fun](**cmd_kw)
         cmd_ret = {__opts__['id']: {
@@ -581,6 +581,9 @@ def function(
             m_ret = mdata['ret']
             m_func = (not fail_function and True) or __salt__[fail_function](m_ret)
 
+            if m_ret is False:
+                m_func = False
+
         if not m_func:
             if minion not in fail_minions:
                 fail.add(minion)
