@@ -26,7 +26,7 @@ The api key can be specified in the master or minion configuration like below:
 '''
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Salt libs
 from salt.exceptions import SaltInvocationError
