@@ -2,11 +2,11 @@
 '''
 Encapsulate the different transports available to Salt.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 
 # Import third party libs
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import range
 
 log = logging.getLogger(__name__)
@@ -57,8 +57,10 @@ class MessageClientPool(object):
     def __init__(self, tgt, opts, args=None, kwargs=None):
         sock_pool_size = opts['sock_pool_size'] if 'sock_pool_size' in opts else 1
         if sock_pool_size < 1:
-            log.warn('sock_pool_size is not correctly set, \
-                     the option should be greater than 0 but, {0}'.format(sock_pool_size))
+            log.warning(
+                'sock_pool_size is not correctly set, the option should be '
+                'greater than 0 but is instead %s', sock_pool_size
+            )
             sock_pool_size = 1
 
         if args is None:
