@@ -3,7 +3,7 @@
 Utilities to enable exception reraising across the master commands
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import python libs
 try:
@@ -50,5 +50,5 @@ def fire_exception(exc, opts, job=None, node='minion'):
     '''
     if job is None:
         job = {}
-    event = salt.utils.event.SaltEvent(node, opts=opts)
+    event = salt.utils.event.SaltEvent(node, opts=opts, listen=False)
     event.fire_event(pack_exception(exc), '_salt_error')
