@@ -15,7 +15,6 @@ import time
 # Import Salt Libs
 import salt.modules.cmdmod
 import salt.utils.args
-import salt.utils.decorators as decorators
 import salt.utils.files
 import salt.utils.path
 import salt.utils.platform
@@ -119,6 +118,18 @@ def _run_all(cmd):
     return ret
 
 
+def _check_launchctl_stderr(ret):
+    '''
+    helper class to check the launchctl stderr.
+    launchctl does not always return bad exit code
+    if there is a failure
+    '''
+    err = ret['stderr'].lower()
+    if 'service is disabled' in err:
+        return True
+    return False
+
+
 def execute_return_success(cmd):
     '''
     Executes the passed command. Returns True if successful
@@ -274,9 +285,10 @@ def launchctl(sub_cmd, *args, **kwargs):
     kwargs['python_shell'] = False
     kwargs = salt.utils.args.clean_kwargs(**kwargs)
     ret = __salt__['cmd.run_all'](cmd, **kwargs)
+    error = _check_launchctl_stderr(ret)
 
     # Raise an error or return successful result
-    if ret['retcode']:
+    if ret['retcode'] or error:
         out = 'Failed to {0} service:\n'.format(sub_cmd)
         out += 'stdout: {0}\n'.format(ret['stdout'])
         out += 'stderr: {0}\n'.format(ret['stderr'])
@@ -286,14 +298,18 @@ def launchctl(sub_cmd, *args, **kwargs):
         return ret['stdout'] if return_stdout else True
 
 
-def _available_services():
+def _available_services(refresh=False):
     '''
-    This is a helper function needed for testing. We are using the memoziation
-    decorator on the `available_services` function, which causes the function
-    to run once and then return the results of the first run on subsequent
-    calls. This causes problems when trying to test the functionality of the
-    `available_services` function.
+    This is a helper function for getting the available macOS services.
     '''
+    try:
+        if __context__['available_services'] and not refresh:
+            log.debug('Found context for available services.')
+            __context__['using_cached_services'] = True
+            return __context__['available_services']
+    except KeyError:
+        pass
+
     launchd_paths = [
         '/Library/LaunchAgents',
         '/Library/LaunchDaemons',
@@ -346,14 +362,22 @@ def _available_services():
                         'file_path': true_path,
                         'plist': plist}
 
-    return _available_services
+    # put this in __context__ as this is a time consuming function.
+    # a fix for this issue. https://github.com/saltstack/salt/issues/48414
+    __context__['available_services'] = _available_services
+    # this is a fresh gathering of services, set cached to false
+    __context__['using_cached_services'] = False
+
+    return __context__['available_services']
 
 
-@decorators.memoize
-def available_services():
+def available_services(refresh=False):
     '''
     Return a dictionary of all available services on the system
 
+    :param bool refresh: If you wish to refresh the available services
+    as this data is cached on the first run.
+
     Returns:
         dict: All available services
 
@@ -364,4 +388,5 @@ def available_services():
         import salt.utils.mac_service
         salt.utils.mac_service.available_services()
     '''
-    return _available_services()
+    log.debug('Loading available services')
+    return _available_services(refresh)
