@@ -4,7 +4,7 @@ Define some generic socket functions for network modules
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 import itertools
 import os
 import re
@@ -12,11 +12,12 @@ import types
 import socket
 import logging
 import platform
+import random
 import subprocess
 from string import ascii_letters, digits
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 # Attempt to import wmi
 try:
@@ -26,15 +27,31 @@ except ImportError:
     pass
 
 # Import salt libs
-import salt.utils
+import salt.utils.args
+import salt.utils.files
+import salt.utils.path
+import salt.utils.platform
+import salt.utils.stringutils
+import salt.utils.zeromq
 from salt._compat import ipaddress
+from salt.exceptions import SaltClientError, SaltSystemExit
+from salt.utils.decorators.jinja import jinja_filter
+from salt.utils.versions import LooseVersion
 
 # inet_pton does not exist in Windows, this is a workaround
-if salt.utils.is_windows():
+if salt.utils.platform.is_windows():
     from salt.ext import win_inet_pton  # pylint: disable=unused-import
 
 log = logging.getLogger(__name__)
 
+try:
+    import ctypes
+    import ctypes.util
+    libc = ctypes.cdll.LoadLibrary(ctypes.util.find_library("c"))
+    res_init = libc.__res_init
+except (ImportError, OSError, AttributeError, TypeError):
+    pass
+
 # pylint: disable=C0103
 
 
@@ -132,17 +149,21 @@ def _generate_minion_id():
                 addr=hosts.first() or 'localhost (N/A)', message=socket.gaierror)
             )
     # Universal method for everywhere (Linux, Slowlaris, Windows etc)
-    for f_name in ['/etc/hostname', '/etc/nodename', '/etc/hosts',
-                   r'{win}\system32\drivers\etc\hosts'.format(win=os.getenv('WINDIR'))]:
-        if not os.path.exists(f_name):
-            continue
-        with salt.utils.fopen(f_name) as f_hdl:
-            for hst in (line.strip().split('#')[0].strip().split() or None for line in f_hdl.read().split(os.linesep)):
-                if hst and (hst[0][:4] in ['127.', '::1'] or len(hst) == 1):
-                    hosts.extend(hst)
+    for f_name in ('/etc/hostname', '/etc/nodename', '/etc/hosts',
+                   r'{win}\system32\drivers\etc\hosts'.format(win=os.getenv('WINDIR'))):
+        try:
+            with salt.utils.files.fopen(f_name) as f_hdl:
+                for line in f_hdl:
+                    line = salt.utils.stringutils.to_unicode(line)
+                    hst = line.strip().split('#')[0].strip().split()
+                    if hst:
+                        if hst[0][:4] in ('127.', '::1') or len(hst) == 1:
+                            hosts.extend(hst)
+        except IOError:
+            pass
 
     # include public and private ipaddresses
-    return hosts.extend([addr for addr in salt.utils.network.ip_addrs()
+    return hosts.extend([addr for addr in ip_addrs()
                          if not ipaddress.ip_address(addr).is_loopback])
 
 
@@ -152,7 +173,11 @@ def generate_minion_id():
 
     :return:
     '''
-    return _generate_minion_id().first() or 'localhost'
+    try:
+        ret = salt.utils.stringutils.to_unicode(_generate_minion_id().first())
+    except TypeError:
+        ret = None
+    return ret or 'localhost'
 
 
 def get_socket(addr, type=socket.SOCK_STREAM, proto=0):
@@ -198,13 +223,29 @@ def ip_to_host(ip):
     '''
     try:
         hostname, aliaslist, ipaddrlist = socket.gethostbyaddr(ip)
-    except Exception:
+    except Exception as exc:
+        log.debug('salt.utils.network.ip_to_host(%r) failed: %s', ip, exc)
         hostname = None
     return hostname
 
 # pylint: enable=C0103
 
 
+def is_reachable_host(entity_name):
+    '''
+    Returns a bool telling if the entity name is a reachable host (IPv4/IPv6/FQDN/etc).
+    :param hostname:
+    :return:
+    '''
+    try:
+        assert type(socket.getaddrinfo(entity_name, 0, 0, 0, 0)) == list
+        ret = True
+    except socket.gaierror:
+        ret = False
+
+    return ret
+
+
 def is_ip(ip):
     '''
     Returns a bool telling if the passed IP is a valid IPv4 or IPv6 address.
@@ -232,6 +273,34 @@ def is_ipv6(ip):
         return False
 
 
+def is_subnet(cidr):
+    '''
+    Returns a bool telling if the passed string is an IPv4 or IPv6 subnet
+    '''
+    return is_ipv4_subnet(cidr) or is_ipv6_subnet(cidr)
+
+
+def is_ipv4_subnet(cidr):
+    '''
+    Returns a bool telling if the passed string is an IPv4 subnet
+    '''
+    try:
+        return '/' in cidr and bool(ipaddress.IPv4Network(cidr))
+    except Exception:
+        return False
+
+
+def is_ipv6_subnet(cidr):
+    '''
+    Returns a bool telling if the passed string is an IPv6 subnet
+    '''
+    try:
+        return '/' in cidr and bool(ipaddress.IPv6Network(cidr))
+    except Exception:
+        return False
+
+
+@jinja_filter('is_ip')
 def is_ip_filter(ip, options=None):
     '''
     Returns a bool telling if the passed IP is a valid IPv4 or IPv6 address.
@@ -295,7 +364,7 @@ def _ip_options(ip_obj, version, options=None):
     }
 
     if not options:
-        return str(ip_obj)  # IP version already checked
+        return six.text_type(ip_obj)  # IP version already checked
 
     options_list = [option.strip() for option in options.split(',')]
 
@@ -306,7 +375,7 @@ def _ip_options(ip_obj, version, options=None):
                 return None
                 # stop at first failed test
             # else continue
-    return str(ip_obj)
+    return six.text_type(ip_obj)
 
 
 def _is_ipv(ip, version, options=None):
@@ -334,6 +403,7 @@ def _is_ipv(ip, version, options=None):
     return _ip_options(ip_obj, version, options=options)
 
 
+@jinja_filter('is_ipv4')
 def is_ipv4_filter(ip, options=None):
     '''
     Returns a bool telling if the value passed to it was a valid IPv4 address.
@@ -351,6 +421,7 @@ def is_ipv4_filter(ip, options=None):
     return isinstance(_is_ipv4, six.string_types)
 
 
+@jinja_filter('is_ipv6')
 def is_ipv6_filter(ip, options=None):
     '''
     Returns a bool telling if the value passed to it was a valid IPv6 address.
@@ -386,6 +457,7 @@ def _ipv_filter(value, version, options=None):
     return None
 
 
+@jinja_filter('ipv4')
 def ipv4(value, options=None):
     '''
     Filters a list and returns IPv4 values only.
@@ -393,6 +465,7 @@ def ipv4(value, options=None):
     return _ipv_filter(value, 4, options=options)
 
 
+@jinja_filter('ipv6')
 def ipv6(value, options=None):
     '''
     Filters a list and returns IPv6 values only.
@@ -400,6 +473,7 @@ def ipv6(value, options=None):
     return _ipv_filter(value, 6, options=options)
 
 
+@jinja_filter('ipaddr')
 def ipaddr(value, options=None):
     '''
     Filters and returns only valid IP objects.
@@ -431,6 +505,7 @@ def _filter_ipaddr(value, options, version=None):
     return ipaddr_filter_out
 
 
+@jinja_filter('ip_host')
 def ip_host(value, options=None, version=None):
     '''
     Returns the interfaces IP address, e.g.: 192.168.0.1/28.
@@ -439,20 +514,26 @@ def ip_host(value, options=None, version=None):
     if not ipaddr_filter_out:
         return
     if not isinstance(value, (list, tuple, types.GeneratorType)):
-        return str(ipaddress.ip_interface(ipaddr_filter_out[0]))
-    return [str(ipaddress.ip_interface(ip_a)) for ip_a in ipaddr_filter_out]
+        return six.text_type(ipaddress.ip_interface(ipaddr_filter_out[0]))
+    return [six.text_type(ipaddress.ip_interface(ip_a)) for ip_a in ipaddr_filter_out]
 
 
 def _network_hosts(ip_addr_entry):
     return [
-        str(host)
+        six.text_type(host)
         for host in ipaddress.ip_network(ip_addr_entry, strict=False).hosts()
     ]
 
 
+@jinja_filter('network_hosts')
 def network_hosts(value, options=None, version=None):
     '''
     Return the list of hosts within a network.
+
+    .. note::
+
+        When running this command with a large IPv6 network, the command will
+        take a long time to gather all of the hosts.
     '''
     ipaddr_filter_out = _filter_ipaddr(value, options=options, version=version)
     if not ipaddr_filter_out:
@@ -469,6 +550,7 @@ def _network_size(ip_addr_entry):
     return ipaddress.ip_network(ip_addr_entry, strict=False).num_addresses
 
 
+@jinja_filter('network_size')
 def network_size(value, options=None, version=None):
     '''
     Get the size of a network.
@@ -665,7 +747,7 @@ def _interfaces_ifconfig(out):
 
     piface = re.compile(r'^([^\s:]+)')
     pmac = re.compile('.*?(?:HWaddr|ether|address:|lladdr) ([0-9a-fA-F:]+)')
-    if salt.utils.is_sunos():
+    if salt.utils.platform.is_sunos():
         pip = re.compile(r'.*?(?:inet\s+)([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)(.*)')
         pip6 = re.compile('.*?(?:inet6 )([0-9a-fA-F:]+)')
         pmask6 = re.compile(r'.*?(?:inet6 [0-9a-fA-F:]+/(\d+)).*')
@@ -692,7 +774,7 @@ def _interfaces_ifconfig(out):
                 iface = miface.group(1)
             if mmac:
                 data['hwaddr'] = mmac.group(1)
-                if salt.utils.is_sunos():
+                if salt.utils.platform.is_sunos():
                     expand_mac = []
                     for chunk in data['hwaddr'].split(':'):
                         expand_mac.append('0{0}'.format(chunk) if len(chunk) < 2 else '{0}'.format(chunk))
@@ -724,11 +806,11 @@ def _interfaces_ifconfig(out):
                 mmask6 = pmask6.match(line)
                 if mmask6:
                     addr_obj['prefixlen'] = mmask6.group(1) or mmask6.group(2)
-                    if not salt.utils.is_sunos():
+                    if not salt.utils.platform.is_sunos():
                         ipv6scope = mmask6.group(3) or mmask6.group(4)
                         addr_obj['scope'] = ipv6scope.lower() if ipv6scope is not None else ipv6scope
                 # SunOS sometimes has ::/0 as inet6 addr when using addrconf
-                if not salt.utils.is_sunos() \
+                if not salt.utils.platform.is_sunos() \
                         or addr_obj['address'] != '::' \
                         and addr_obj['prefixlen'] != 0:
                     data['inet6'].append(addr_obj)
@@ -757,8 +839,8 @@ def linux_interfaces():
     Obtain interface information for *NIX/BSD variants
     '''
     ifaces = dict()
-    ip_path = salt.utils.which('ip')
-    ifconfig_path = None if ip_path else salt.utils.which('ifconfig')
+    ip_path = salt.utils.path.which('ip')
+    ifconfig_path = None if ip_path else salt.utils.path.which('ifconfig')
     if ip_path:
         cmd1 = subprocess.Popen(
             '{0} link show'.format(ip_path),
@@ -773,18 +855,97 @@ def linux_interfaces():
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT).communicate()[0]
         ifaces = _interfaces_ip("{0}\n{1}".format(
-            salt.utils.to_str(cmd1),
-            salt.utils.to_str(cmd2)))
+            salt.utils.stringutils.to_str(cmd1),
+            salt.utils.stringutils.to_str(cmd2)))
     elif ifconfig_path:
         cmd = subprocess.Popen(
             '{0} -a'.format(ifconfig_path),
             shell=True,
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT).communicate()[0]
-        ifaces = _interfaces_ifconfig(salt.utils.to_str(cmd))
+        ifaces = _interfaces_ifconfig(salt.utils.stringutils.to_str(cmd))
     return ifaces
 
 
+def _netbsd_interfaces_ifconfig(out):
+    '''
+    Uses ifconfig to return a dictionary of interfaces with various information
+    about each (up/down state, ip address, netmask, and hwaddr)
+    '''
+    ret = dict()
+
+    piface = re.compile(r'^([^\s:]+)')
+    pmac = re.compile('.*?address: ([0-9a-f:]+)')
+
+    pip = re.compile(r'.*?inet [^\d]*(.*?)/([\d]*)\s')
+    pip6 = re.compile(r'.*?inet6 ([0-9a-f:]+)%([a-zA-Z0-9]*)/([\d]*)\s')
+
+    pupdown = re.compile('UP')
+    pbcast = re.compile(r'.*?broadcast ([\d\.]+)')
+
+    groups = re.compile('\r?\n(?=\\S)').split(out)
+    for group in groups:
+        data = dict()
+        iface = ''
+        updown = False
+        for line in group.splitlines():
+            miface = piface.match(line)
+            mmac = pmac.match(line)
+            mip = pip.match(line)
+            mip6 = pip6.match(line)
+            mupdown = pupdown.search(line)
+            if miface:
+                iface = miface.group(1)
+            if mmac:
+                data['hwaddr'] = mmac.group(1)
+            if mip:
+                if 'inet' not in data:
+                    data['inet'] = list()
+                addr_obj = dict()
+                addr_obj['address'] = mip.group(1)
+                mmask = mip.group(2)
+                if mip.group(2):
+                    addr_obj['netmask'] = cidr_to_ipv4_netmask(mip.group(2))
+                mbcast = pbcast.match(line)
+                if mbcast:
+                    addr_obj['broadcast'] = mbcast.group(1)
+                data['inet'].append(addr_obj)
+            if mupdown:
+                updown = True
+            if mip6:
+                if 'inet6' not in data:
+                    data['inet6'] = list()
+                addr_obj = dict()
+                addr_obj['address'] = mip6.group(1)
+                mmask6 = mip6.group(3)
+                addr_obj['scope'] = mip6.group(2)
+                addr_obj['prefixlen'] = mip6.group(3)
+                data['inet6'].append(addr_obj)
+        data['up'] = updown
+        ret[iface] = data
+        del data
+    return ret
+
+
+def netbsd_interfaces():
+    '''
+    Obtain interface information for NetBSD >= 8 where the ifconfig
+    output diverged from other BSD variants (Netmask is now part of the
+    address)
+    '''
+    # NetBSD versions prior to 8.0 can still use linux_interfaces()
+    if LooseVersion(os.uname()[2]) < LooseVersion('8.0'):
+        return linux_interfaces()
+
+    ifconfig_path = salt.utils.path.which('ifconfig')
+    cmd = subprocess.Popen(
+        '{0} -a'.format(ifconfig_path),
+        shell=True,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.STDOUT).communicate()[0]
+    return _netbsd_interfaces_ifconfig(salt.utils.stringutils.to_str(cmd))
+
+
 def _interfaces_ipconfig(out):
     '''
     Returns a dictionary of interfaces with various information about each
@@ -884,8 +1045,10 @@ def interfaces():
     '''
     Return a dictionary of information about all the interfaces on the minion
     '''
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return win_interfaces()
+    elif salt.utils.platform.is_netbsd():
+        return netbsd_interfaces()
     else:
         return linux_interfaces()
 
@@ -895,7 +1058,7 @@ def get_net_start(ipaddr, netmask):
     Return the address of the network
     '''
     net = ipaddress.ip_network('{0}/{1}'.format(ipaddr, netmask), strict=False)
-    return str(net.network_address)
+    return six.text_type(net.network_address)
 
 
 def get_net_size(mask):
@@ -918,7 +1081,7 @@ def calc_net(ipaddr, netmask=None):
     if netmask is not None:
         ipaddr = '{0}/{1}'.format(ipaddr, netmask)
 
-    return str(ipaddress.ip_network(ipaddr, strict=False))
+    return six.text_type(ipaddress.ip_network(ipaddr, strict=False))
 
 
 def _ipv4_to_bits(ipaddr):
@@ -975,7 +1138,7 @@ def hw_addr(iface):
         Added support for AIX
 
     '''
-    if salt.utils.is_aix():
+    if salt.utils.platform.is_aix():
         return _hw_addr_aix
 
     iface_info, error = _get_iface_info(iface)
@@ -1005,7 +1168,8 @@ def interface_ip(iface):
     iface_info, error = _get_iface_info(iface)
 
     if error is False:
-        return iface_info.get(iface, {}).get('inet', {})[0].get('address', '')
+        inet = iface_info.get(iface, {}).get('inet', None)
+        return inet[0].get('address', '') if inet else ''
     else:
         return error
 
@@ -1047,7 +1211,7 @@ def _subnets(proto='inet', interfaces_=None):
                 intf = ipaddress.ip_interface('{0}/{1}'.format(intf['address'], dflt_cidr))
             if not intf.is_loopback:
                 ret.add(intf.network)
-    return [str(net) for net in sorted(ret)]
+    return [six.text_type(net) for net in sorted(ret)]
 
 
 def subnets(interfaces=None):
@@ -1112,7 +1276,7 @@ def _ip_addrs(interface=None, include_loopback=False, interface_data=None, proto
             addr = ipaddress.ip_address(addr.get('address'))
             if not addr.is_loopback or include_loopback:
                 ret.add(addr)
-    return [str(addr) for addr in sorted(ret)]
+    return [six.text_type(addr) for addr in sorted(ret)]
 
 
 def ip_addrs(interface=None, include_loopback=False, interface_data=None):
@@ -1196,8 +1360,9 @@ def active_tcp():
     ret = {}
     for statf in ['/proc/net/tcp', '/proc/net/tcp6']:
         if os.path.isfile(statf):
-            with salt.utils.fopen(statf, 'rb') as fp_:
+            with salt.utils.files.fopen(statf, 'rb') as fp_:
                 for line in fp_:
+                    line = salt.utils.stringutils.to_unicode(line)
                     if line.strip().startswith('sl'):
                         continue
                     ret.update(_parse_tcp_line(line))
@@ -1231,8 +1396,9 @@ def _remotes_on(port, which_end):
     for statf in ['/proc/net/tcp', '/proc/net/tcp6']:
         if os.path.isfile(statf):
             proc_available = True
-            with salt.utils.fopen(statf, 'r') as fp_:
+            with salt.utils.files.fopen(statf, 'r') as fp_:
                 for line in fp_:
+                    line = salt.utils.stringutils.to_unicode(line)
                     if line.strip().startswith('sl'):
                         continue
                     iret = _parse_tcp_line(line)
@@ -1241,17 +1407,17 @@ def _remotes_on(port, which_end):
                         ret.add(iret[sl]['remote_addr'])
 
     if not proc_available:  # Fallback to use OS specific tools
-        if salt.utils.is_sunos():
+        if salt.utils.platform.is_sunos():
             return _sunos_remotes_on(port, which_end)
-        if salt.utils.is_freebsd():
+        if salt.utils.platform.is_freebsd():
             return _freebsd_remotes_on(port, which_end)
-        if salt.utils.is_netbsd():
+        if salt.utils.platform.is_netbsd():
             return _netbsd_remotes_on(port, which_end)
-        if salt.utils.is_openbsd():
+        if salt.utils.platform.is_openbsd():
             return _openbsd_remotes_on(port, which_end)
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             return _windows_remotes_on(port, which_end)
-        if salt.utils.is_aix():
+        if salt.utils.platform.is_aix():
             return _aix_remotes_on(port, which_end)
 
         return _linux_remotes_on(port, which_end)
@@ -1298,7 +1464,7 @@ def _sunos_remotes_on(port, which_end):
         log.error('Failed netstat')
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
     for line in lines:
         if 'ESTABLISHED' not in line:
             continue
@@ -1338,13 +1504,13 @@ def _freebsd_remotes_on(port, which_end):
     remotes = set()
 
     try:
-        cmd = salt.utils.shlex_split('sockstat -4 -c -p {0}'.format(port))
+        cmd = salt.utils.args.shlex_split('sockstat -4 -c -p {0}'.format(port))
         data = subprocess.check_output(cmd)  # pylint: disable=minimum-python-version
     except subprocess.CalledProcessError as ex:
         log.error('Failed "sockstat" with returncode = {0}'.format(ex.returncode))
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
 
     for line in lines:
         chunks = line.split()
@@ -1398,13 +1564,13 @@ def _netbsd_remotes_on(port, which_end):
     remotes = set()
 
     try:
-        cmd = salt.utils.shlex_split('sockstat -4 -c -n -p {0}'.format(port))
+        cmd = salt.utils.args.shlex_split('sockstat -4 -c -n -p {0}'.format(port))
         data = subprocess.check_output(cmd)  # pylint: disable=minimum-python-version
     except subprocess.CalledProcessError as ex:
         log.error('Failed "sockstat" with returncode = {0}'.format(ex.returncode))
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
 
     for line in lines:
         chunks = line.split()
@@ -1493,7 +1659,7 @@ def _windows_remotes_on(port, which_end):
         log.error('Failed netstat')
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
     for line in lines:
         if 'ESTABLISHED' not in line:
             continue
@@ -1540,7 +1706,7 @@ def _linux_remotes_on(port, which_end):
         log.error('Failed "lsof" with returncode = {0}'.format(ex.returncode))
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
     for line in lines:
         chunks = line.split()
         if not chunks:
@@ -1599,7 +1765,7 @@ def _aix_remotes_on(port, which_end):
         log.error('Failed netstat')
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
     for line in lines:
         if 'ESTABLISHED' not in line:
             continue
@@ -1613,3 +1779,130 @@ def _aix_remotes_on(port, which_end):
             continue
         remotes.add(remote_host)
     return remotes
+
+
+@jinja_filter('gen_mac')
+def gen_mac(prefix='AC:DE:48'):
+    '''
+    Generates a MAC address with the defined OUI prefix.
+
+    Common prefixes:
+
+     - ``00:16:3E`` -- Xen
+     - ``00:18:51`` -- OpenVZ
+     - ``00:50:56`` -- VMware (manually generated)
+     - ``52:54:00`` -- QEMU/KVM
+     - ``AC:DE:48`` -- PRIVATE
+
+    References:
+
+     - http://standards.ieee.org/develop/regauth/oui/oui.txt
+     - https://www.wireshark.org/tools/oui-lookup.html
+     - https://en.wikipedia.org/wiki/MAC_address
+    '''
+    return '{0}:{1:02X}:{2:02X}:{3:02X}'.format(prefix,
+                                                random.randint(0, 0xff),
+                                                random.randint(0, 0xff),
+                                                random.randint(0, 0xff))
+
+
+@jinja_filter('mac_str_to_bytes')
+def mac_str_to_bytes(mac_str):
+    '''
+    Convert a MAC address string into bytes. Works with or without separators:
+
+    b1 = mac_str_to_bytes('08:00:27:13:69:77')
+    b2 = mac_str_to_bytes('080027136977')
+    assert b1 == b2
+    assert isinstance(b1, bytes)
+    '''
+    if len(mac_str) == 12:
+        pass
+    elif len(mac_str) == 17:
+        sep = mac_str[2]
+        mac_str = mac_str.replace(sep, '')
+    else:
+        raise ValueError('Invalid MAC address')
+    chars = (int(mac_str[s:s+2], 16) for s in range(0, 12, 2))
+    return bytes(chars) if six.PY3 else b''.join(chr(x) for x in chars)
+
+
+def refresh_dns():
+    '''
+    issue #21397: force glibc to re-read resolv.conf
+    '''
+    try:
+        res_init()
+    except NameError:
+        # Exception raised loading the library, thus res_init is not defined
+        pass
+
+
+@jinja_filter('dns_check')
+def dns_check(addr, port, safe=False, ipv6=None):
+    '''
+    Return the ip resolved by dns, but do not exit on failure, only raise an
+    exception. Obeys system preference for IPv4/6 address resolution.
+    Tries to connect to the address before considering it useful. If no address
+    can be reached, the first one resolved is used as a fallback.
+    '''
+    error = False
+    lookup = addr
+    seen_ipv6 = False
+    try:
+        refresh_dns()
+        hostnames = socket.getaddrinfo(
+            addr, None, socket.AF_UNSPEC, socket.SOCK_STREAM
+        )
+        if not hostnames:
+            error = True
+        else:
+            resolved = False
+            candidates = []
+            for h in hostnames:
+                # It's an IP address, just return it
+                if h[4][0] == addr:
+                    resolved = addr
+                    break
+
+                if h[0] == socket.AF_INET and ipv6 is True:
+                    continue
+                if h[0] == socket.AF_INET6 and ipv6 is False:
+                    continue
+
+                candidate_addr = salt.utils.zeromq.ip_bracket(h[4][0])
+
+                if h[0] != socket.AF_INET6 or ipv6 is not None:
+                    candidates.append(candidate_addr)
+
+                try:
+                    s = socket.socket(h[0], socket.SOCK_STREAM)
+                    s.connect((candidate_addr.strip('[]'), port))
+                    s.close()
+
+                    resolved = candidate_addr
+                    break
+                except socket.error:
+                    pass
+            if not resolved:
+                if len(candidates) > 0:
+                    resolved = candidates[0]
+                else:
+                    error = True
+    except TypeError:
+        err = ('Attempt to resolve address \'{0}\' failed. Invalid or unresolveable address').format(lookup)
+        raise SaltSystemExit(code=42, msg=err)
+    except socket.error:
+        error = True
+
+    if error:
+        err = ('DNS lookup or connection check of \'{0}\' failed.').format(addr)
+        if safe:
+            if salt.log.is_console_configured():
+                # If logging is not configured it also means that either
+                # the master or minion instance calling this hasn't even
+                # started running
+                log.error(err)
+            raise SaltClientError()
+        raise SaltSystemExit(code=42, msg=err)
+    return resolved
