@@ -13,6 +13,7 @@ import tarfile
 import zipfile
 import tempfile
 import subprocess
+import concurrent
 
 # Import third party libs
 import jinja2
@@ -110,6 +111,8 @@ def get_tops(extra_mods='', so_mods=''):
             os.path.dirname(msgpack.__file__),
             ]
 
+    if _six.PY2:
+        tops.append(os.path.dirname(concurrent.__file__))
     tops.append(_six.__file__.replace('.pyc', '.py'))
     tops.append(backports_abc.__file__.replace('.pyc', '.py'))
 
@@ -464,6 +467,7 @@ def gen_min(cachedir, extra_mods='', overwrite=False, so_mods='',
         'salt/__init__.py',
         'salt/utils',
         'salt/utils/__init__.py',
+        'salt/utils/atomicfile.py',
         'salt/utils/validate',
         'salt/utils/validate/__init__.py',
         'salt/utils/validate/path.py',
@@ -491,7 +495,7 @@ def gen_min(cachedir, extra_mods='', overwrite=False, so_mods='',
         'salt/utils/openstack',
         'salt/utils/openstack/__init__.py',
         'salt/utils/openstack/swift.py',
-        'salt/utils/async.py',
+        'salt/utils/asynchronous.py',
         'salt/utils/process.py',
         'salt/utils/jinja.py',
         'salt/utils/rsax931.py',
